:
## BPcleanup.sh
## This script cleans up any extra or temporary files
##  Cleans BP temp files in system TMP directories
## 	Cleans logs.dir 
##  Cleans list.dir
##  Detects failed or aborted database updates and restarts them
##
## Edit these LIST variables to add more file patterns to remove

# Determine where BP is installed
eval `grep -v "^[;#]" /etc/default/bp.ini | grep '=[^	]'` > /dev/null 2>&1

# Set basic environment variables
eval `${BPDIR}/bin/bputil -e -c`

LOGDIR_SAVE="*.sav"
LOGDIR_LIST="*.stderr tcmd_* AAA* aAAA* .AAA*"
LISTDIR_LIST="*.inc"
SYS=`uname`
if [ "$SYS" = "Linux" ]; then
	TMPDIR_LIST="juke_* unloa* profi* BPcle* DBupd* fslbl* tplbl*"
else
	TMPDIR_LIST="juke_[A-Z]* unloa[A-Z]* profi[A-Z]* BPcle[A-Z]* DBupd[A-Z]* fslbl[A-Z]* tplbl[A-Z]*"
fi

Done() {
	${BP_BINDIR}/bputil -j clear
	exit $1
}

CleanIt() {
	if [ $# != 3 ]; then
		echo "Nothing to clean - improper number of arguments"
		return 1
	fi
	if [ "$3" = "0" ]; then
		rm -f $1/$2 2>/dev/null                  # Remove all if 0
	else
		for n in `find $1/$2 -mtime +$3 -print 2>/dev/null`
		do
			echo "rm -f $n"
			rm -f $n                               # Remove it
		done
	fi
}

######################
## TMP DIR SECTION  ##
######################

${BP_BINDIR}/bputil -j 0 "Cleaning /tmp..."
for P in $TMPDIR_LIST
do
		CleanIt "/tmp" "${P}" 1
    CleanIt "/usr/tmp" "${P}" 1
    CleanIt "/var/tmp" "${P}" 1
done
	
#######################
## LOG DIR SECTION   ##
#######################

if [ -d "${BP_LOGDIR}" ];then                     # Usually /usr/bp/logs.dir
	cd $BP_LOGDIR || Done 1                         # Abort if we cannot cd
	${BP_BINDIR}/bputil -j 0 "Cleaning logs now..."
	for P in $LOGDIR_LIST
	do
		CleanIt $BP_LOGDIR "${P}"  4
	done
	for P in $LOGDIR_SAVE
	do
		CleanIt $BP_LOGDIR "${P}"  60
	done
fi

#######################
## LIST DIR SECTION  ##
#######################

if [ -d "${BP_LISTDIR}" ];then                      # Usually /usr/bp/lists.dir
	cd $BP_LISTDIR || Done 1
	${BP_BINDIR}/bputil -j 0 "Cleaning up old list files now..."
	for P in $LISTDIR_LIST
	do
		CleanIt $BP_LISTDIR "${P}" 4
	done

	# WARN - Must detect a backup that is in progress but not complete yet
	# Find aborted backups and update the database with what files actually
	# made it to the tape before the backup aborted.
	for n in *.file
	do
		[ -f $n ] || continue  # In case there are no matches
		# Below is backward compatible with 1.7 also
		BACKUP_NO=`echo ${n} | cut -d_ -f2 | cut -d. -f1`
		FILESET_NO=`echo ${n} | cut -d_ -f3 | cut -d. -f1 | sed 's/fs//'`
		${BP_BINDIR}/bputil -j 0 "Updating database with residual file information from Backup ${BACKUP_NO} ..."
		# This loads into database all the file information for the backup
		# It first deletes the database version, then rebuilds it from scratch
		# using the contents of this file
		# When update_db is complete, file is automatically removed
		echo "${BP_BINDIR}/updatedb -b ${BACKUP_NO} -f ${FILESET_NO} -a remove &"
		${BP_BINDIR}/updatedb -b ${BACKUP_NO} -f ${FILESET_NO} -a remove &
                sleep 1
	done
fi
Done 0
