#!/bin/sh
#
# restore_iseries.sh
#
# ME	4623	040608	Interactive script to restore data to iSeries client.
# ME	5099	051908	Fix typo in message.
# ME	5179	052708	retrieve retention days from lmmgr instead of unitrends-retenion file.
# ME	5239	053008	remove dependency on profiles. broken by on-demand backups
# ME	5239	060208	left one check for profile in the select client routine.
#

. /etc/default/bp.ini 2> /dev/null
if [ "$BPDIR" = "" ]
then
	export BPDIR="/usr/bp"
fi
BPBIN=$BPDIR/bin
BPRETENTION="/etc/unitrends-retention"
BPLOG=$BPDIR/logs.dir
iSERIESLOG=$BPLOG/iSeries_restore$$.log
BPPRO=$BPDIR/profiles.dir
i5PRO=$BPPRO/iSeries
mkdir -p $i5PRO

ECHO="echo -e"


#########################################################################
#
# cleanup
#
# End state routine to clean up any temporary files and exit.
#
#########################################################################
cleanup()
{
	Status=$1
	# keep the number of iSeries restore logs from growing exponetially
	NumiSeriesLogs=`ls $BPLOG/iSeries_restore* 2> /dev/null | wc -l`
	while [ $NumiSeriesLogs -gt 20 ]
	do
		ls -tr $BPLOG/iSeries_restore* > /tmp/blogs$$ 2> /dev/null
		thisLog=`head -1 /tmp/blogs$$`
		rm -f $thisLog 2> /dev/null
		NumiSeriesLogs=`ls $BPLOG/iSeries_restore* 2> /dev/null | wc -l`
	done
	rm -f $iSERIESLOCK
	rm -f /$i5PRO/$$_${iSeriesClient}.excl
	rm -f /$i5PRO/$$_${iSeriesClient}.incl
	stty sane
	exit $Status
}

#########################################################################
#
# GetiSeriesClientName
#
# Get the name of the iSeries client (must already be setup on network.
#
#########################################################################
GetiSeriesClientName()
{
	clear

	clientList=""
	if [ "$1" == "local" ]
	then
		clientList=`$BPBIN/iSeries/iSeries_util.php -l | grep -v -e '->'`
	else
		clientList=`$BPBIN/iSeries/iSeries_util.php -l`
	fi

	$ECHO "Please select the client you would like to restore:"
	> /tmp/clientlist%%
	listItem=1
	for client in $clientList
	do
		$ECHO "$listItem.  $client" 
		$ECHO "$client" >> /tmp/clientlist$$
		listItem=`expr $listItem + 1`
	done

	if [ "$listItem" == "1" ]
	then
		$ECHO "There are no registered clients to restore."
		$ECHO "Press <Enter> to quit."
		read
		exit 1
	fi

	${ECHO} "Enter selection for client: \c" 
	read ANS
	case ${ANS} in
	[0-9]*)
		# 5239 remove code checking for profile
		SelectedClient=`cat /tmp/clientlist$$ |  awk -vthisfile="$ANS" '{if(NR == thisfile) print $1}'`
		;;
	*)
		$ECHO "Invalid Selection. Please try again."
		sleep 2
		GetiSeriesClientName $*
		;;
	esac
	iSeriesClient=$SelectedClient
	$ECHO "You have selected client $iSeriesClient."
	return
}
	
#########################################################################
#
# GetUserNameAndPassword
#
# Get the name of the iSeries client (must already be setup on network.
# Store the username in the profile, but hide the password in a
# "dotted" file saved in the profiles directory using the name of the
# client_IPW for the filename.
#
# Restore will have to know about this naming converntion.
# This makes it a little harder to find the password since we are
# being a but unsecure in storing a password.
#
#########################################################################
GetUserNameAndPassword()
{
	$ECHO ""
	$ECHO "In order to connect to the iSeries server, you must supply an"
	$ECHO "authorized username and password for this connection."
	$ECHO "Please enter the username you wish to use to connect: \c"
	read iSeriesUsername

	if [ "$iSeriesUsername" = "" ]
	then
		clear
		$ECHO "You must enter a valid username. Please try again."
		GetUserNameAndPassword
	fi

	$ECHO "Please enter the password for the supplied username: \c"
	stty -echo
	read iSeriesPassword
	stty echo
	$ECHO 

	if [ "$iSeriesPassword" = "" ]
	then
		clear
		$ECHO "You must enter a valid password. Please try again."
		GetUserNameAndPassword
	fi

	echo "$iSeriesPassword" > $i5PRO/.restore_IPW

	return
}

#########################################################################
#
# TestiSeriesConnection
#
# backupFTP offers a --test option to allow us to validate the client
# connectitiy settings we have been supplied during this configuration.
#
#########################################################################
TestiSeriesConnection()
{
	$ECHO "Testing client credentials"
	export IPW=`cat $i5PRO/.restore_IPW`
	$ECHO "`date`: Testing restore configuration for $iSeriesClient with the following command:" >> $iSERIESLOG
	$ECHO "$BPBIN/backupFTP --test --server $iSeriesClient --credentials ${iSeriesUsername}:ENV:IPW" >> $iSERIESLOG
	$BPBIN/backupFTP --test --server "$iSeriesClient" --credentials ${iSeriesUsername}:ENV:IPW >> $iSERIESLOG 2>&1
	RC=$?
	if [ $RC -ne 0 ]
	then
		$ECHO "`date`: The test connection to $iSeriesClient FAILED with return code $RC." >> $iSERIESLOG
		$ECHO "The test connection to $iSeriesClient FAILED with return code $RC."
		$ECHO "Please verify the client name and user/password  credentials you"
		$ECHO "supplied are correct and rerun this configuration utility."
		# 4919
		$ECHO "Press <Enter> to return: \c"
		read
		rm $i5PRO/.restore_IPW
		exit 2
	else
		$ECHO "`date`: The test connection to $iSeriesClient SUCCEEDED." >> $iSERIESLOG
		$ECHO "The test connection to $iSeriesClient SUCCEEDED."
	fi
	$ECHO ""
}

#########################################################################
#
# GetRestoreType
#
# Ask for Full or Selective Restore.
#
#########################################################################
GetRestoreType()
{
	clear
	$ECHO "You may choose to restore all files from this backup or"
	$ECHO "you may perform a selective restore of only certain files."
	${ECHO} "1.  Full Restore"
	${ECHO} "2.  Selective Restore"
	${ECHO} "Enter selection: \c"
	read pick
	case ${pick} in
	1|f|F|"Full"|"full")
		restoreType="full"
                $ECHO "`date`: User request Full restore for $iSeriesClient:" >> $iSERIESLOG
	;;
	2|s|S|"selective"|"Selective")
		restoreType="selective"
                $ECHO "`date`: User request Selective restore for $iSeriesClient:" >> $iSERIESLOG
		GetIncludeList
		GetExcludeList
	;;
	*)
		$ECHO "You have entered an invalid selection."
		$ECHO "Please try again."
		sleep 2
		GetRestoreType
	esac

	return
}

#########################################################################
#
# profile_read
#
# Read the profile entry requested and echo it for assignment in calling
# routine
#
#########################################################################
profile_read()
{
	Section="$1"
	Keyword="$2"
	$BPBIN/bputil -g "$Section" "$Keyword" "xxx" $iSERIESPROFILE
}

#########################################################################
#
# GetFiles
#
# Ask for file names one at a time until they enter a 'q'
#
#########################################################################
GetFiles()
{
	ListFile=$1
	filename=""
	new_name=""
	while [ "$filename" != "q" ]
	do
		$ECHO "Original Filename: \c"
		read filename
		
		if [ "$filename" != "" -a "$filename" != "q" ]
		then
			if [ "${filename:0:1}" != "/" ]
			then
				filename=/$filename
			fi

			if [ $2 == 1 ]
			then
				$ECHO "Target Filename  : \c"
				read new_name
			fi

			if [ "$new_name" == "" -o "$new_name" == "q" ]
			then
				if [ "${new_name:0:1}" != "/" ]
				then
					new_name=/$new_name
				fi

				$ECHO "$filename" >> $ListFile
			else
				$ECHO "$filename :: $new_name" >> $ListFile
			fi
		fi

		if [ "$new_name" == "q" ]
		then
			filename="q"
		fi

		new_name=""
	done
	return
}

#########################################################################
#
# GetExcludeList
#
# Ask for Full Pathnames to Exclude until they type "q"
#
#########################################################################
GetExcludeList()
{
	clear
	$ECHO "You may enter a list of files (full pathname) to exclude"
	$ECHO "from your restore if you wish. Please specify one filename"
	$ECHO "per line. When you have completed your exclude list, please"
	$ECHO "type 'q' to end your list."
	>$ExclListFileName
	EListOpt=
	GetFiles $ExclListFileName 0
	if [ -s $ExclListFileName ]
	then
		$ECHO "`date`: Added the following files for exclusion for $iSeriesClient:" >> $iSERIESLOG
		cat $ExclListFileName >> $iSERIESLOG
		EListOpt="-e $ExclListFileName"
	fi
}

#########################################################################
#
# GetIncludeList
#
# Ask for Full Pathnames to Includes until they type "q"
#
#########################################################################
GetIncludeList()
{
	clear
	$ECHO "You may enter a list of files (full pathname) to include"
	$ECHO "in your restore if you wish. Please specify one filename"
	$ECHO "per line. You may leave the Target Filename field blank "
	$ECHO "in order to restore to the file's original location. When"
	$ECHO "you have completed your include list, please type 'q' to"
	$ECHO "end your list."
	>$InclListFileName
	IListOpt=
	GetFiles $InclListFileName 1
	if [ -s $InclListFileName ]
	then
		$ECHO "`date`: Added the following files for inclusion for $iSeriesClient:" >> $iSERIESLOG
		cat $InclListFileName >> $iSERIESLOG
		IListOpt="-i $InclListFileName"
	fi
}
#########################################################################
#
# DisplayBackups
#
# Get all the Default path information from the profile.
#
#########################################################################
DisplayBackups()
{
	backupsList="$1"

	clear
	# 5239
	backupPath="$DefaultD2DDevice"
	if [ "$backupPath" = "xxx" ]
	then
		$ECHO "ERROR: Cannot determine path to locate backups."
		$ECHO "`date`: Cannot determine path to locate backups. Aborting" >> $iSERIESLOG
		cleanup 1
	fi
	$ECHO "Please select the backup you would like to restore from week $week:"
	> /tmp/backuplist$$
	listItem=1
	for backupString in $backupsList
	do
		backupNum=`$ECHO $backupString | awk -F"|" '{print $1}'`
		backupType=`$ECHO $backupString | awk -F"|" '{print $2}'`
		backupStart=`$ECHO $backupString | awk -F"|" '{print $3}'`
		backupReadableStart="`date -d "1970-01-01 $backupStart sec UTC" +"%m/%d/%Y"`"
		if [ -a "${backupPath}backup_$backupNum" ]
		then
			$ECHO "${backupPath}backup_$backupNum" >> /tmp/backuplist$$
		elif [ -a "${backupPath}backup_${backupNum}.gz" ]
		then
			$ECHO "${backupPath}backup_${backupNum}.gz" >> /tmp/backuplist$$
		else
			$ECHO "`date`: Cannot locate backup file ${backupPath}backup_${backupNum}." >> $iSERIESLOG
			continue
			#GetBackups
		fi
		$ECHO "$listItem.  backup_$backupNum	$backupType	$backupReadableStart" 
		listItem=`expr $listItem + 1`
	done
	${ECHO} "Enter selection for backup or 'q' for none of these, go to previous week: \c" 
	read ANS
	case ${ANS} in
	[0-9]*)
		SelectedBackup=`cat /tmp/backuplist$$ |  awk -vthisfile="$ANS" '{if(NR == thisfile) print $1}'`
		if [ ! -r "$SelectedBackup" ]
		then
			$ECHO "Backup $SelectedBackup no longer exists. It may have been purged."
			sleep 2
			DisplayBackups $backupsList
		fi
		return 0
		;;
	"q"|"Q")
		return 2
		;;
	*)
		$ECHO "Invalid Selection. Please try again."
		sleep 2
		DisplayBackups $backupsList
		;;
	esac
	return
		
}
#########################################################################
#
# GetBackups
#
# Display one week's backups at a time for them to select to restore from.
# Display the number of weeks matching their retention days in the
# /etc/unitrends-retention file.
#
#########################################################################
GetBackups()
{

	clear
	# 5179
	#retentionDays=`$BPBIN/bputil -g License RetentionDays "xxx" $BPRETENTION`
	retentionDays=`$BPBIN/lmmgr -info /usr/bp/bin/tasker 2> /dev/null | awk '{if ($1 ~ "Feature:") print $2}' | awk -F"," '{for (i=1; i <= NF; i++) if($i ~ "RD=") print $i }' | awk -F"=" '{print $2}'`
	if [ "$retentionDays" = "" ]
	then
		retentionDays=30
		$ECHO "Could not determine license retention days from database. Defaulting to 30 days."
                $ECHO "`date`: Couldn't read RD from license string. Defaulting to 30 days." >> $iSERIESLOG
	else
                $ECHO "`date`: License indicates $retentionDays retention days." >> $iSERIESLOG
	fi
	
	# Grab the current time in EPOCH so we can display backups one week at a time.
	Now=`date`
	ThisRestoreTime=`date --date="$Now" +%s`
	# There are 604,800 seconds in a week. Use this for EPOCH math.
	# There are 86,400 seconds in a day. Use this for EPOCH math.
	offset=0
	oneWeek=604800
	oneDay=86400
	lastWeek=`expr $ThisRestoreTime - $oneWeek`
	maxEpoch=$ThisRestoreTime
	minEpoch=$lastWeek
	retentionEPOCH=`expr $retentionDays \* $oneDay`
	week=1
	while [ $offset -le $retentionEPOCH ]
	do
		backupNumberList=`$BPBIN/iSeries/iSeries_util.php -r $iSeriesClient $minEpoch $maxEpoch`
		if [ "$backupNumberList" != "" ]
		then
			DisplayBackups "$backupNumberList"
			case $? in
			0)
				break
			;;
			1)
				$ECHO "ERROR: Could not process backups for $iSeriesClient. Aborting."
				$ECHO "`date`: ERROR: Could not process backups for $iSeriesClient. Aborting." >> $iSERIESLOG
				cleanup 1
			;;
			2)
				offset=`expr $offset + $oneWeek`
				minEpoch=`expr $minEpoch - $oneWeek`
				maxEpoch=`expr $maxEpoch - $oneWeek`
				week=`expr $week + 1`
			;;
			esac
		else
			$ECHO "No backups available for week $week."
			sleep 2
			$ECHO "`date`: No backups available for week $week for $iSeriesClient." >> $iSERIESLOG
			offset=`expr $offset + $oneWeek`
			minEpoch=`expr $minEpoch - $oneWeek`
			maxEpoch=`expr $maxEpoch - $oneWeek`
			week=`expr $week + 1`
		fi
	done

	if [ $offset -gt $retentionEPOCH ]
	then
		$ECHO "There were no backups available in the last $retentionDays days. Press <Enter> to exit."
		read
		cleanup 1
	fi
	
	return
}

#
# MAIN PROGRAM
#
# iSeries restores will connect to the Server and use
# backupFTP to send files.

DAClient=`$BPBIN/iSeries/iSeries_util.php -l | grep -v -e '->' | wc -l`
if [ $DAClient -le 1 ]
then
	$ECHO "You must have at least one direct-attached client to restore."
	sleep 5
	exit 1
fi


GetiSeriesClientName

iSERIESLOCK="/var/lock/$iSeriesClient"
if ! lockfile -r 0 $iSERIESLOCK 2> /dev/null
then
        $ECHO "Processing already in progress for iSeries client $iSeriesClient. Cannot continue."
        $ECHO "`date`: Processing already in progress for iSeries client $iSeriesClient. Cannot continue."  >> $iSERIESLOG
        sleep 5
        exit 1
fi

trap 'cleanup $?' EXIT

iSERIESPROFILE=$i5PRO/$iSeriesClient.spr
iSeriesLogLevel=`$BPBIN/bputil -g debugging iseriesclient 0 $BPDIR/bpinit/master.ini`

# 5239
DefaultD2DDevice=`$BPBIN/iSeries/iSeries_util.php -d`
if [ ! -d "$DefaultD2DDevice" ]
then
	$ECHO "Cannot locate default D2D device $DefaultD2DDecive. Cannot continue." | tee -a $iSERIESLOG
	exit 1
fi

# 5239
DefaultD2DDeviceName=`$BPBIN/iSeries/iSeries_util.php -n`
if [ "$DefaultD2DDeviceName" = "xxx" ]
then
	$ECHO "Cannot determine default D2D device name. Cannot continue." | tee -a $iSERIESLOG
	exit 1
fi

ExclListFileName=$i5PRO/$$_${iSeriesClient}.excl
InclListFileName=$i5PRO/$$_${iSeriesClient}.incl
>$ExclListFileName
>$InclListFileName

GetBackups

GetRestoreType

$ECHO ""
ANS=""
echo $iSeriesClient | grep -q -e '->'
if [ $? = 0 ]
then
	$ECHO "You must restore to a directly-attached client."
	$ECHO "Press <Enter> to select a target."
	ANS="y"
	read
else
	$ECHO "Would you like to restore to an alternate server? <y or n - default n>: \c"
	read ANS
fi

case ${ANS} in
	"y"|"Y"|"1")
		GetiSeriesClientName local
		$ECHO "Automatically excluding \"/Security Data\" and \"/System Configuration\"" | tee -a $iSERIESLOG
		echo "/Security Data" >> $ExclListFileName
		echo "/System Configuration" >> $ExclListFileName
		EListOpt="-e $ExclListFileName"
		;;
esac	

GetUserNameAndPassword

TestiSeriesConnection

# Consider the case where they have not selected a backup.
if [ "$SelectedBackup" = "" ]
then
	# 5239
	$ECHO "The restore for $iSeriesClient cannot continue. No backups were selected." | tee -a $iSERIESLOG
	$ECHO "Press <Enter> to continue: \c"
	read
	cleanup 1
fi
export IPW=`cat $i5PRO/.restore_IPW`
# 5099

processID=$$
$BPBIN/bptool -j create $iSeriesClient $DefaultD2DDeviceName restore $processID > /tmp/bpt$$ 2>&1
bptoolRC=$?
cat /tmp/bpt$$ >> $iSERIESLOG
jobNumber=`cat /tmp/bpt$$ | grep "^job_id" 2> /dev/null | awk -F"=" '{print $2}'`
backupNumber=${SelectedBackup##*_}
ulimit -c unlimited
if [ $bptoolRC -eq 0 ]
then
	$ECHO "`date`: iSeries restore beginning. This could take some time. Please wait...\c"
	$BPBIN/bputil -j $jobNumber "Sending files to $iSeriesClient."


	cat $iSERIESLOG > $BPLOG/iSeries_restore${SelectedBackup:0-1:1}.log
	rm $iSERIESLOG
	iSERIESLOG=$BPLOG/iSeries_restore${SelectedBackup:0-1:1}.log

# 8695: run all backups through reduplication; this also uncompresses if it's compressed.
	RCFILE=/tmp/rc$$
	(
		$BPBIN/fileDedup --redup=$SelectedBackup 2> /dev/null | $BPBIN/backupFTP --server $iSeriesClient --restore - $IListOpt $EListOpt --credentials ${iSeriesUsername}:ENV:IPW --verbose=$iSeriesLogLevel >> $iSERIESLOG 2>&1
		echo $? > $RCFILE	
	) > /dev/null 2>&1 &

	while [ ! -s $RCFILE ]
	do
		sleep 5
		if [ `psql bpdb -U postgres -tAc "select status & 32768 from bp.jobs where job_no = $jobNumber"` -ne 0 ]
		then
			pkill -15 -g $$ backupFTP
		fi
	done

	RC=`cat $RCFILE`
	rm -f $RCFILE
	rm $i5PRO/.restore_IPW

	$ECHO ""
	
	case $RC in
	0)
		$ECHO "The restore of $SelectedBackup to $iSeriesClient is SUCCESSFUL... \c" | tee -a $iSERIESLOG
		$ECHO "`date`: iSeries restore -- Complete." >> $iSERIESLOG
		$BPBIN/bputil -j $jobNumber "Restore of backup $backupNumber was successful."
		$BPBIN/bptool -j update_status $jobNumber success > /tmp/bpt$$ 2>&1
		read
		exit $RC
	;;
	143)
		$ECHO "The restore of $SelectedBackup was cancelled." | tee -a $iSERIESLOG
		$BPBIN/bputil -j $jobNumber "Restore of backup $backupNumber was cancelled."
		$BPBIN/bptool -j update_status $jobNumber failed > /tmp/bpt$$ 2>&1
		read
		exit $RC
	;;
	*)
		$ECHO "The restore of $SelectedBackup FAILED with return code $RC." | tee -a $iSERIESLOG
		$ECHO "Please verify the client name and username-password  credentials you" | tee -a $iSERIESLOG
		$ECHO "supplied are correct and rerun this restore... \c" | tee -a $iSERIESLOG
		$BPBIN/bputil -j $jobNumber "Restore of backup $backupNumber failed."
		$BPBIN/bptool -j update_status $jobNumber failed > /tmp/bpt$$ 2>&1
		read
		exit $RC
	;;
	esac
else
	$ECHO "The restore of $SelectedBackup FAILED with return code $bptoolRC." | tee -a $iSERIESLOG
	$ECHO "Please verify the client name and username-password  credentials you" | tee -a $iSERIESLOG
	$ECHO "supplied are correct and rerun this restore... \c" | tee -a $iSERIESLOG
	$BPBIN/bptool -j update_status $jobNumber failed > /tmp/bpt$$ 2>&1

	rm -f $i5PRO/.restore_IPW 

	read
	exit $bptoolRC
fi
