use strict;
use Pg;
use IO::Handle;
use Sys::Hostname;
use EventLogging;


my $debug = 0;

# command line args:
# pglobal:       t/f (if true => it goes into bp.eventlog)
# plevel:        0=FATAL 1=ERROR 2=WARNING 3=INFO 4=DEBUG
# pevtype:       an eventtypeid from the eventtype table (preloaded)
# psource:       process or application
# pmsg:          message that explains the event

my $usage = "\nUsage: logevent.pl <pglobal> <plevel> <pevtype> <psource> <pmsg>
               pglobal:       t/f (if true => it goes into bp.eventlog)
               plevel:        0=FATAL 1=ERROR 2=WARNING 3=INFO 4=DEBUG
               pevtype:       an eventtypeid from the eventtype table (preloaded)
               psource:       process or application
               pmsg:          message that explains the event\n\n";

if (@ARGV != 5) {
	die $usage; 
}

my $dpuname = hostname();

my $connStr = "host=localhost dbname=bpdb user=postgres password=";
my $dbConn = Pg::connectdb($connStr);
unless($dbConn->status == PGRES_CONNECTION_OK) {
	if ($debug) {
		print "Can't connect to the database!\n";
	}
	exit 2;
}

if ($debug) {
	print "Command line args are:\n";
	print "pglobal=$ARGV[0]\n";
	print "plevel=$ARGV[1]\n";
	print "pevtype=$ARGV[2]\n";
	print "psource=$ARGV[3]\n";
	print "pmsg=$ARGV[4]\n";
}

# Set up an event group for logging.
my ($ret, $egrp) = geteventgroup($dbConn, 't', "Event logging for $ARGV[3]");
unless ( $ret == PGRES_TUPLES_OK ) {
	if ($debug) {
		print "Can't get eventgroup, return code was $ret\n";
	}
	exit 4; # can't call addeventlog if this fails...
}
if ($debug) {
	print "ret is $ret, egrp is $egrp\n";
}

my $result = logevent($dbConn,$ARGV[0], $ARGV[1], $ARGV[2], $ARGV[3], $ARGV[4], $dpuname, $egrp);
if ($debug) {
	print "return code from logevent was $result\n";
}

if ($result == PGRES_TUPLES_OK ) {
	exit 0;
}
else {
	exit 4;
}
