#!/bin/bash

progName=$(basename $0)
lockFile=/var/lock/"$progName"-lock

#
#  Where're we installed?
#
eval $(egrep '^[[:space:]]*BPDIR[[:space:]]*=' /etc/default/bp.ini)

#
#  Are we configured for the correct metadata generation?
#
metadataGeneration=$(${BPDIR}/bin/bputil -g fileDedup			\
			MetadataGeneration 0 ${BPDIR}/bpinit/master.ini)
if ((metadataGeneration != 1)); then
    echo "$progName: Applicance not configured for second generation metadata"
    exit 1
fi

#
#  Make sure only one copy of this script can run at a time.
#
if ! lockfile -r0 "$lockFile"; then
    echo "$progName: Can't get the lock"
    exit 1
fi
trap "rm -f $lockFile" EXIT

#
#  Rotate the log files
#
logFile="${BPDIR}"/logs.dir/"${progName}"-log
for ((i = 8; (i > 0); --i)); do
    mv -f "$logFile".$(printf "%02d" $i)				\
    	"$logFile".$(printf "%02d" $((i + 1)))	> /dev/null 2>&1
done
mv -f "$logFile" "$logFile".$(printf "%02d" 1)	> /dev/null 2>&1

#
#  From here on everything goes into the log file
#
exec > "$logFile" 2>&1

#
#  Timestamp the log file
#
date

#
#  For every backup in the metadata ...
#
${BPDIR}/bin/metadataUtil -b						|
while read metadataBackupNumber; do
	#
	#  See if the backup is in the database.
	#
	DBbackupNumber=$(psql -Atf - bpdb postgres <<-EoF
	SELECT '$metadataBackupNumber missing'
	WHERE (NOT EXISTS(SELECT 1
			  FROM bp.backups
			  WHERE (backup_no = $metadataBackupNumber)))
	EoF)

	if [[ "$DBbackupNumber" != "$metadataBackupNumber missing" ]]; then
	    #
	    #  The backup is in the database.  So, skip it.
	    #
	    continue
	fi

	#
	#  The backup is not in the database.  So, free its metadata.
	#
	echo "Freeing backup $metadataBackupNumber"
	${BPDIR}/bin/metadataUtil --free-backup="$metadataBackupNumber"
done
