#!/usr/bin/php
<?php

# Alertman will run this script if the status column of the
# notification_mgmt table is 'needs migration' (set during upgrade).
# Any blocks that apply between the previous revision and the current
# revision will be executed.

function do_update($query) {
	// This function updates the alerts table and exits if it fails
	global $debug;
	global $dbconn;

	if ($debug) print $query."\n";
	$upd_res = pg_query($dbconn, $query);
	if ($upd_res === false) {
		print "ERROR: failed to update notification_id in the alerts table:\n";
		print pg_last_error($dbconn)."\n";
		die(1);
	}
}

$debug=false; // set this to see all queries printed.
$dbconn_string = "port=5432 dbname=bpdb user=postgres";

$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	print "ERROR: migrate_alerts: Failed to connect to the database.\n";
	die(1);
}

// Make sure we should run (check status).  There should only be one
// record in the table, but use ORDER & LIMIT in case there are more.
$query = "SELECT * FROM bp.notification_mgmt ORDER BY revision DESC LIMIT 1";
if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
if ($res === false or pg_num_rows($res) == 0) {
	print "ERROR: migrate_alerts failed to access the notification_mgmt table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}

$row = pg_fetch_assoc($res);
if ($row['status'] != 'needs migration') {
	print "No migration is needed for notification management.\n";
	exit(0);
}

$prev_rev = $row['upgrade_from'];
$new_rev = $row['revision'];

if ($prev_rev < 1 ) {
	// This block of code accomplishes migration of records in the alerts
	// table for revision #1 of the notification framework.  The new column
	// in the table, notification_id, must be set for any existing alerts.
	$query = "SELECT DISTINCT(source_id) FROM bp.alerts WHERE notification_id=0";
	if ($debug) print $query."\n";
	$res = pg_query($dbconn, $query);
	if ($res === false) {
		print "ERROR: migrate_alerts failed to access the alerts table:\n";
		print pg_last_error($dbconn)."\n";
		die(1);
	}
	$nrows = pg_num_rows($res);
	// if there are no rows to migrate, the loop is skipped.
	$upd_str = "UPDATE bp.alerts SET notification_id=";
	$where = " WHERE notification_id=0";
	$txt = " AND alert_text LIKE ";
	for ($i = 0; $i < $nrows; $i++) {
		$src = pg_fetch_result($res, $i, 0);
		$and_src = " AND source_id=".$src;
		switch($src) {
			case 1: // Clients
				if($debug) print "Processing Clients alert_source (source_id=$src).\n";
				$q = $upd_str."9".$where.$txt."'%no longer%'";
				$q.= $and_src;
				do_update($q);
				$q = $upd_str."10".$where.$txt."'%now listen%'";
				$q.= $and_src;
				do_update($q);
				break;
			case 2: // 3ware
				if($debug) print "Processing 3ware alert_source (source_id=$src).\n";
				$str1 = $txt."'%unit status%' AND severity=";
				$str2 = $txt."'%drive status%' AND severity=";
				$q=$upd_str."2" . $where .$str1 . "1" . $and_src;
				do_update($q);
				$q=$upd_str."3" . $where .$str2 . "1" . $and_src;
				do_update($q);
				$q=$upd_str."4" . $where .$str1 . "2" . $and_src;
				do_update($q);
				$q=$upd_str."5" . $where .$str2 . "2" . $and_src;
				do_update($q);
				$q=$upd_str."6" . $where .$str1 . "3" . $and_src;
				do_update($q);
				$q=$upd_str."7" . $where .$str2 . "3" . $and_src;
				do_update($q);
				break;
			case 3: // PCI
				if($debug) print "Processing PCI alert_source (source_id=$src).\n";
				$q = $upd_str."17".$where.$txt."'%disconnect%'";
				$q.= $and_src;
				do_update($q);
				$q = $upd_str."18".$where.$txt."'%installed%'";
				$q.= $and_src;
				do_update($q);
				break;
			case 5: // Process
				if($debug) print "Processing Process alert_source (source_id=$src).\n";
				$q = $upd_str . "20". $where . " AND target='license'". $and_src;
				do_update($q);

				$str = " AND target='cryptoDaemon'";
				$q = $upd_str."21".$where.$str.$txt."'%not%'";
				$q.= $and_src;
				do_update($q);
				$q = $upd_str."22".$where.$str.$txt."'%master%'";
				$q.= $and_src;
				do_update($q);
				$q = $upd_str."39".$where.$str." AND severity=3". $and_src;
				do_update($q);

				$str = " AND target='retention'";
				$q = $upd_str."23".$where.$str.$txt."'%Grace%'" . $and_src;
				do_update($q);
				$q = $upd_str."24".$where.$str.$txt."'%Failure%'" . $and_src;
				do_update($q);
				$q = $upd_str."27".$where.$str.$txt."'%70%'" . $and_src;
				do_update($q);
				$q = $upd_str."28".$where.$str.$txt."'%80%'" . $and_src;
				do_update($q);
				$q = $upd_str."29".$where.$str.$txt."'%90%'" . $and_src;
				do_update($q);
				$q = $upd_str."30".$where.$str.$txt."'%unable to calc%'".$and_src;
				do_update($q);
				$q = $upd_str."31".$where.$str.$txt."'%tool has not%'".$and_src;
				do_update($q);

				$q = $upd_str . "33". $where . " AND target='reboot'". $and_src;
				do_update($q);
				$q=$upd_str."34".$where. " AND target='drivers-update-4.1'".$and_src;
				do_update($q);

				$str = " AND target='Migration'";
				$q = $upd_str."35".$where.$str.$and_src;
				do_update($q);

				$str = " AND target='DA'";
				$q = $upd_str."36".$where.$str.$txt."'%expired%'". $and_src;
				do_update($q);
				$q = $upd_str."37".$where.$str.$txt."'%expires on%'". $and_src;
				do_update($q);

				$q = $upd_str . "25". $where . " AND target='update'". $and_src;
				do_update($q);

				$str = " AND target='tasker'";
				$q = $upd_str."26".$where.$str." AND severity=2". $and_src;
				do_update($q);
				$q = $upd_str."38".$where.$str." AND severity=3". $and_src;
				do_update($q);

				break;
			case 6: // Disk
				if($debug) print "Processing Disk alert_source (source_id=$src).\n";
				$q = $upd_str . "12" . $where . $and_src ; 
				do_update($q);
				break;
			case 7: // CEP
				if($debug) print "Processing CEP alert_source (source_id=$src).\n";
				$q = $upd_str."50".$where.$txt."'Workspace%'";
				$q.= $and_src;
				do_update($q);
				$q = $upd_str."51".$where.$txt."'Global%'";
				$q.= $and_src;
				do_update($q);
				break;
			case 8: // DPU
				if($debug) print "Processing DPU alert_source (source_id=$src).\n";
				$q = $upd_str."14".$where.$txt."'%cannot %'";
				$q.= $and_src;
				do_update($q);
				$q = $upd_str."15".$where.$txt."'%now respond%'";
				$q.= $and_src;
				do_update($q);
				break;
			case 9: // RAID
				if($debug) print "Processing RAID alert_source (source_id=$src).\n";
				$q = $upd_str."41".$where.$txt."'%changed to offline%'".$and_src;
				do_update($q);

				$str = $txt."'%Array status%' AND severity=";
				$q= $upd_str."43" . $where . $str . "1" . $and_src;
				do_update($q);
				$q= $upd_str."45" . $where . $str . "2" . $and_src;
				do_update($q);
				$q= $upd_str."48" . $where . $str . "3" . $and_src;
				do_update($q);

				$str = $txt."'Drive status%' AND severity=";
				$q= $upd_str."42" . $where . $str . "1" . $and_src;
				do_update($q);
				$q= $upd_str."44" . $where . $str . "2" . $and_src;
				do_update($q);

				$str = $txt."'Drive status%REBUILDING%' AND ";
				$q= $upd_str."46" . $where . $str . "severity=3" . $and_src;
				do_update($q);

				$str = $txt."'Drive status%OK%' AND ";
				$q= $upd_str."47" . $where . $str . "severity=3" . $and_src;
				do_update($q);
				break;
			default:
				break;
		}
	}
}

// Add new blocks here for migration of other revisions
// if ($prev_rev < N) {
//	code to migrate for revision #N
// }

$query = "UPDATE bp.notification_mgmt SET status = 'complete', ";
$query .="upgrade_from = ". $new_rev;

if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
if ($res === false) {
	print "ERROR: migrate_alerts failed to update notification_mgmt table.\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}
else {
	print "Migration complete for notification management.\n";
	exit(0);
}

?>
