#!/usr/bin/php
<?php

// This script goes through all the systems given remote access permission
// in pg_hba.conf and inserts a corresponding record in the bp.managers
// table.  It attempts to determine the hostname of the system, but if it
// can't, it enters the hostname as 'Unknown' (the version will be updated
// later by the manager system when it connects to the DPU during the
// alertman "alive" check).
// Returns: 0 on success, 1 on failure.

$debug=false;
$dbconn_string = "port=5432 dbname=bpdb user=postgres";
$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	print "ERROR: Failed to connect to the database.\n";
	die(1);
}

$bpdir = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));
$hba_conf=$bpdir."/data/pg_hba.conf";
if ($debug) print "hba_conf is $hba_conf\n";

// Identify any external systems that may be managing this one.
$mgr_query = "SELECT ip, hostname from bp.managers";
$res = pg_query($dbconn, $mgr_query);
if ($res === false) {
	print "ERROR: failed to access the managers table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}
if (pg_num_rows($res) > 0) {
	print "It appears that managing systems have already been identified.\n";
	exit(0);
}
else {
	if($debug) print "Identifying managing systems...\n";
}

$insert_str = "INSERT INTO bp.managers (ip, hostname) VALUES ('";

// Get the IP addresses in pg_hba.conf allowing remote access
$cmd = "grep \"^hostssl[[:space:]].*bpdb[[:space:]].*postgres\" $hba_conf | awk '{ split($4,ip,\"/\"); print ip[1]}'";

exec($cmd, $mgrs, $ret);
if (!empty($mgrs)) {
	foreach($mgrs as $ip) {
		// Try to get the hostname for this IP address
		$host = trim(`grep "^$ip" /etc/hosts | awk '{print $2}'`);
		if (empty($host)) {
			$host = gethostbyaddr($ip);
			if ($host === $ip) {
				$host = "Unknown";
			}
		}
		if ($debug) print "IP: $ip  hostname: $host\n";

		// Insert the record into the managers table (version will be
		// 'Unknown' by default and will later be updated by alertman on
		// the managing system).
		$query = $insert_str.$ip."','".$host."')";
		if ($debug) print $query."\n";
		$res = pg_query($dbconn, $query);
		if ($res === false) {
			print "ERROR: failed to add $ip, $host to managers table:\n";
			print pg_last_error($dbconn)."\n";
			die(1);
		}
	}
	$res = pg_query($dbconn, $mgr_query);
	if ($res !== false) {
		$n = pg_num_rows($res);
		printf("%d managing system%s found:\n", $n, $n==1?"":"s");
		printf("IP ADDRESS        HOSTNAME\n");
		printf("----------------  ------------------------------\n");
		for ($i = 0; $i < $n; $i++) {
			$row = pg_fetch_assoc($res, $i);
			printf("%-16s  %s\n", $row["ip"], $row["hostname"]);
		}
	}
}
else {
	if ($debug) print "No managing systems found.\n";
}
exit (0);

?>
