<?php
require_once '/usr/bp/bin/psa/PSAServices.php';
require_once '/usr/bp/bin/psa/util.php';

/*
 * A function that creates a service ticket.
 * @param $args: all the arguments that are passed to the cmc_psa script.
 * 
 */
function queryAndCreateServiceTicket($args){
    global $logHandle;
    $notification_id = $args[2];
    $argc = count($args);
    $now = date("Y-m-d H:i:s"); 
    $arr = getShortAndLongDescription($notification_id);
    $summary = $arr["short_description"];
    $detailNotes = $arr["long_description"]."\n";
    //pg_free_result($arr);
    fwrite ($logHandle, $now ." Creating a service ticket for notification: ".$notification_id." (argc=".$argc.") Summary: ".$summary."\n");
    
    if ( $argc > 3){
        // this assumes that the number of specifiers in "long_description" 
        // is equal to number of the notification arguments passed to cmc_psa
        $detailNotes = vsprintf($detailNotes, array_slice($args, 3));
    }
    queryAndCreateTicket($args[1],$notification_id, $summary, $detailNotes);
}

/*
 * A function that creates a test ticket.
 * @param $args: all the arguments that are passed to the cmc_psa script.
 */
function queryAndCreateTestTicket($args){
    global $logHandle;
    $now = date("Y-m-d H:i:s"); 
    fwrite ($logHandle, $now . " Creating test ticket\n");
    //@todo update the text of the test ticket 
    $summary = "Test Ticket";
    $detailNotes = "This ticket is a confirmation of correct ".$args[1]." configuration.\n";
    queryAndCreateTicket($args[1],0, $summary, $detailNotes);
}

/*
 * A function to create a billing ticket. 
 *  
 */
function queryAndCreateBillingTicket ($args){
    $now = date("Y-m-d H:i:s"); 
    fwrite ($logHandle, $now  . " Creating a billing ticket\n");
    // This can take a long time on a fully loaded system.
    $ret = bp_get_retention_info();
    if ($ret == false) {
        global $logHandle;
        $error = "Could not create a Billing ticket as retention information could not be retrieved.\n";
        writeToHistoryTable($args[2], 3, $args[1], "N/A", $error);
	writeErrorAndExit($error);
    }
    
    // Get the values from the retention output.
    $used = $ret["used"];
    $capacity = $ret["available"];
        
    // @todo modify the summary text
    $summary = "Information for Billing";
    // @todo check that this is what is needed
    $notes = "D2D Licensed Capacity: ".$used."\n";
    $notes .= "Used Capacity: ".$capacity."\n";
    queryAndCreateTicket($args[1],0, $summary, $notes);    
}

/*
 * A generic function to create a ticket. 
 * @param $not_id : notification id
 * @summary: summary of the ticket
 * @notes: detailed notes of the ticket 
 */
function queryAndCreateTicket ($psa_tool_name, $not_id, $summary, $detailNotes){
    global $logHandle;
    $notification_id = $not_id;
    
    //default values for variables so that they can be inserted into history table
    $severity="-1";
    $ticket_id=null;
    
    try{ 
        $arr = getPSAConfiguration($psa_tool_name);
        $url = $arr["url"]; 
        $company = $arr["company_id"];
        $credential_id = $arr["credential_id"];
        if ( $psa_tool_name == "default"){
            $psa_tool_name = $arr["psa_tool_name"];
        }

        $arr = getPSACredentials($credential_id);
        $username = $arr["username"];
        $password = $arr["password"];
        
        exec("uname -n", $hostname, $retval);
        if ( $retval == 0 ){
            $hostname = $hostname[0];
            $metaDetail = "Host Name: ".$hostname."\n";
        } else{
            fwrite($logHandle, "Could not retrieve the hostname.\n");
            $metaDetail = "Host Name: Unknown\n";
        }

	exec("cat /etc/unitrends-asset", $out, $retval);
        if ($retval == 0) {
            $tag = $out[0];
            $metaDetail .= "Asset tag: ".$tag."\n";
        }
        else {
            fwrite($logHandle, "Could not retrieve the Asset Tag.\n");
            $metaDetail .= "Asset tag: Unknown\n";
        }
        
	// Sometimes need company for credentials different than companyId.
	// see master.ini ClientCompany* handling in createTicket
        $companyId = $company;
        
	global $type;
        if ( $type == "test" | $type == "billing" ){
		$severity = 3;
	} else {
		$severity = getSeverity($notification_id);
	}
	// remove bogus escape chars in message
	$find = array("\033","\034","\035","\036","\037");
	$detailmsg = str_replace($find, "", $detailNotes.$metaDetail);
        fwrite($logHandle, "Ticket Detail Message:\n".$detailmsg."\n"); 
        $ticket_id = createTicket($psa_tool_name, $url, $company, $username, $password, $summary, $detailmsg, $companyId, $severity);
        
	$rv = 0;
	$now = date("Y-m-d H:i:s"); 
        $message = $now . " Ticket [".$ticket_id." sev=".$severity." ".$summary."] created successfully for notifID ".$notification_id."\n";
        fwrite($logHandle, $message);

        fwrite(STDOUT, $ticket_id . "\n");
	// the below message will make sure that the user gets the ticket number when writing to history table fails.                
	fwrite(STDERR, $message);
    }catch (Exception $e){
	$rv = 1;
	$now = date("Y-m-d H:i:s"); 
        $logmsg = $now . " Ticket [".$ticket_id." sev=".$severity." ".$summary."] creation failed for notifid ".$notification_id.": \n".$e->getMessage()."\n";
        $error = "Ticket ".$ticket_id." creation failed: \n".getErrorFromException ($e->getMessage());
	// log raw message, but show verbose error message
        fwrite($logHandle, $logmsg); 
	fwrite(STDERR, $error); 
	}
    try{
        // if ( $ticket_id ){
        if ($rv == 0) {
	       $historyDetail = "Summary: ".$summary."\nTicket Detail:\n".$detailNotes."\n";
               writeToHistoryTable($notification_id, $severity, $psa_tool_name, $ticket_id, $historyDetail);
	       okExit();
        } else{
	       $historyDetail = "Summary: ".$summary."\nTicket Detail:\n".$detailNotes."Error Detail:\n".$error."\n";
               writeToHistoryTable($notification_id, $severity, $psa_tool_name, "N/A", $historyDetail);
	       writeErrorAndExit("");
        }
    }catch(Exception $e){
        $message = "Ticket information could not be written to the History table. Reason: \n".$e->getMessage()."\n";
        // writeErrorAndExit($message);
        fwrite($logHandle, $message);
	okExit();
    }
}

// this function checks if the error contains a particular string and if it does returns a corresponding message to the GUI. 
// don't really like scrapping the message like this. 
function getErrorFromException ( $reason ){
	
	$key = array ("Username or password is incorrect", 
			"Cannot find company",
			"This company is in a status that does not allow",
			"PHP Warning: SoapClient::",
			"PHP Fatal error: SOAP-ERROR: Parsing WSDL: Couldn't load from", 
			"SOAP-ERROR: Parsing WSDL: Couldn't load from", 
			"Soap error with WSDL file");
	$value = array ("Incorrect authentication credentials.\n",
	"Company ID could not be found. Please correct the company ID and try again.\n",
	"The company ID you are using does not have sufficient privileges to complete this operation. Please check with your ConnectWise administrator.\n",
	"SoapClient error.  Check that the URL is correct.\n", 
	"Could not load the WSDL file. Check the URL is correct and that there is network connectivity.\n",
	"Could not load the WSDL file. Check the URL is correct and that there is network connectivity.\n",
	"Could not load the WSDL file. Check the URL is correct and that there is network connectivity.\n");
	
	for ( $i = 0 ; $i< count($key) ; $i++) {
		if ( stripos ($reason, $key[$i] ) !== FALSE ){
			return $value[$i];
		}
	} 
        // if not matching common errors, check/truncate raw reason
	$msg = str_check_quotes($reason);
	return $msg; 
}
?>
