#!/usr/bin/php -q
<?php

$modeDebug = false; // if set to yes, it will create a minimal report
$modePDF = false;

$reportWidth = 870;
$reportWidthMinusOffset = 860;

$tableCellHeight = 24;
$tableCellWidth = 30;
$tableCellSpacerWidth = 10;

function getClientCellWidth() {
	GLOBAL $reportWidthMinusOffset;
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellSpacerWidth;
	$clientWidth = $reportWidthMinusOffset - ( $tableCellWidth * 7 * 2 + $tableCellSpacerWidth * 2 );	
	return $clientWidth;
}
function getBackupsHeaderCellWidth() {
    // used for both Backups and Replication headers
	GLOBAL $reportWidthMinusOffset;
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellSpacerWidth;
	$clientWidth = ( $reportWidthMinusOffset - getClientCellWidth() ) / 2;	
	return $clientWidth;
}

function build_legend() {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;
	}
	GLOBAL $reportWidth;
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellHeight;
	
	$redImg		= getRedImageSrc();
	$greenImg	= getGreenImageSrc();
	$yellowImg	= getYellowImageSrc();
	
	$legendStyle = getLegendStyle();
	
    $results = <<<EOHTML
					<table width="$reportWidth" border="0" cellspacing="0" cellpadding="0" align='center'>
						<tr>
							<td style="$legendStyle">&nbsp;</td>
							<td width="100" height="$tableCellHeight" style="$legendStyle">Success&nbsp;</td>
							<td width="$tableCellWidth" height="$tableCellHeight" style="$legendStyle" bgcolor="#007733"></td>
							<td width="100" height="$tableCellHeight" style="$legendStyle">Warning&nbsp;</td>
							<td width="$tableCellWidth" height="$tableCellHeight" style="$legendStyle" bgcolor="#FFFF66"></td>
							<td width="100" height="$tableCellHeight" style="$legendStyle">Failed&nbsp;</td>
							<td width="$tableCellWidth" height="$tableCellHeight" style="$legendStyle" bgcolor="#EE3333"></td>
						</tr>
					</table>
EOHTML;
	return $results;
}


function build_filler($height) {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;	
	}
	GLOBAL $tableCellWidth;
	
	$day_cols = "";
	for ($i=0; $i<7; $i++) {
		$daydiff = $i - date("w");
		switch ($i) {
			case 0:  $loc = 'L'; break; # left end of table
			case 6:  $loc = 'R'; break; # right end of table
			default: $loc = 'C'; break; # neither end of table
		}
		switch ($daydiff) {
			case -1: $hl = 'R';  break; # on left of current day
			case 0:  $hl = 'LR'; break; # on current day
			case 1:  $hl = 'L';  break; # on right of current day
			default: $hl = 'n';  break; # not near current day
		}
		$class = "Cal_{$loc}_{$hl} Cal_filler";
		
		if (date("w") == $i) $bgcolor = getCurrentDayColor();
		else $bgcolor = "#FFFFFF";

		$day_cols .= '		<td width="' . $tableCellWidth . '" height="' . $height . '" bgcolor="' . $bgcolor . '" class="' . $class . '">&nbsp;</td>' . "\n";
	}


	$results = <<<EOHTML
				<tr>
					<td height="$height"></td>
					<td height="$height"></td>
					$day_cols
					<td height="$height"></td>
					$day_cols
				</tr>
EOHTML;

	return $results;
}

function build_footer() {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;	
	}

	GLOBAL $reportWidth;
	GLOBAL $tableCellWidth;

	$height = "2";

	$day_cols = "";
	for ($i=0; $i<7; $i++) {
		$class = "Cal_footer_n";

		if ($i == date("w")) $class = "Cal_footer_LR";
		else $class = "Cal_footer_n";

		$day_cols .= '		<td width="' . $tableCellWidth . '" height="' . $height . '" bgcolor="#F1F1F1" class="' . $class . '">&nbsp;</td>' . "\n";
	}

	$results = <<<EOHTML
	<tr>
		<td height="$height"></td>
		<td height="$height"></td>
		$day_cols
		<td height="$height"></td>
		$day_cols
	</tr>
	</table>
	</td>
	</tr>
	<tr>
		<td width="$reportWidth" colspan='3' valign='top'>
			<img class='image_fix' src='http://www.unitrends.com/images/notifications/bottom-grey.png' alt='Backup Summary Area Bottom' title='Backup Summary Area Bottom' width="$reportWidth" height='17' />
		</td>
	</tr>
</table>
EOHTML;

	return $results;
}

function dpu_summary_build_title($dpuname, $systemVersion) {
	GLOBAL $reportWidth;
	GLOBAL $reportWidthMinusOffset;
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellSpacerWidth;
	
	$title = "System Status Report";
	$date = date("m-d-Y");
	$results = <<<EOHTML
	
	<table cellpadding='0' cellspacing='0' border='0' align='center'>
        	<tr>
            	<td colspan='3'><img class='image_fix' src='http://www.unitrends.com/images/notifications/pixel.gif' alt='tinySpacer' title='tinySpacer' width='1' height='6' /></td>
            </tr>
			<tr>
				<td width="$reportWidth" colspan='3' valign='bottom'>
               		<img class='image_fix' src='http://www.unitrends.com/images/notifications/white-top.png' alt='Title Area Top' title='Title Area Top' width="$reportWidth" height='17' />
                </td>
			</tr>
            <tr>
            	<td width='15' bgcolor='#F7F7F7'>&nbsp;</td>
            	<td width='215' align='justify' valign='middle' bgcolor='#F7F7F7'><img class='image_fix' src='http://www.unitrends.com/reports/color/logo.png' alt='Unitrends Logo' title='unitrends Logo' width='198' height='52' />
                </td>
				<td width='600' valign='middle' bgcolor='#F7F7F7'>
                    <table width='600' cellpadding='0' cellspacing='0' border='0' align='center'>
                        <tr>
                            <td><img class='image_fix' src='http://www.unitrends.com/images/notifications/pixel.gif' alt='tinySpacer' title='tinySpacer' width='1' height='10' /></td>
                        </tr>
                        <tr>
                            <td><h1 style='margin-top: 1px; margin-bottom: 1px; font-size: 32px; color: #E00034;}'>$title</h1></td>
                        </tr>
                        <tr>
                            <td><h3 style='font-size: 15px; color: #444444;'>System:  <em>$dpuname</em>  System Version: <em>$systemVersion</em></h3></td>
                        </tr>
                        <tr>
                            <td><span class='replicationHeadTitle'>System Activity for $date</span></td>
                        </tr>
                        <tr>
                            <td><img class='image_fix' src='http://www.unitrends.com/images/notifications/pixel.gif' alt='tinySpacer' title='tinySpacer' width='1' height='8' /></td>
                        </tr>
                    </table>
                 </td>
            </tr>
            <tr>
				<td width="$reportWidth" colspan='3' valign='top'>
                    <img class='image_fix' src='http://www.unitrends.com/images/notifications/bottom-white.png' alt='Title Area Bottom' title='Title Area Bottom' width="$reportWidth" height='17' />
                </td>
			</tr>
		</table>
EOHTML;

	return $results;
}

function dpu_summary_build_header($title, $col1, $col2, $col3) {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;	
	}
	GLOBAL $reportWidth;
	GLOBAL $reportWidthMinusOffset;
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellSpacerWidth;
	
	$days = array(0 => "Sun", 1 => "Mon", 2 => "Tue", 3 => "Wed", 4 => "Thu", 5 => "Fri", 6 => "Sat");

	$day_cols = '';
	for($i=0; $i<7; $i++) {
		$daydiff = $i - date("w");
		switch ($i) {
			case 0:  $loc = 'L'; break; # left end of table
			case 6:  $loc = 'R'; break; # right end of table
			default: $loc = 'C'; break; # neither end of table
		}
		$class = "Cal_header_$loc";

		if (date("w") == $i) $bgcolor = "#CCCCCC";
		else $bgcolor = "#FFFFFF";

		$day_cols .= '		<td width="' . $tableCellWidth . '" height="' .$tableCellHeight . '" bgcolor="' . $bgcolor . '" class="' . $class . '">' . $days[$i] . "</td>\n";
	}
	
	$clientWidth = getClientCellWidth();	
	$backupsHeaderWidth = getBackupsHeaderCellWidth();

	$results = <<<EOHTML
	<table width="$reportWidth" border="0" cellpadding="0" cellspacing="0" align='center'>
		<tr>
			<td width="$reportWidth" valign='bottom'>
				<img class='image_fix' src='http://www.unitrends.com/images/notifications/top-grey.png' alt='Backup Summary Area Top' title='Backup Summary Area Top' width="$reportWidth" height='17' />
			</td>
		</tr>
		<tr>
			<td width="$reportWidth" bgcolor="#666666" class="Overview_Header">
				<h2 style='font-size: 24px; color: white; margin-top: 1px; margin-bottom: 5px; margin-left: 15px;'>&nbsp;$title</h2>
			</td>
		</tr>
		<tr>
			<td width="$reportWidth" valign='top' align='center' bgcolor='#666666'>
				<table width="$reportWidthMinusOffset" border="0" cellpadding="0" cellspacing="0" bgcolor='#F1F1F1' align='center' style='border-collapse:collapse'>
					<tr style='color: #EFEFEF; background-color: #333333;'>
						<td width="$clientWidth" height="20" bgcolor="#333333" class="Header" style='margin-left: 20px; margin-bottom: 3px; font-size: 16px; color: #EFEFEF;'>&nbsp;$col1</td>
						<td width="$backupsHeaderWidth" colspan="8" height="20" class="Header" style='margin-left: 20px; margin-bottom: 3px; font-size: 16px; color: #EFEFEF;'><div align="center">$col2</div></td>
						<td width="$backupsHeaderWidth" colspan="8" height="20" class="Header" style='margin-left: 20px; margin-bottom: 3px; font-size: 16px; color: #EFEFEF;'><div align="center">$col3</div></td>
					</tr>
					<tr style='font-size: 10px; align="center'>
						<td width="$clientWidth" height="$tableCellHeight">&nbsp;</td>
						<td width="$tableCellSpacerWidth" height="$tableCellHeight">&nbsp;</td>
						$day_cols
						<td width="$tableCellSpacerWidth" height="$tableCellHeight">&nbsp;</td>
						$day_cols
					</tr>
EOHTML;
	$results .= build_filler(10);

	return $results;
}

function dpu_summary_build_row($dpu_id, $client, $client_id) {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;	
	}
	GLOBAL $tableCellHeight;
	GLOBAL $tableCellSpacerWidth;
	
	$backups0 = create_client_table(get_client_backups($client_id, time(), 0));
	$backups1 = create_client_table(get_client_backups($client_id, time(), 1));
	
	$clientCellWidth = getClientCellWidth();
	$clientStyle = getClientStyle();

	$results = <<<EOHTML
	<tr>
		<td width="$clientCellWidth" style="$clientStyle" height="$tableCellHeight">$client</td>
		<td width="$tableCellSpacerWidth" height="$tableCellHeight" bgcolor="#F1F1F1"></td>
		$backups0
		<td width="$tableCellSpacerWidth" height="$tableCellHeight" bgcolor="#F1F1F1"></td>
		$backups1
	</tr>
EOHTML;

	return $results;
}

function dpu_alerts_build_header() {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;	
	}	
	GLOBAL $reportWidth;
	GLOBAL $reportWidthMinusOffset;
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellHeight;
	
	$results = <<<EOHTML
			<table width="$reportWidth" border="0" cellspacing="0" cellpadding="0" align='center'>
				<tr>
					<td width="$reportWidth" valign='bottom'>
						<img class='image_fix' src='http://www.unitrends.com/images/notifications/white-top.png' alt='Title Area Top' title='Title Area Top' width="$reportWidth" height='17' />
					</td>
				</tr>
				<tr>
					<td width="$reportWidth" bgcolor="#F7F7F7" class="Overview_Header">
						<h1 style='font-size: 24px; color: #cc0000; margin-top: 5px; margin-bottom: 5px; margin-left: 15px;'>&nbsp;Alerts</h1>
					</td>
				</tr>
				<tr>
					<td width="$reportWidth" align='center' bgcolor="#F7F7F7">
						<table width="$reportWidthMinusOffset" border="1" cellspacing="0" cellpadding="0">
							<tr style='color: #EFEFEF; background-color: #333333;'>								
								<td width="50" class="Alert_Header">Severity</td>
								<td width="100" class="Alert_Header">Date</td>
								<td width="100" class="Alert_Header">Time</td>
								<td width="570" class="Alert_Header">Alert</td>
							</tr>
EOHTML;

        return $results;
}

function dpu_alerts_build_row($date, $time, $alert, $severity, $alert_no) {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;	
	}
	GLOBAL $ALERT_CRITICAL_IMG_PDF;
	GLOBAL $ALERT_WARNING_IMG_PDF;
	
	$severityImg = "";
	$severityAlt = "";
	
	switch($severity) {
		case "critical":
			//$severityImg = getAlertCriticalImageSrc();
			$severityAlt = "Critical";
			break;
		case "warning":
			//$severityImg = getAlertWarningImageSrc();
			$severityAlt = "Warning";
			break;
		case "notice":
			//$severityImg = ""; //TODO
			$severityAlt = "Notice";
			break;
	}
	
	$bgcolor = getOffRowColor();
	if ($alert_no % 2 == 0) {
		$bgcolor = "#FFFFFF";
	}
	
	$results = <<<EOHTML
			<tr style="color: #000000" bgcolor="$bgcolor">
				<td width="50" class="Alert" valign='top' bgcolor="$bgcolor" style='font-size: 14px;'>$severityAlt</td>
				<td width="100" class="Alert" valign='top' bgcolor="$bgcolor" style='font-size: 14px;'>$date</td>
				<td width="100" class="Alert" valign='top' bgcolor="$bgcolor" style='font-size: 14px;'>$time</td>
				<td width="570" class="Alert" bgcolor="$bgcolor" style='font-size: 14px;'>$alert</td>
			</tr>
EOHTML;

     return $results;
}

function dpu_alerts_build_footer() {
	GLOBAL $modeDebug;
	if ($modeDebug) {
		return;
	}
	GLOBAL $reportWidth;
	
	$results = <<<EOHTML
					</table>
				</td>
			</tr>
			<tr>
				<td width="$reportWidth" valign='top'>
					<img class='image_fix' src='http://www.unitrends.com/images/notifications/bottom-white.png' alt='Title Area Bottom' title='Title Area Bottom' width="$reportWidth" height='17' />
				</td>
			</tr>
		</table>
EOHTML;

     return $results;
}


function get_client_backups($client, $end, $type) {
	$start = strtotime(date("Y-m-d", $end - 518400));

	$backups = FALSE;
	if ($type != 0) {
		if ($client === NULL) {
			$backups = bp_get_backup_status(array(
				"start_time" => $start,
				"end_time" => $end,
				"type" => array("securesync master", "securesync differential",
					"securesync incremental", "securesync baremetal",
					"securesync localdir")));
		} else {
			$backups = bp_get_backup_status(array(
				"client_id" => $client,
				"start_time" => $start,
				"end_time" => $end,
				"type" => array("securesync master", "securesync differential",
					"securesync incremental", "securesync baremetal",
					"securesync localdir")));
		}
	} else {
		if ($client === NULL) {
			$backups = bp_get_backup_status(array(
				"start_time" => $start,
				"end_time" => $end,
				"type" => array("master", "differential", "incremental", "selective", "baremetal")));
		} else {
			$backups = bp_get_backup_status(array(
				"client_id" => $client,
				"start_time" => $start,
				"end_time" => $end,
				"type" => array("master", "differential", "incremental", "selective", "baremetal")));
		}
	}
	$status_array = array(-1, -1, -1, -1, -1, -1, -1);
	if ($backups !== false) {
		for ($i = 0; $i < count($backups); $i++) {
			$day = date('w', $backups[$i]['start_time']);
			if ($backups[$i]['complete'] == true and $status_array[$day] < $backups[$i]['status']) {
				$status_array[$day] = $backups[$i]['status'];
			}
		}
	}

	return $status_array;
}


function create_client_table($client_status) {
	GLOBAL $tableCellWidth;
	GLOBAL $tableCellHeight;
	
	$results = '';
	for($i=0; $i<7; $i++) {

		switch($client_status[$i]) {
			case -2:
				// case for empty filler rows for application backup tables
				$bgcolor = "#FFFFFF";
				if (date("w") == $i) $bgcolor = getCurrentDayColor();	
				$image = "";
				break;
			case -1:
				$bgcolor = getOffRowColor();
				if (date("w") == $i) $bgcolor = getCurrentDayColor();
				$image = "";
				break;
			case 0:
				$bgcolor = "#007733";
				$image = getGreenImageSrc();
				break;
			case 1:
				$bgcolor = "#FFFF66";
				$image = getYellowImageSrc();
				break;
			case 2:
				$bgcolor = "#EE3333";
				$image = getRedImageSrc();
				break;
		}
		
		$daydiff = $i - date("w");
		switch ($i) {
			case 0:  $loc = 'L'; break; # left end of table
			case 6:  $loc = 'R'; break; # right end of table
			default: $loc = 'C'; break; # neither end of table
		}
		switch ($daydiff) {
			case -1: $hl = 'R';  break; # on left of current day
			case 0:  $hl = 'LR'; break; # on current day
			case 1:  $hl = 'L';  break; # on right of current day
			default: $hl = 'n';  break; # not near current day
		}
		$class = "Cal_{$loc}_{$hl}";

		$results .= '		<td width="' . $tableCellWidth . '" height="' . $tableCellHeight . '" bgcolor="' . $bgcolor . '" class="' . $class . '">';
		if ($image != "") {
            //18478 - Remove gifs from reports
            //$results .= '<img src="' . $image . '" width="' . $tableCellWidth . '" height="' . $tableCellHeight . '" />';
		} else {
			$results .= '&nbsp;';
		}
		$results .= "</td>";
	}

	return $results;
}

?>
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv='Content-Type' content='text/HTML; charset=utf-8' />
<meta name='viewport' content='width=device-width, initial-scale=1.0'/>
<!--<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />-->
<title>System Status Report</title>
<style type="text/css">

body {
	font-family: Arial, Helvetica, sans-serif;
}

.Cal_filler {
	font-size: xx-small;
}

<?php
	# generate table border css: Cal_<L C R>_<LR L R n> for Cal_<location>_<highlight>
	$types = array(
		array('type' => 'Std', 'borders' => 'none', 'topcolor' => '#FFFFFF', 'bottomcolor' => '#FFFFFF'),
		array('type' => 'Hdr', 'borders' => 'solid', 'topcolor' => '#FFFFFF', 'bottomcolor' => '#FFFFFF'),
	);
	$location = array(
		array('loc' => 'L', 'left' => '1px', 'right' => '1px'),
		array('loc' => 'C', 'left' => '1px', 'right' => '1px'),
		array('loc' => 'R', 'left' => '1px', 'right' => '1px'),
	);
	$high = '#333333'; # highlighted day border color
	$low = '#BBBBBB'; # non-highlighted day border color
	$highlight = array(
		array('hl' => 'LR', 'left' => $high, 'right' => $high),
		array('hl' => 'L', 'left' => $high, 'right' => $low),
		array('hl' => 'R', 'left' => $low, 'right' => $high),
		array('hl' => 'n', 'left' => $low, 'right' => $low),
	);
	for ($loc = 0; $loc < 3; $loc++) {
		for ($hl = 0; $hl < 4; $hl++) {
			echo <<<EOHTML
.Cal_{$location[$loc]['loc']}_{$highlight[$hl]['hl']} {
	border-width: 0px {$location[$loc]['right']} 0px {$location[$loc]['left']};
	border-color: $low {$highlight[$hl]['right']} #FFFFFF {$highlight[$hl]['left']};
	border-style: none solid none solid;
	font-size: 12px;
	font-weight: normal;
	color: #000000;
	text-align: center;
}

EOHTML;
		}
	}
	for ($loc = 0; $loc < 3; $loc++) {
		if ($location[$loc]['loc'] == 'L') $lcolor = $low;
		else $lcolor = $high;
#$lcolor = $low;
		echo <<<EOHTML
.Cal_header_{$location[$loc]['loc']} {
	border-width: 1px {$location[$loc]['right']} 1px {$location[$loc]['left']};
	border-color: $low $high $high $lcolor;
	border-style: solid solid solid solid;
	font-size: 12px;
	font-weight: normal;
	color: #000000;
	text-align: center;
}

EOHTML;
	}

	foreach (array('LR', 'n') as $highlight) {
		$color = ($highlight == 'LR' ? $high : $low);
		echo <<<EOHTML
.Cal_footer_$highlight {
	font-size: xx-small;
	border-width: 1px 0px 0px 0px;
	border-color: $color #FFFFFF #FFFFFF #FFFFFF;
	border-style: solid none none none;
}

EOHTML;
	}

?>

		/* Based on The MailChimp Reset INLINE: Yes. */  
		/* Client-specific Styles */
		#outlook a {padding:0;} /* Force Outlook to provide a 'view in browser' menu link. */
		body{width:100% !important; -webkit-text-size-adjust:100%; -ms-text-size-adjust:100%; margin:0; padding:0;} 
		/* Prevent Webkit and Windows Mobile platforms from changing default font sizes.*/ 
		.ExternalClass {width:100%;} /* Force Hotmail to display emails at full width */  
		.ExternalClass, .ExternalClass p, .ExternalClass span, .ExternalClass font, .ExternalClass td, .ExternalClass div {line-height: 100%;}
		/* Forces Hotmail to display normal line spacing.  More on that: http://www.emailonacid.com/forum/viewthread/43/ */ 
		#backgroundTable {margin:0; padding:0; width:100% !important; line-height: 100% !important;}
		/* End reset */

		/* Some sensible defaults for images
		Bring inline: Yes. */
		img {outline:none; text-decoration:none; -ms-interpolation-mode: bicubic;} 
		a img {border:none;} 
		.image_fix {display:block;}

		/* Yahoo paragraph fix
		Bring inline: Yes. */
		p {margin: 1em 0;}

		/* Hotmail header color reset
		Bring inline: Yes. */
		/*h1, h2, h3, h4, h5, h6 {color: black !important;}*/

		a, h2 a, h3 a, h4 a, h5 a, h6 a {color: blue !important;}

		a:active, h2 a:active,  h3 a:active, h4 a:active, h5 a:active, h6 a:active {
		color: red !important; /* Preferably not the same color as the normal header link color.  There is limited support for psuedo classes in email clients, this was added just for good measure. */
		}

		a:visited, h2 a:visited,  h3 a:visited, h4 a:visited, h5 a:visited, h6 a:visited {
		color: purple !important; /* Preferably not the same color as the normal header link color. There is limited support for psuedo classes in email clients, this was added just for good measure. */
		}

		/* Outlook 07, 10 Padding issue fix
		Bring inline: No.*/
		table td {border-collapse: collapse;}

    /* Remove spacing around Outlook 07, 10 tables
    Bring inline: Yes */
    table { border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt; }

		/* Styling your links has become much simpler with the new Yahoo.  In fact, it falls in line with the main credo of styling in email and make sure to bring your styles inline.  Your link colors will be uniform across clients when brought inline.
		Bring inline: Yes. */
		a {color: #E00034;}

		/***************************************************
		****************************************************
		MOBILE TARGETING
		****************************************************
		***************************************************/
		@media only screen and (max-device-width: 480px) {
			/* Part one of controlling phone number linking for mobile. */
			a[href^='tel'], a[href^='sms'] {
						text-decoration: none;
						color: blue; /* or whatever your want */
						pointer-events: none;
						cursor: default;
					}

			.mobile_link a[href^='tel'], .mobile_link a[href^='sms'] {
						text-decoration: default;
						color: orange !important;
						pointer-events: auto;
						cursor: default;
					}

		}

		/* More Specific Targeting */

		@media only screen and (min-device-width: 768px) and (max-device-width: 1024px) {
		/* You guessed it, ipad (tablets, smaller screens, etc) */
			/* repeating for the ipad */
			a[href^='tel'], a[href^='sms'] {
						text-decoration: none;
						color: blue; /* or whatever your want */
						pointer-events: none;
						cursor: default;
					}

			.mobile_link a[href^='tel'], .mobile_link a[href^='sms'] {
						text-decoration: default;
						color: orange !important;
						pointer-events: auto;
						cursor: default;
					}
		}

		@media only screen and (-webkit-min-device-pixel-ratio: 2) {
		/* Put your iPhone 4g styles in here */ 
		}

		/* Android targeting */
		@media only screen and (-webkit-device-pixel-ratio:.75){
		/* Put CSS for low density (ldpi) Android layouts in here */
		}
		@media only screen and (-webkit-device-pixel-ratio:1){
		/* Put CSS for medium density (mdpi) Android layouts in here */
		}
		@media only screen and (-webkit-device-pixel-ratio:1.5){
		/* Put CSS for high density (hdpi) Android layouts in here */
		}
		/* end Android targeting */
		
		/* Yahoo targeting */
		.ReadMsgBody { 
			width: 100%;
		}
		.ExternalClass {
			width: 100%;
		}
		/* end Yahoo targeting */

		.Legend {
			font-size: 14px;
			font-weight: bold;
			text-align: right;
			vertical-align: middle;
		}
		.AppClient {
			font-size: 16px;
			font-weight: bold;
			text-align: right;
		}
		.Client {
			font-size: 14px;
			text-align: right;
		}
		.ServerInstance {
			font-size: 14px;
			font-weight: bold;
			text-align: left;
		}
		.Alert {
		}
		.Alert_Header {
		}
		.Header {
		}
		.Overview_Header {
		}
		.Overview_Title {
		}

	</style>
</head>
<!-- 
$comment
-->

<body style="background-color: #EFEFEF; font-family: Calibri, Candara, Segoe, Segoe UI, Optima, Arial, sans-serif; letter-spacing: 1px; background-image:url('http://www.unitrends.com/templates/unitrends-homepage-bifurcated/images/main-content-bg.jpg');">
<table cellpadding='0' cellspacing='0' border='0' id='backgroundTable'>
	<tr>
		<td valign='middle'> 
<?php

// $argv[1] is appname ("alertman") and $argv[2] is the current logfile
if (isset($argv[1])) {
	bp_bypass_cookie(3, $argv[1]);
}
else {
	bp_bypass_cookie(3);
}
$bpdir = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));
require("$bpdir/bin/app_status_funcs.php");
require("$bpdir/bin/report_images_funcs.php");

ob_start();

$dpu_id = bp_get_local_system_id();
$dpu = bp_get_system_info($dpu_id);
print(dpu_summary_build_title($dpu["name"], $dpu["version"]));

$client_list = bp_get_client_list();
natcasesort($client_list);

end($client_list);
$last_client = key($client_list);
reset($client_list);

$license_info = bp_get_license_info();
$feature_string = $license_info['feature_string'];

$ce = false;
if(substr($feature_string,0,2) === "CE"){
    $ce= true;
}

print(dpu_summary_build_header("File-Level and Bare Metal Backups", "Client", "Backups", "Replication"));

if(!$ce) {

    foreach ($client_list as $client_id => $client_name) {

        if ($client_name == "vCenter-RRC") {
            $vCenterRRC_id = $client_id;
            continue;
        }
        $client_info = bp_get_client_info($client_id, $dpu_id);
        if ($client_info["os_type_id"] == 40 &&
            $client_info["generic_property"] == 0) {
            // Skip Cisco UCS clients
            continue;
        }
        print(dpu_summary_build_row($dpu_id, $client_name, $client_id));

        if ($client_id == $last_client) {
            break;
        } else {
            print(build_filler(1));
        }
    }

    print(build_footer());

    $output = ob_get_contents();
    ob_clean();
}

foreach ($client_list as $client_id => $client_name) {
    if ($client_name == "vCenter-RRC") {
        $vCenterRRC_id = $client_id;
        continue;
    }
}

// The SQL backups section of the report
print(dpu_summary_build_header("Microsoft SQL Server Backups",
			"Client: Server Instance: Database", "SQL Server Backups",
			"Replication"));
$haveBackups = false;
foreach($client_list as $client_id => $client_name) {
	$ret = dpu_summary_sql_backups($dpu_id, $client_name, $client_id);
	if ($ret == true) {
		$haveBackups = true;
	}
}
if ($haveBackups) {
	print(build_footer());
	$output .= ob_get_contents();
}
ob_clean();
// end SQL backups
 
// The Exchange backups section of the report
print(dpu_summary_build_header("Microsoft Exchange Server Backups",
			"Client: DB or Storage Group", "Exchange Server Backups",
			"Replication"));
$haveBackups = false;
foreach($client_list as $client_id => $client_name) {
	$ret = dpu_summary_exch_backups($dpu_id, $client_name, $client_id);
	if ($ret == true) {
		$haveBackups = true;
	}
}
if ($haveBackups) {
	print(build_footer());
	$output .= ob_get_contents();
}
ob_clean();
// end Exchange backups

// The VMware backups section of the report
if (isset($vCenterRRC_id)) {
	print(dpu_summary_build_header("VMware Backups",
			"Server: Virtual Machine", "VMware Backups",
			"Replication"));
	$ret = dpu_summary_vmware_backups($dpu_id, $vCenterRRC_id);
	if ($ret == true) {
		print(build_footer());
		$output .= ob_get_contents();
	}
	ob_clean();
}
// end VMware backups

// The Hyper-V backups section of the report
print(dpu_summary_build_header("Hyper-V Server Backups",
			"Client: Virtual Machine", "Hyper-V Backups",
			"Replication"));
$haveBackups = false;
foreach($client_list as $client_id => $client_name) {
	$ret = dpu_summary_hyperv_backups($dpu_id, $client_name, $client_id);
	if ($ret == true) {
		$haveBackups = true;
	}
}
if ($haveBackups) {
	print(build_footer());
	$output .= ob_get_contents();
}
ob_clean();
// end Hyper-V backups

// The Oracle backups section of the report
print(dpu_summary_build_header("Oracle Server Backups",
			"Client: Database", "Oracle Server Backups",
			"Replication"));
$haveBackups = false;
foreach($client_list as $client_id => $client_name) {
	$ret = dpu_summary_oracle_backups($dpu_id, $client_name, $client_id);
	if ($ret == true) {
		$haveBackups = true;
	}
}
if ($haveBackups) {
	print(build_footer());
	$output .= ob_get_contents();
}
ob_clean();
// end Oracle backups

// The SharePoint backups section of the report
print(dpu_summary_build_header("SharePoint Server Backups",
			"Client: SharePoint Farm", "SharePoint Backups",
			"Replication"));
$haveBackups = false;
foreach($client_list as $client_id => $client_name) {
	$ret = dpu_summary_sharepoint_backups($dpu_id, $client_name, $client_id);
	if ($ret == true) {
		$haveBackups = true;
	}
}
if ($haveBackups) {
	print(build_footer());
	$output .= ob_get_contents();
}
ob_clean();
// end SharePoint backups

// The Cisco UCS backups section of the report
print(dpu_summary_build_header("Cisco UCS Backups",
			"Client: Service Profiles", "Cisco UCS Backups",
			"Replication"));
$haveBackups = false;
foreach($client_list as $client_id => $client_name) {
	$ret = dpu_summary_cisco_ucs_backups($dpu_id, $client_name, $client_id);
	if ($ret == true) {
		$haveBackups = true;
	}
}
if ($haveBackups) {
	print(build_footer(1) . "<br/>");
	$output .= ob_get_contents();
}
ob_clean();
// end Cisco UCS backups

print(build_legend() . "<br/>");
print(dpu_alerts_build_header());

$alert_array = bp_get_alerts("descending", "timestamp", array("resolved" => false));

$array_no = 1;
foreach($alert_array as $alert) {
        $date = date("m-d-Y", $alert["timestamp"]);
        $time = date("h:i:s A", $alert["timestamp"]);

        print(dpu_alerts_build_row($date, $time, $alert["text"], $alert["severity"], $array_no));
		$array_no = $array_no + 1;
}

print (dpu_alerts_build_footer());

print ("</td></tr></table>");

$output .= ob_get_contents();
ob_end_clean();

#$output = preg_replace('/>\s+</', '><', $output);
print $output;

bp_destroy_cookie();
?>
</body>
</html>
