#!/bin/sh
# ----------------------------------------------------------------------------------
# Change log entries
# Bug# Date    Who Description
# ---- ------- --- -----------------------------------------------------------------
# 18234 11Mar14 VSK start_db.sh check if db server is running. start it.

. /etc/default/bp.ini 2> /dev/null

## this is required during reboot since we don't have $PG env vars
[ "x$PGDATA" == "x" ] && . /etc/profile.d/postgresql_envars.sh
PGPIDFILE="$PGDATA/postmaster.pid"
STOPFILE="$PGDATA/db_stopped"

verbose=0

# 18234
## returns 0 if server is up and 1 otherwise
# ----------------------------------------------------------------------------------
function serverUp() {
  
  verbose=${1:-"0"}

  # first try pg_ctl 
  sudo -iu postgres pg_ctl status -D $PGDATA > /dev/null 2>&1
  pg_ctl_status=$?

  [ $verbose == "1" ] && echo pg_ctl_status: $pg_ctl_status

  # then try ps to see if server is up
  pid_pgdata=(`ps --no-headers -o pid,cmd -u postgres | egrep "\-D /" | awk '{print $1,$4}'`)

  pid_status=1
  pgdata_status=1

  ## output of the ps command
  if [ ${#pid_pgdata[@]} -eq 2 ]; then
    ## we have pid and $PGDATA from $PGPIDFILE if server is up
    ## compare PID
    if [ -f $PGPIDFILE ]; then 
      pid1=$(head -n 1 $PGPIDFILE | awk '{print $1}')
      pid2=${pid_pgdata[0]}
      [ $verbose == "1" ] && echo pid: $pid1==$pid2
      [ $pid1 -eq $pid2 ] && pid_status=0
    fi

    ## compare PGDATA
    pgdata1=$PGDATA
    pgdata2=${pid_pgdata[1]}
    [ $verbose == "1" ] && echo pgdata: $pgdata1==$pgdata2
    [ "$pgdata1" == "$pgdata2" ] && pgdata_status=0

  fi


  ([ $pid_status -eq 0 ] || [ $pgdata_status -eq 0 ]) && [ $pg_ctl_status -eq 0 ] && return 0

  return 1
}

function metadataCheck() {

  pgrep "fileDedup|devmonitor" > /dev/null

  if [[ $? -eq 1 ]]; then
    ## Run a quick metadata check.
    ## This should be run before tasker but after the database has started
    ## because it can generate alerts and traps.
    nohup $BPDIR/bin/metadataUtil --check --quick --notifications --wait-on-lock >/dev/null 2>&1 &
  fi
}

# ----------------------------------------------------------------------------------

serverUp
status=$?

if [ $status -eq 0 ]; then
  echo -n `date +%F.%H:%M:%S`"| Postmaster seems to be running. "
  sudo -iu postgres pg_ctl status -D $PGDATA
  metadataCheck
  exit 1
else 
  ## start the server 
  echo -n `date +%F.%H:%M:%S`"| Starting database. " 
  sudo -iu postgres pg_ctl start -D $PGDATA -l $PGDATA/logfile -w | xargs
  if [ $? -eq 0 ]; then
     tail -n20 $STOPFILE >/tmp/dbstop.tmp 2>/dev/null
     cp -f /tmp/dbstop.tmp $STOPFILE
     echo "`date +%F.%H:%M:%S`| database started " >>$STOPFILE
     metadataCheck
  fi
fi

