<?

// Connect to the Postgres Database
# Database connection info
$db_host = 'localhost';
$db_port = '5432'; // Standard is 5432
$db_name = 'bpdb';
$db_user = 'postgres';
$db_pass = 'postgres';

$conn = pg_connect("port=$db_port dbname=$db_name user=$db_user password=$db_pass");

if (!$conn) {
  	echo "A database connection error occurred.\n"; 
	exit(1);
}

if ($argc < 2) {
	echo "**************************************\n";
	echo "No workspace specified\n";
	echo "Try again and add a workspace name to the command\n";
	echo "toggle_workspace.sh <workspace_name>\n\n";
	echo "**************************************\n";
        echo "Here is a list of current workspace names to choose from:\n\n";
        $result = pg_query($conn, "select workspacename from bp.workspace order by workspacename;");
	if ($result) {
   		for ($row = 0; $row < pg_numrows($result); $row++) {
       		  $fullname = pg_result($result, $row, 'workspacename') . " ";
        	  echo ($row + 1) . ": " . $fullname . "\n";
   		}
	} else {
   		echo "Query failed with the following error:\n";
   		echo pg_errormessage($conn);
	} 
	echo "**************************************\n";
	exit(1);
}

$workspace_to_toggle = $argv[1];

$query = "select workspacename, dpuid from bp.workspace where workspacename = '$workspace_to_toggle'";
// When doing a DPU restore from the vault, toggle_vaulting must be run after dumping a
// workspace since unitrends_pgdump.php set isvaulted=TRUE in bp.workspace and creates
// triggers on the workspace tables to enable vaulting.  We don't want the workspace in
// this state on the vault because mirrored records arriving from the DPU will trigger
// copies into dbmirror tables that continuously accumulate and never get cleaned out.
// It is possible that there are multiple workspaces with the same name on the vault,
// in which case we must also have a dpuid to uniquely identify the schema name.
if (isset($argv[2])) {
	$query .= " and dpuid = '{$argv[2]}'";
}
$result = pg_query($conn, $query);

$nrows = pg_numrows($result);
if (!$nrows) {
	echo "**************************************\n";
	echo "$workspace_to_toggle not found\n\n";
	echo "Try again and add a workspace name to the command\n";
	echo "toggle_workspace.sh <workspace_name>\n";
	echo "OR\n";
	echo "toggle_workspace.sh (will give a list of workspaces)\n\n";
	echo "**************************************\n";
	exit(1);
} 

// Require a dpuid arg to be specified if needed.
if ($nrows > 1) {
	echo "There are $nrows workspaces named $workspace_to_toggle.\n";
	echo "Please specify a DPU id:  toggle_workspace.sh <workspace_name> <dpuid>\n";
   	while ($row = pg_fetch_assoc($result)) {
		echo "  " . $row['workspacename'] . " has DPU id " . $row['dpuid'] ."\n";
	}
	exit(1); 
}

$dpuid = pg_result($result, 0 , 'dpuid');

$result = pg_query($conn, "select bpexch.getworkspaceschema('" . $workspace_to_toggle . "','" . $dpuid . "')  as full_name ;");
$fullworkspace_name = pg_result($result, 0 , 'full_name');

// Begin the transaction
$result = pg_query($conn, "BEGIN;");
if (!$result) {
	echo "An error occurred beginning the transaction.\n";
	exit(1);
}

$query = "SELECT isvaulted::int FROM bp.workspace WHERE workspacename = '$workspace_to_toggle' and dpuid = '$dpuid';";
$result = pg_query($conn, $query);
$workspace_vaulting_enabled = pg_result($result, 'isvaulted');

if ($workspace_vaulting_enabled == TRUE) {
	$enable = "false";
	$state = "disabled";
	$action = "disabling";
}
else {
	$enable = "true";
	$state = "enabled";
	$action = "enabling";
}

$result = pg_query($conn, "set search_path to \"$fullworkspace_name\";");
if (!$result) {
	echo "An error occurred setting search path to $fullworkspace_name.\n";
	exit(1);
}

$result = pg_query($conn, "SELECT bpexch.setdbmirrortriggers($enable);");
if (!$result) {
	echo "An error occurred while $action triggers on the $workspace_to_toggle tables.\n";
	pg_query($conn, "ROLLBACK;");
	exit(1);
}

$query = "UPDATE bp.workspace set isvaulted = '$enable' where workspacename = '$workspace_to_toggle' and dpuid = '$dpuid'";
$result = pg_query( $conn, $query );
if (!$result) {
	echo "An error occurred while $action vaulting for $workspace_to_toggle.\n";
	pg_query($conn, "ROLLBACK;");
	exit(1);
}
else { 
	echo "Vaulting is now $state.\n\n";
}

pg_query($conn, "COMMIT;");
pg_close($conn);
exit(0);

?>

