#!/bin/sh -u

# Consumers of this script show stderr as log/debug output, and stdout as
# either parsable output (on success, 0 return code) or an error string.
DUP="tee -a /dev/stderr"

declare -a scachedevs
declare -a scache

# Retrieves a setting from bp.archive_device_settings, printing the value to
# stdout.
# @param $1  serial number of device to retrieve setting for
# @param $2  setting name to retrieve
# @return  0 on success, 1 on failure
#TODO could get list of key=>value settings and cache for each serial, either
# in an array or just big string like:
#  serial key value
#  serial key value
get_setting()
{
	serial="$1"
	setting="$2"
#return 0

	devname=$(psql bpdb -U postgres -tAq -c "SELECT name FROM bp.archive_devices WHERE serial = '$serial'")
	if [ $? -ne 0 ]; then
		echo "Failed to retrieve name for device with serial $serial" | $DUP
		exit 1
	fi

	read -d '' script <<EOD
bp_bypass_cookie(2);
\$s = bp_get_archive_media_settings('$devname');
if (\$s === false) {
    echo bp_error();
    exit(1);
} else {
    foreach (\$s as \$set) {
        if (\$set['name'] == '$setting') {
            echo \$set['value'];
            exit(0);
        }
    }
    echo "Value for setting '$setting' not found\\n";
    exit(1);
}
EOD
#echo "$script" > /tmp/output
	value=$(php -r "$script")
	if [ $? -ne 0 ]; then
		echo "Failed to retrieve setting \"$setting\" for device" | $DUP
		exit 1
	else
		echo "$value"
		return 0
	fi
}
get_setting_broken()
{
	serial="$1"
	setting="$2"

	devname=$(psql bpdb -U postgres -tAq -c "SELECT name FROM bp.archive_devices WHERE serial = '$serial'")
	if [ $? -ne 0 ]; then
		echo "Failed to retrieve name for device with serial $serial" | $DUP
		exit 1
	fi

	devidx=-1
	for i in $(seq 0 $((${#scachedevs[@]} - 1))); do
		if [ ${scachedevs[$i]} = "$devname" ]; then
			devidx=$i
			break
		fi
	done

	if [ $devidx -eq -1 ]; then
		# This is the first time settings for this device were requested. We
		# cache the settings so that we don't call the API to get settings
		# multiple times, which keeps the uarchive log cleaner and easier to
		# debug.  PHP's serialize() function can store arbitrary characters,
		# so encode them to avoid shell interactions below when we retrieve
		# the settings from the "cache".
		read -d '' script <<EOD
bp_bypass_cookie(2);
\$s = bp_get_archive_media_settings('$devname');
if (\$s === false) {
    echo bp_error();
    exit(1);
} else {
	echo base64_encode(serialize(\$s));
}
EOD
		str=$(php -r "$script")
		if [ $? -ne 0 ]; then
			echo "Failed to retrieve setting \"$setting\" for device" | $DUP
			exit 1
		else
			devidx=${#scachedevs[@]}
			scache[$devidx]="$str"
			scachedevs[$devidx]="$devname"
			echo "Stored settings for device $devname in index $devidx" 1>&2
		fi
	fi

	# Retrieve the cached setting
	str="${scache[$devidx]}"
	read -d '' script <<EOD
	\$s = unserialize(base64_decode('$str'));
    foreach (\$s as \$set) {
        if (\$set['name'] == '$setting') {
            echo \$set['value'];
            exit(0);
        }
    }
    echo "Value for setting '$setting' not found\\n";
    exit(1);
EOD
#echo "$script" > /tmp/output
	value=$(php -r "$script")
	if [ $? -ne 0 ]; then
		echo "Failed to retrieve setting \"$setting\" for device" | $DUP
		exit 1
	else
		echo "$value"
		return 0
	fi
}

# Determines if a given tape bit is set, e.g. ONLINE or EOF.
# Returns 0 if bit is set (same as /bin/true), 1 if not
tape_bit_set()
{
	devnode="$1"
	bit="$2"
	mt=$(mt -f $devnode status | grep -A1 'General status bits' | tail -1)
	if [ $? -ne 0 ]; then
		echo "Could not query tape drive $devnode status" | $DUP
		exit 1
	fi
        uname -r | grep -q el5
        if [ $? -eq 0 ]; then
		[[ "$mt" =~ "\<$bit\>" ]]
        else
		[[ "$mt" =~ "$bit" ]]
        fi
	return $?
}


# Scans for tape devices on centOS6
tape_scan_os6()
{
	for devpath in $(find /sys/class/scsi_generic -regex '.*/sg[0-9]*'); do
		dev=$(basename $devpath)
		#tape_path=$(find $devpath/device -regex 'scsi_tape:nst[0-9]*')
		tape_path=$(find $devpath/device/scsi_tape -name 'nst[0-9]' 2>/dev/null)
		if [ -n "$tape_path" ]; then
			dev=$(basename "$tape_path" | cut -d: -f2)
		fi
		#devinfo=$(udevinfo -q all -n /dev/$dev 2>/dev/null)
		devinfo=$(udevadm info -q all -n /dev/$dev 2>/dev/null)
		[ $? -eq 0 ] || continue
		# host:channel:id:lun
		scsi_id=$(basename $(readlink $devpath/device))
		vendor=$(echo "$devinfo" | grep "ID_VENDOR=" | cut -d'=' -f2)
		model=$(echo "$devinfo" | grep "ID_MODEL=" | cut -d'=' -f2)
		serial=$(echo "$devinfo" | grep "ID_SERIAL=" | cut -d'=' -f2)
		if echo "$devinfo" | grep -q changer; then
			type="changer"
		#elif tpath=$(ls -d $devpath/device/scsi_tape:nst[0-9] 2>/dev/null); then
		elif tpath=$(ls -d $devpath/device/scsi_tape/nst[0-9] 2>/dev/null); then
			type="tape"
			# Tape device serials come from `tapeinfo`, not udev
			gen_dev=$(basename $(ls -d $devpath/device/scsi_generic/sg*) | cut -d: -f2)
			temp_sn=$(tapeinfo -f /dev/$gen_dev | grep -a '^SerialNumber')
			serial=$(echo $temp_sn | sed "s/.*'\(.*\)'/\1/")
#			dev=$(basename "$tpath" | cut -d: -f2)
		else
			continue
		fi

		serial=$(echo $serial | sed 's/ //g')
		echo "$type|scsi|$scsi_id|$vendor|$model|$serial|/dev/$dev"
	done
}


# Scans for tape devices on centOS5
tape_scan_os5()
{
        for devpath in $(find /sys/class/scsi_generic -regex '.*/sg[0-9]*'); do
                dev=$(basename $devpath)
                #tape_path=$(find $devpath/device -regex 'scsi_tape:nst[0-9]*')
                tape_path=$(find $devpath/device/ -name 'scsi_tape:nst[0-9]')
                if [ -n "$tape_path" ]; then
                        dev=$(basename "$tape_path" | cut -d: -f2)
                fi
                devinfo=$(udevinfo -q all -n /dev/$dev 2>/dev/null)
                [ $? -eq 0 ] || continue
                # host:channel:id:lun
                scsi_id=$(basename $(readlink $devpath/device))
                vendor=$(echo "$devinfo" | grep "ID_VENDOR=" | cut -d'=' -f2)
                model=$(echo "$devinfo" | grep "ID_MODEL=" | cut -d'=' -f2)
                serial=$(echo "$devinfo" | grep "ID_SERIAL=" | cut -d'=' -f2)
                if echo "$devinfo" | grep -q changer; then
                        type="changer"
                elif tpath=$(ls -d $devpath/device/scsi_tape:nst[0-9] 2>/dev/null); then
                        type="tape"
                        # Tape device serials come from `tapeinfo`, not udev
                        gen_dev=$(basename $(ls -d $devpath/device/scsi_generic:sg*) | cut -d: -f2)
                        temp_sn=$(tapeinfo -f /dev/$gen_dev | grep -a '^SerialNumber')
                        serial=$(echo $temp_sn | sed "s/.*'\(.*\)'/\1/")
#                       dev=$(basename "$tpath" | cut -d: -f2)
                else
                        continue
                fi

                serial=$(echo $serial | sed 's/ //g')
                echo "$type|scsi|$scsi_id|$vendor|$model|$serial|/dev/$dev"
        done
}

tape_scan()
{
	os_version=`uname -r | sed -e "s/.*el/el/" -e "s/\..*//" -e "s/.//4g"`
	if [ "$os_version" == "el5" ]; then
		tape_scan_os5
	elif [ "$os_version" == "el6" ]; then
		tape_scan_os6
	else
		echo "could not recognize os version: $os_version"
	fi
}

# Query for information about a tape or changer device:
# @param $1 - serial number of tape or changer device
# @param $2 - required for changers, the serial number of changer's tape drive
# @param $3 - required for changers, comma-separated slot list
# Prints:
#  connected|devnode|parentdevnode|hastape|initialized|label|hasarchives
#   0 or 1           changers only  0 or 1   0 or 1            0 or 1
tape_info()
{
	if [ $# -lt 1 ]; then
		echo "Drive serial number required" | $DUP
		exit 1;
	fi
	serial="$1"
	parentserial=
	slots=

	# Output fields, initialized to their defaults
	connected=0
	devnode=
	parentdevnode=
	hastape=0
	initialized=0
	label=
	hasarchives=0

	subserial=
	slots=
	if [ $# -ge 3 ]; then
		subserial="$2"
		slots="$3"
	fi
		
	# Sets $connected.  If $connected=1, sets $type and $devnode
	# If $type==changer, sets $parentdevnode, $serial, $parentserial
	get_connected_device "$serial" "$subserial" "$slots"

	# Initialize the devices.  If initialization fails, exit, so that this
	# script won't ever say a device is connected without initializing first.
	if [ "$connected" -eq 1 ]; then
		# Initialize the devices
		if [ "$type" = "changer" ]; then
			init=$(get_setting "$parentserial" init_cmd)
			[ $? -eq 0 ] || exit 1
			if [ -n "$init" ]; then
				init="changerdev='$parentdevnode'; $init"
				echo "Executing changer init_cmd: $init" 1>&2
				eval "($init) 1>&2"
				if [ $? -ne 0 ]; then
					echo "Failed to execute changer init_cmd" | $DUP
					exit 1
				fi
				sleep 1
			fi
		fi
		init=$(get_setting "$serial" init_cmd)
		[ $? -eq 0 ] || exit 1
		if [ -n "$init" ]; then
			init="tapedev='$devnode'; $init"
			echo "Executing tape init_cmd: $init" 1>&2
			eval "($init) 1>&2"
			if [ $? -ne 0 ]; then
				echo "Failed to execute tape init_cmd" | $DUP
				exit 1
			fi
			sleep 1
		fi
	fi

	# See if a tape is present
	if [ "$connected" -eq 1 ]; then
		if [ "$type" = "changer" ]; then
			# Sets $hastape
			#load_first_tape "$parentdevnode" "$parentserial" "$serial" "$slots"
			# Consider changers to never have a tape loaded
			hastape=0
		else
			hastape=0
			tape_bit_set "$devnode" "ONLINE" && hastape=1
		fi
	fi

	# Sets $initialized, $label, $hasarchive
	[ "$hastape" -eq 1 ] && get_tape_init "$devnode" 

	echo "$connected|$devnode|$parentdevnode|$hastape|$initialized|$label|$hasarchives"
}

# Looks for a connected device and gets device properties.
# Sets $connected.  If $connected=1, sets $type and $devnode
# If $type==changer, sets $parentdevnode
# @param $1  device serial
# @param $2  tape device serial, if device is a changer
# @param $3  changer slot list, if device is a changer
get_connected_device()
{
	serial="$1"
	subserial="$2"
	slots="$3"

	# Output of scan
	devs=$(tape_scan)

	echo "Looking for device with serial $serial" 1>&2

	#dev=$(echo "$devs" | awk -F'|' '{ if ($6 == "'$serial'") print }')
	dev=$(echo "$devs" |sort | awk -F'|' '{ if ($6 == "'$serial'") print }')
	dev=$(echo $dev | awk '{ print $1 }' )
	if [ -z "$dev" ]; then
		# Device not found, assume it's been disconnected
		echo "Device not connected" 1>&2
		return 0
	fi
	type=$(echo "$dev" | cut -d'|' -f1)

	echo "Device type: $type" 1>&2
	if [ "$type" = "changer" ]; then
		# Require 3 arguments for changers
		if [ -z "$subserial" ] || [ -z "$slots" ]; then
			echo "Tape drive serial number and slot list required for changer devices" | $DUP
			exit 1;
		fi
		parentserial="$serial"
		serial="$2"
		slots="$3"
		echo "Looking for tape drive with serial $serial" 1>&2
		echo "Using changer slots: $slots" 1>&2
		parentdev="$dev"
		#dev=$(echo "$devs" | awk -F'|' '{ if ($6 == "'$serial'") print }')
		dev=$(echo "$devs" |sort |awk -F'|' '{ if ($6 == "'$serial'") print }')
		dev=$(echo $dev | awk '{ print $1 }' )
		if [ -z "$dev" ]; then
			# Tape device not found, assume it's been disconnected
			echo "Changer's tape device not connected" 1>&2
			return 0
		fi
		devnode=$(echo "$dev" | cut -d'|' -f7)
		parentdevnode=$(echo "$parentdev" | cut -d'|' -f7)
	elif [ "$type" = "tape" ]; then
		devnode=$(echo "$dev" | cut -d'|' -f7)
	else
		echo "Invalid device type: $type" | $DUP
		exit 1
	fi

	# Both are connected
	connected=1
	echo "Device connected: $devnode $parentdevnode" 1>&2
	return 0
}

# Try to read a proper blocksize from the tape file.  As long as we read an
# equal or larger blocksize than what was written and read as much data as
# requested, we succeed.
read_tape_file()
{
	devnode="$1"
	tmpfile="$2"
	request="$3"

	# Start at 64MB blocks and work our way down.  By doing this rather than
	# starting from smaller blocks, the tape won't move with the bad reads
	# because the driver will reject sizes that are too large.
	bs=$((1024*1024*64))
	bsmin=512
	while [ $bs -ge $bsmin ]; do
		echo "Trying blocksize of $bs" 1>&2
		count=$(echo "($request + $bs - 1) / $bs" | bc | xargs printf "%.0f")
#mt -f $devnode rewind
#mt -f $devnode status
		set -o pipefail
		output=$(dd if=$devnode of=$tmpfile bs=$bs count=$count 2>&1)
		if [ $? -eq 0 ]; then
			# We successfully read data
			set +o pipefail
			return 0
		fi
		set +o pipefail
		# EINVAL indicates an incompatible blocksize; sleep for all other errors
#		echo "$output" | grep -qi 'Invalid argument'
#		[ $? -eq 0 ] || sleep 1
		((bs=$bs/2))
	done
	return 1
}

# Updates $initialized, $label, $hasarchive flags to non-zero/non-empty for
# devnode $1
get_tape_init()
{
	devnode="$1"

	echo "Rewinding tape using 'mt -f $devnode rewind'" 1>&2
	mt -f $devnode rewind 1>&2
	if [ $? -ne 0 ]; then
		echo "Could not rewind tape drive $devnode" 1>&2
		return 0
	fi
	sleep 1

	tmpfile=/tmp/ua_tape.$$

	echo "Reading tape label" 1>&2
	read_tape_file $devnode $tmpfile 16384
	# Read up to 16K of first file
	if [ $? -ne 0 ]; then
		echo "Tape label not found" 1>&2
		return 0
	fi

	lmagic="$(dd if=$tmpfile bs=1 count=8)"
	if [ "$lmagic" != "uArchive" ]; then
		echo "First file on tape not a uArchive label" 1>&2
		rm -f $tmpfile 2>/dev/null
		return 0
	fi
	echo "Found uArchive label on tape" 1>&2

	revision=$(cat $tmpfile | grep '^label_revision=' | cut -d'=' -f2-)
	if [ -z "$revision" ]; then
		echo "No uArchive tape label revision found" 1>&2
		rm -f $tmpfile 2>/dev/null
		return 0
	elif [ "$revision" -gt 1 2>/dev/null ]; then
		# We don't understand this label format
		echo "uArchive tape label revision too new for this software version (max=1, rev=$revision)" | $DUP
		rm -f $tmpfile 2>/dev/null
		exit 1
	fi
	echo "Valid uArchive label found (revision $revision)" 1>&2
	initialized=1
	label=$(cat $tmpfile | grep -a '^user_label=' | cut -d'=' -f2-)
	rm -f $tmpfile 2>/dev/null

	# Sleep between drive read and query
	sleep 1
	if ! tape_bit_set "$devnode" "EOF"; then
		mt -f $devnode fsf 1
		if [ $? -ne 0 ]; then
			echo "Could not space to end of file on tape" | 1>&2
			return 0
		fi
		sleep 1
	fi
	echo "Now at:" 1>&2
	mt -f $devnode status 1>&2

	echo "Reading archive profile" 1>&2
	# Read up to 16K of profile
	dd if=$devnode of=$tmpfile bs=16384 count=1
	if [ $? -ne 0 ]; then
		echo "Archive profile not found" 1>&2
		return 0
	fi

	# Don't bother trying to validate the profile, just check root element
	if grep -q '<archive_set>' $tmpfile; then
		echo "Archive profile found on tape" 1>&2
		hasarchives=1
	else
		echo "Second file on tape not an archive profile" 1>&2
	fi
	rm -f $tmpfile 2>/dev/null
	
	return 0
}


# Returns slot information for the specified changer:
#  loaded_tape|loaded_slots|slot_count
# Where loaded_tape is the slot number in the changer's specified drive
#       loaded_slots are comma-delimited slot numbers, incl. loaded tape
#       slot_count is the number of slots in the changer, e.g. 16
# @param $1  changer serial
# @param $2  tape drive serial
slot_info()
{
	if [ $# -lt 2 ]; then
		echo "Changer serial and drive serial must be supplied" | $DUP
		exit 1
	fi
	parentserial="$1"
	serial="$2"

	# Output of scan
	devs=$(tape_scan)

	#echo "Looking for device with serial $parentserial" 1>&2
	dev=$(echo "$devs" |sort | awk -F'|' '{ if ($6 == "'$parentserial'") print }')
	dev=$(echo $dev | awk '{ print $1 }' )

	dev=$(echo "$devs" | awk -F'|' '{ if ($6 == "'$parentserial'") print }')
	if [ -z "$dev" ]; then
		# Device not found, assume it's been disconnected
		echo "Device with serial $parentserial is not connected" 1>&2
		echo "||"
		exit 0
	fi
	type=$(echo "$dev" | cut -d'|' -f1)
	parentdevnode=$(echo "$dev" | cut -d'|' -f7)
	if [ "$type" != "changer" ]; then
		echo "Device with serial $parentserial is not a changer" | $DUP
		exit 1
	fi

	# Get mtx output to scan for slots
	echo "Trying 'mtx -f $parentdevnode status' command" 1>&2
	mtx=$(mtx -f $parentdevnode status)
	if [ $? -ne 0 ]; then
		# Changer might need to re-initialize
		mtx -f $parentdevnode inquiry >/dev/null 2>&1
		for (( i=0; i < 6; i++)); do
			sleep 5
			echo "Retrying command" 1>&2
			mtx=$(mtx -f $parentdevnode status)
			status=$?
			if [ $status -eq 0 ]; then break; fi
		done
		if [ $status -ne 0 ]; then
			echo "Could not read changer slot list" | $DUP
			exit 1
		fi
	fi

	# Got mtx output, now see which drive we're querying
	echo "mtx output:\n$mtx\n" 1>&2

	driveno=$(get_setting "$serial" parent_changer_driveno)
	[ $? -eq 0 ] || exit 1
	echo "Using changer tape drive number $driveno" 1>&2

	# See which tape, if any, is loaded in the drive
	loaded=$(echo "$mtx" | sed -nr 's/Data Transfer Element '$driveno':Full.*\(Storage Element ([0-9]+) Loaded\)/\1/p')
	if [ -z "$loaded" ]; then
		# It's possible for an "Unknown" element to be loaded
		loaded=$(echo "$mtx" | sed -nr 's/Data Transfer Element '$driveno':Full.*\(Unknown Storage Element Loaded\)/-1/p')
	fi
	# Make sure it's a number
	if [ -n "$loaded" ] && [ "$loaded" -ne "$loaded" 2>/dev/null ]; then
		loaded=
	fi

	# How many slots are there?
	max=$(echo "$mtx" | sed -nr 's/.* ([0-9]+) Slots.*/\1/p')
	# Make sure it's a number
	if [ -z "$max" ] || [ "$max" -ne "$max" 2>/dev/null ]; then
		echo "Could not determine changer slot count" | $DUP
		exit 1
	fi

	# Scan mtx output to see which slots are loaded, they're numbered from 1
	occupied=
	for i in $(seq 1 $max); do
		echo "$mtx" | grep -q "Storage Element $i:Full"
		if [ $? -eq 0 ]; then
			echo "Slot $i occupied" 1>&2
			[ -n "$occupied" ] && occupied="$occupied,"
			occupied="$occupied$i"
		fi
	done

	echo "$loaded|$occupied|$max"
	exit 0
}


case $1 in
	'scan')
		tape_scan
		;;
	'info')
		shift;
		tape_info $@
		;;
	'slots')
		shift;
		slot_info $@
		;;
esac

exit 0;

