#!/bin/sh
#    ucs_restorefilter.sh <backup dir> 
#  Parse each UCS XML file to filter out attributes that would not restore.
#  Use the UCSM-IN.xsd schema to find the attributes to keep.
#  09/29/2014 acress - handle identRequestEp also
#  10/06/2014 acress - v1.3 handle lsbootSanCatSanImagePath:vnicName
#  10/16/2014 acress - v1.4 handle 'has not dn' without aborting
ver=1.4
xmldir=$1
eval `grep BPDIR /etc/default/bp.ini`
xsdin=$BPDIR/schema/UCSM-IN.xsd
log=$BPDIR/logs.dir/ucs_restorefilter.log
tmpclass=/tmp/xmlclass.tmp$$
tmpprop=/tmp/xmlprop.tmp$$
tmpattr=/tmp/xmlattr.tmp$$
tmpkeep=/tmp/xmlkeep.tmp$$
tmpdel=/tmp/xmldel.tmp$$
tmpro=/tmp/xmlro.tmp$$
tmpa=/tmp/a.tmp$$
tmpd=/tmp/d.tmp$$
tmpf=/tmp/f.tmp$$
tmpg=/tmp/g.tmp$$
tmpx=/tmp/x.tmp$$
tmpx2=/tmp/x2.tmp$$
tmpskip=/tmp/skip.tmp$$
tmpname=/tmp/name.tmp$$
tmpname0=/tmp/name0.tmp$$
proplist=/tmp/xmllist.tmp$$
debug=0

remove_attrs()
{
   # param:  eclass=$1
   # global input vars:  $ln, $tmpf, $tmpattr, $xmlout, $log
   eclass=$1
   grep -q $eclass $xmlout
   if [ $? -eq 0 ]; then
      echo "found class: [$eclass] in $xmlout" | tee -a $log
      grep "<$eclass " $xmlout |sed -e "s/<$eclass / /" |sed -e 's%/>%%' |sed -e 's/" /"\n/g' >$tmpf
      grep "^$eclass" $tmpattr  >$tmpkeep 2>/dev/null
      # make sure these attrs are kept (dn not in schema for lsServer)
      echo "${eclass}:dn   add_keep" >>$tmpkeep
      echo "${eclass}:rn   add_keep" >>$tmpkeep
      echo "${eclass}:descr  add_keep" >>$tmpkeep
      # do not always include name attr, see $tmpattr/$tmpkeep
      echo $eclass |grep Pool >/dev/null 2>&1
      if [ $? -eq 0 ]; then
         ln=
      fi 
      if [ $debug -eq 1 ]; then
         grep -q "${eclass}" $tmpname 
         if [ $? -eq 0 ]; then
	    iclass=`grep "${eclass}" $tmpname |awk '{ print $1 }'`
            echo "  found naming for $eclass:$iclass" >>$log
         fi
      fi
      # check for malformed xml header, correct it.
      sed -i -e '1s/?xml ?/?xml version="1.0"?/' $xmlout
      while read line
      do
         obj=`echo $line |awk '{ print $1 }' |cut -f1 -d'='`
         grep -q "${eclass}:${obj}" $tmpkeep 
         if [ $? -ne 0 ]; then
            echo "  remove attribute $eclass:$obj (line=$ln)" >> $log
            sed -i -e "${ln}s/ $obj=\"[^\"]*\"/ /" $xmlout 
         fi
         if [ $debug -eq 1 ]; then
            grep -q "${eclass}:${obj}" $proplist 
            if [ $? -eq 0 ]; then
               echo "  found $eclass:$obj in proplist" >>$log
            fi
         fi
      done <$tmpf
   fi
}

remove_class()
{
   # params:  eclass=$1 xmlf=$2, also uses $tmpd 
   eclass=$1
   xmlf=$2
   grep -q $eclass $xmlf
   if [ $? -eq 0 ]; then
      echo "removed class: [$eclass] from $xmlf" | tee -a $log
      xmlstarlet ed -d "//$eclass" $xmlf >$tmpd 2>&1
      if [ $? -eq 0 ]; then
         mv -f $tmpd $xmlf
      fi
   fi
}

remove_1attr()
{
   fclass=$1
   iclass=$2
   attr=$3
   # there can be more than one, so match both iclass and attr
   sed -i -E "s/(<${iclass}.+) ${attr}=\"[^\"]*\"(.+)/\1 \2/" $xmlout
   echo "  remove attribute $attr from $fclass:$iclass"| tee -a $log
}

function cleantmp()
{
   rm -f $tmpclass $tmpprop $tmpattr $tmpkeep $tmpdel $tmpro 2>/dev/null
   rm -f $tmpa $tmpd $tmpf $tmpg $tmpx $tmpx2 2>/dev/null
   rm -f $tmpskip $tmpname $tmpname0 $proplist 2>/dev/null
}

echo "`date` -- $ver started $*" >$log
if [ "x$xmldir" = "x" ]; then
   echo "Usage $0 <XML orig directory>" | tee -a $log
   exit 1
fi 

cat - <<EOF >$tmpskip
sys.orig
fabric.orig
EOF

# check for operations classes from UCSM-IN.xsd 
xmlstarlet sel -t -m '//moClass' -v @value -n $xsdin  >$tmpx
xmlstarlet sel -t -m '//access' -v @value -n $xsdin  >$tmpx2
paste $tmpx $tmpx2 > $tmpclass 
xmlstarlet sel -t -m '//moProperty' -v @value -n $xsdin  >$tmpx
xmlstarlet sel -t -m '//externAccess' -v @value -n $xsdin  >$tmpx2
paste $tmpx $tmpx2 > $tmpprop
sort -u $tmpx >$proplist
# delete classes with access: operations or admin
# access types: operations, admin, read-only
cat - >$tmpx <<EOF
operations
EOF
fgrep -f $tmpx $tmpclass |sort -u >$tmpdel
fgrep 'read-only' $tmpclass |sort -u >$tmpro
# delete Pooled classes, only restorable with read-only privileges, not needed
# note that AuthProfiles cannot get the password field and fail, so skip them
cat - >>$tmpdel <<EOF
computePooled
computePooledRackUnit
computePooledSlot
fabricPooledVlan
ippoolPooled
iqnpoolPooled
macpoolPooled
uuidpoolPooled
vnicIpV4PooledAddr
vnicIPv4PooledIscsiAddr
vnicIPv4PooledIscsiAddr
lsServerAssocCtx
iscsiAuthProfile
aaaEpAuthProfile
EOF
# classes where the name attribute cannot be changed
cat - >>$tmpname0 <<EOF
computePsuPolicy
lsBinding
lsComputeBinding
lsRequirement
lsTier
lsbootDef
lsbootIScsi
lsbootIScsiImagePath
lsbootItem
lsbootADef
lsbootPolicy
lsbootCategory
lsbootStorage
lsbootLan
lsbootVirtualMedia
lsbootImage
lsbootRemoteImage
lsbootSanImage
lsbootLocalImage
lsbootLocalStorage
lsbootImagePath
lsbootLanImagePath
lsbootSanImagePath
lsbootSanCatSanImagePath 
EOF
fgrep '	naming' $tmpprop |sort -u >$tmpname

# check for readWrite attributes from UCSM-IN.xsd 
# access: readWrite, readOnly, createOnly
xmlstarlet sel -t -m '//moProperty' -v @value -n $xsdin  >$tmpx
xmlstarlet sel -t -m '//externAccess' -v @value -n $xsdin  >$tmpx2
paste $tmpx $tmpx2 > $tmpattr 
# add some explicit attributes to keep
cat - <<EOF >>$tmpattr
faultPolicy:clearAction   add_keep
faultPolicy:clearInterval add_keep
faultPolicy:descr         add_keep
faultPolicy:flapInterval  add_keep
faultPolicy:name        add_keep
faultPolicy:retentionInterval   add_keep
faultPolicy:sizeLimit   add_keep
computePsuPolicy:descr  add_keep
computePsuPolicy:redundancy   add_keep
computeServerDiscPolicy:action    add_keep
computeServerDiscPolicy:qualifier    add_keep
computeServerDiscPolicy:scrubPolicyName  add_keep
callhomePeriodicSystemInventory:adminState    add_keep
callhomePeriodicSystemInventory:intervalDays    add_keep
callhomePeriodicSystemInventory:maximumRetryCount    add_keep
callhomePeriodicSystemInventory:minimumSendNowIntervalSeconds    add_keep
callhomePeriodicSystemInventory:pollIntervalSeconds    add_keep
callhomePeriodicSystemInventory:retryDelayMinutes    add_keep
callhomePeriodicSystemInventory:sendNow    add_keep
callhomePeriodicSystemInventory:timeOfDayHour    add_keep
callhomePeriodicSystemInventory:timeOfDayMinute    add_keep
adaptorFcErrorRecoveryProfile:fcpErrorRecovery   add_keep
adaptorFcErrorRecoveryProfile:linkDownTimeout   add_keep
adaptorFcErrorRecoveryProfile:portDownIoRetryCount   add_keep
adaptorFcErrorRecoveryProfile:portDownTimeout   add_keep
adaptorFcWorkQueueProfile:ringSize  add_keep
adaptorFcRecvQueueProfile:ringSize  add_keep
adaptorFcPortProfile:ioThrottleCount   add_keep
adaptorFcPortProfile:lunsPerTarget  add_keep
adaptorFcInterruptProfile:mode      add_keep
adaptorEthCompQueueProfile:count    add_keep
firmwareComputeHostPack:mode        add_keep 
firmwareComputeHostPack:ignoreCompCheck  add_keep 
firmwareComputeHostPack:stageSize      add_keep 
firmwareComputeHostPack:updateTrigger  add_keep 
computePoolingPolicy:qualifier     add_keep
computeBladeInheritPolicy:qualifier  add_keep
computeBladeInheritPolicy:dstDn    add_keep
computeAutoconfigPolicy:qualifier  add_keep
computeAutoconfigPolicy:dstDn      add_keep
solPolicy:speed    add_keep
EOF

pushd $xmldir
for xmlfile in `find . -name '*.orig' -print |sort -u`
do

   xmldone=0
   if [ ! -f $xmlfile ]; then
      echo "Error: $xmlfile not found" | tee -a $log
      pwd >>$log
      ls  >>$log 
      cleantmp
      exit 1
   fi
   basefn=`basename $xmlfile`
   grep -q $basefn $tmpskip 
   if [ $? -eq 0 ]; then
      echo "skipping $xmlfile" | tee -a $log
      rm -f $xmlfile 
      continue
   fi
   xmlout=`echo ${xmlfile} | sed -e 's%orig$%xml%' `
   
   nlines=`cat $xmlfile |wc -l`
   # The primary class name is right after <inConfig>
   fline=`grep "<inConfig>" -A 1 $xmlfile |tail -n1`
   fclass=`echo $fline | awk '{ print $1 }'| cut -f2 -d'<'`
   echo "Input  XML: $xmlfile  ($nlines lines)" | tee -a $log
   echo "Output XML: $xmlout  (class=$fclass)" | tee -a $log
   
   ln=`grep -m1 -n "<${fclass}" $xmlfile | cut -f1 -d':'`
   if [ "x$fclass" = "x" ]; then
      echo "Found empty XML class ($fclass) in $xmlfile" | tee -a $log
      cleantmp
      exit 1
   fi 
   grep -q "^$fclass" $tmpattr  2>/dev/null
   if [ $? -ne 0 ]; then
      echo "$xmlfile: schema info not found for $fclass, try next" >> $log
      # try again with next line
      fline=`grep "<inConfig>" -A 2 $xmlfile |tail -n1`
      fclass=`echo $fline | awk '{ print $1 }'| cut -f2 -d'<'`
      grep -q "^$fclass" $tmpattr  2>/dev/null
      if [ $? -ne 0 ]; then
         echo "schema info not found for $fclass, skipping" | tee -a $log
         rm -f $xmlfile 
         continue
      fi
      ln=
   fi
   
   cp -f $xmlfile $xmlout
   # always remove certain admin attributes
   sed -i -e "s/intId=\"[^\"]*\"//" $xmlout
   sed -i -e "s/access=\"[^\"]*\"//" $xmlout
   echo $fclass |grep faultPolicy >/dev/null 2>&1
   if [ $? -eq 0 ]; then
      sed -i -e "s/totalFaults=\"[^\"]*\"//" $xmlout
   fi 
   # remove callhomePeriodicSystemInventory:lastDeadline
   sed -i -e "s/lastDeadline=\"[^\"]*\"//" $xmlout 

   # Remove any classes that are designated operations 
   for cl in `cat $tmpdel | awk '{ print $1 }'`
   do
      grep -q $cl  $xmlout
      if [ $? -eq 0 ]; then
	  if [ "$cl" = "$fclass" ]; then
	     echo "Deleting oper class [$cl]" | tee -a $log
	     rm -f $xmlfile $xmlout
	     xmldone=1
	     break
          else
	     # echo "Removing oper class [$cl]" | tee -a $log
	     remove_class $cl $xmlout
	  fi
      fi
   done
   if [ $xmldone -eq 1 ]; then
      continue
   fi

   # Remove any attributes that are not in the schema to be kept
   # Inputs:  $fclass, $ln. Uses: $tmpf, $tmpkeep, $xmlout, $log
   ln=`grep -m1 -n "<${fclass}" $xmlout | cut -f1 -d':'`
   remove_attrs $fclass
   
   # handle child elements
   ln=
   children=`xmlstarlet sel -t -m "//$fclass/"'*' -v "name()" -n $xmlout |sort -u`
   echo "$fclass children: $children" >>$log
   for child in $children 
   do
      echo $child |grep -q Fsm
      if [ $? -eq 0 ]; then
         remove_class $child $xmlout
      else
         remove_attrs $child
      fi
   done
   grep -q "dn=" $xmlout
   if [ $? -ne 0 ]; then
      echo "Error: $fclass : $xmlout has no dn attribute" | tee -a $log
      dnval=`echo $xmlout |sed -e 's/^.\///' -e 's/.xml$//'`
      echo "=== dn=$dnval cat $xmlout ===" >>$log
      cat $xmlout >>$log
      echo "=== end, $xmlout ===" >>$log
      cleantmp
      exit 1
   else
      dnval=`xmlstarlet sel -t -m '//*' -s A:T:U @dn -v @dn  $xmlout`
   fi
   if [ "x$dnval" = "x" ]; then
      echo "Error: $fclass : $xmlout has empty dn attribute" | tee -a $log
      cat $xmlout >>$log
      cleantmp
      exit 1
   fi
   # lsbootSanCatSanImagePath vnicName attribute cannot be changed
   iclass=lsbootSanCatSanImagePath
   attr=vnicName
   # grep ImagePath $xmlout |grep -q $attr
   grep $iclass $xmlout |grep -q $attr
   if [ $? -eq 0 ]; then
      #see also: remove_1attr $fclass $iclass $attr
      sed -i -E "s/(<${iclass}.+) ${attr}=\"[^\"]*\"(.+)/\1 \2/" $xmlout
      echo "  remove attribute $attr from $fclass:$iclass" |tee -a $log
   fi
   # iscsiAuthProfile password attr is not readable and cannot be restored
   echo $fclass |grep -q iscsiAuthProfile
   if [ $? -eq 0 ]; then
      sed -i -e "s/password=\"[^\"]*\"//" $xmlout
   fi
   echo $fclass |grep -q lsServer 
   if [ $? -eq 0 ]; then
      # remove several oper/admin child elements
      remove_class lsServerFsm $xmlout
      remove_class vnicEther  $xmlout
      remove_class vnicFc     $xmlout
      remove_class vnicFcNode $xmlout
      remove_class fabricVCon $xmlout
      sed -i -e '/<identRequestEp/d' $xmlout
      sed -i -e '/<lsServerFsmStage/d' $xmlout
      sed -i -e '/<lsmaintAck/d' $xmlout
      sed -i -e '/<vnicConnDef/d' $xmlout
      # avoid any other read-only elements, if not handled above
      while read line
      do
         class=`echo $line |awk '{ print $1 }'`
         remove_class $class $xmlout
      done <$tmpro
   fi
   echo $fclass |grep -q vmLifeCyclePolicy 
   if [ $? -eq 0 ]; then
      remove_class vmLifeCyclePolicyFsm $xmlout
   fi
   sed -i -e '/<faultInst/d' $xmlout

   # Now finished with $xmlout (.xml), so delete $xmlfile (.orig)
   rm -f $xmlfile 

done
cleantmp
popd
echo "`date` -- Done " >>$log

