#!/usr/bin/perl

#   
#  Unitrends:
#

use warnings;
use strict;
use English;
use Data::Dumper;
use File::Basename;
use File::Spec;
use IO::Handle;
use XML::LibXML;
use POSIX ":sys_wait_h";
use POSIX qw(strftime);
use Getopt::Long;

my $log_count = 25;

my $BPUTIL = "/usr/bp/bin/bputil";
my $OK_OUT = "/tmp/ir-msg$$";
my $ERR_OUT = "/tmp/ir-errmsg$$";
my $RESPONSE_STRING = "Client Messages";
my $NIC_GUID = "2fc216b0-d2e2-4967-9b6d-b8a5c9ca2778";
my $SCSI_GUID = "d422512d-2bf2-4752-809d-7b82b5fcb1b4";
my $IDE_GUID = "83f8638b-8dca-4152-9eda-2ca8b33039b4";
my $AUDIT = "audit";
my @scsi_controllers;

my $progname = 'uni_hyperv';

sub setup_logging {
        my $logname = $progname;
        my $i = $log_count;
        while ($i >= 1) {
                my $m = $i + 1;
                rename "/usr/bp/logs.dir/$logname.log.$i", "/usr/bp/logs.dir/$logname.log.$m";

                $i--;
        }
        rename "/usr/bp/logs.dir/$logname.log", "/usr/bp/logs.dir/$logname.log.1";

        open LOGFILE, '>', "/usr/bp/logs.dir/$logname.log" or die $!;
        print LOGFILE "$logname log start\n";
        print LOGFILE "Start time:  " . strftime("%a, %d %b %Y %H:%M:%S %z", localtime(time())) . "\n";
}

sub update_status ($$$$) {
        my $s_file = shift;
        my $state = shift;
        my $percent = shift;
        my $detail = shift;

        die if (not defined($detail));

        # File has a line in it like
        # status:running:50
        # which indicates 50% done.  Update this percentage value.

	print LOGFILE "update_status: s_file = " . $s_file . "\n";
	print LOGFILE "update_status: state = " . $state . "\n";
	print LOGFILE "update_status: percent = " . $percent . "\n";
	print LOGFILE "update_status: detail = " . $detail . "\n";
        if (defined($s_file)) {
                `sed -i s/^status:.*/status:$state:$percent:"$detail"/ $s_file 2>/dev/null`;
        } else {
		print LOGFILE "File not found = " . $$s_file . "\n";
	}
}

sub run_command {
	my $command = shift;
	my $return_string = "Success";
	my $msg_file = '';

	$command = $command . " > $OK_OUT 2> $ERR_OUT";
	print LOGFILE "command = " . $command . "\n";

	`$command`; # Run the command;
        my $ret = $?;

        if ($ret == 0) {
		print LOGFILE "ret = " . $ret . ", STDOUT\n";
		$msg_file = $OK_OUT;
	} else {
		print LOGFILE "ret = " . $ret . ", STDERR\n";
		$msg_file = $ERR_OUT;
	}

	open(my $file,  "<",  "$msg_file")  or die "Can't open $msg_file: $!";
	while (<$file>)  {
		if ($_  !~ /$RESPONSE_STRING/) {
			print LOGFILE "run_command - Return Data: " . "$_";
			chomp;
			$return_string = $_;
		}
	}

	unlink $OK_OUT;
	unlink $ERR_OUT;
	return $ret, $return_string;
}

sub get_owner_node {
        my $server = shift;
	my $option_type = shift;
        my $object = shift;

        my $command = "$BPUTIL --wincmd $server RunCScript.bat wir_hyperv.ps1 -task owner -$option_type '$object'";
	my ($ret, $return_string) = run_command($command);
        return $ret, $return_string;
}

sub get_nic_count {
	my $doc = shift;
	my $nic_count = 0;

	foreach my $node ($doc->findnodes("/configuration/manifest")) {
		my $device_size = $node->findvalue('./size');
		if (length $device_size == 0) {
			print LOGFILE "get_nic_count - device_size length = 0\n";
			return $nic_count;
		}

		foreach my $vdev (1 .. $device_size) {
			my $device_number = sprintf( 'vdev%0*d', 3, $vdev );
			foreach my $dev_node ($doc->findnodes("/configuration/manifest/" . $device_number)) {
				my $device_guid = $dev_node->findvalue('./device');
				if (length $device_guid == 0) {
					print LOGFILE "device_guid length = 0\n";
				} elsif ($device_guid eq $NIC_GUID) {
					$nic_count++;
				}
			}
		}
	}
	print LOGFILE "HYPER-V IR: network_card_count = " . $nic_count . "\n";
	return $nic_count;
}

sub create_SCSI_controller_list {
	my $doc = shift;
	my $scsi_controller_count = 0;

	foreach my $node ($doc->findnodes("/configuration/manifest")) {
		my $device_size = $node->findvalue('./size');
		if (length $device_size == 0) {
			print LOGFILE "create_SCSI_controller_list - device_size length = 0\n";
			return $scsi_controller_count;
		} 

		foreach my $vdev (1 .. $device_size) {
			my $device_number = sprintf( 'vdev%0*d', 3, $vdev );
			foreach my $dev_node ($doc->findnodes("/configuration/manifest/" . $device_number)) {
				my $device_guid = $dev_node->findvalue('./device');
				if (length $device_guid == 0) {
					print LOGFILE "device_guid length = 0\n";
				} elsif ($device_guid eq $SCSI_GUID) {
					my $instance_guid = $dev_node->findvalue('./instance');
					if (length $instance_guid == 0) {
						print LOGFILE "create_SCSI_controller_list - instance_guid length = 0\n";
					} else {
						push @scsi_controllers, $instance_guid;
						print LOGFILE "SCSI Controller[" . $scsi_controller_count . "] = " . $instance_guid . "\n";
						$scsi_controller_count++;
					}
				}
			}
		}
	}
	print LOGFILE "HYPER-V IR: scsi_controller_count = " . $scsi_controller_count . "\n";
	return $scsi_controller_count;
}

sub get_memory_config {
	my $doc = shift;

	my $mem_startup = 0;
	my $mem_dynamic = 0;
	my $mem_min = 0;
	my $mem_max = 0;
	my $mem_buf_percent = 0;
	my $mem_weight = 0;

	foreach my $node ($doc->findnodes("configuration/settings/memory/bank")) {
		$mem_startup = $node->findvalue('./size');
		if (length $mem_startup == 0) {
			print LOGFILE "mem_startup length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: Startup RAM = " . $mem_startup . "\n";
		}
		$mem_dynamic = $node->findvalue('./dynamic_memory_enabled');
		if (length $mem_dynamic == 0) {
			print LOGFILE "mem_dynamic length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: Dynamic RAM enabled = " . $mem_dynamic . "\n";
		}
		$mem_min = $node->findvalue('./reservation');
		if (length $mem_min == 0) {
			print LOGFILE "mem_min length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: Minimum RAM = " . $mem_min . "\n";
		}
		$mem_max = $node->findvalue('./limit');
		if (length $mem_max == 0) {
			print LOGFILE "mem_max length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: Maximum RAM = " . $mem_max . "\n";
		}
		$mem_buf_percent = $node->findvalue('./target_memory_buffer');
		if (length $mem_buf_percent == 0) {
			print LOGFILE "mem_buf_percent length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: Memory Buffer = " . $mem_buf_percent . "%\n";
		}
		$mem_weight = $node->findvalue('./priority');
		if (length $mem_weight == 0) {
			print LOGFILE "mem_weight length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: Memory Weight = " . $mem_weight . "\n";
		}
	}
	return $mem_startup, $mem_min, $mem_max, $mem_dynamic;
}

sub get_cpu_count {
	my $doc = shift;

	my $cpu_count = 0;
	foreach my $node ($doc->findnodes("configuration/settings/processors")) {
		$cpu_count = $node->findvalue('./count');
		if (length $cpu_count eq 0) {
			print LOGFILE "cpu_count length = 0\n";
		} else {
			print LOGFILE "HYPER-V IR: CPU count = " . $cpu_count . "\n";
		}
	}
	return $cpu_count;
}

sub record_hyperv_info {
	my $s_file = shift;
	my $server = shift;
	my $iscluster = shift;
	my $vm_guid = shift;
	my $ir_audit = shift;
	my $vm_name = shift;
	my $disk_source = shift;

	my $xmloutput = XML::LibXML->createDocument;
	$xmloutput->setEncoding('UTF-8');
	my $xmlroot = $xmloutput->createElement('vm_info');
	$xmloutput->setDocumentElement($xmlroot);

	$xmlroot->appendTextChild('server', $server);
	$xmlroot->appendTextChild('iscluster', $iscluster);
	$xmlroot->appendTextChild('vm_guid', $vm_guid);
	$xmlroot->appendTextChild('audit', $ir_audit);
	$xmlroot->appendTextChild('vm_name', $vm_name);
	$xmlroot->appendTextChild('disk_source', $disk_source);

	open(STATUS, '>>', $s_file . '.vm_info') || die "Cannot open $s_file in append mode: $!\n";
	print STATUS $xmloutput->toString(1);
	close STATUS;
}

sub create_vm {
	my $hostname = '';
	my $newvmname = '';
	my $VMDestinationPath = '';
	my $disksource = '';
	my $diskpathlist = '';
	my $XMLPathname = '';
	my $power = 0;
	my $audit = '';
	my $gen = 0;
	my $isCluster = 0;
	my $status_file = '';
	my $switch_name = undef;
	my $efi = 0;

	Getopt::Long::Configure( qw(pass_through) );
	GetOptions('hostname=s' => \$hostname,
		   'newvmname=s' => \$newvmname,
		   'VMDestinationPath=s' => \$VMDestinationPath,
		   'disksource=s' => \$disksource,
		   'diskpathlist=s' => \$diskpathlist,
		   'XMLPathname=s' => \$XMLPathname,
		   'power=i' => \$power,
		   'audit=s' => \$audit,
		   'gen=i' => \$gen,
		   'isCluster=i' => \$isCluster,
		   'ir_status_file=s' => \$status_file,
		   'switchName=s' => \$switch_name);

	print LOGFILE "hostname = " . $hostname . "\n";
	print LOGFILE "newvmname = " . $newvmname . "\n";
	print LOGFILE "VMDestinationPath = " . $VMDestinationPath . "\n";
	print LOGFILE "disksource = " . $disksource . "\n";
	print LOGFILE "diskpathlist = " . $diskpathlist . "\n";
	print LOGFILE "XMLPathname = " . $XMLPathname . "\n";
	print LOGFILE "power = " . $power . "\n";
	print LOGFILE "audit = " . $audit . "\n";
	print LOGFILE "gen = " . $gen . "\n";
	print LOGFILE "isCluster = " . $isCluster . "\n";
	print LOGFILE "ir_status_file = " . $status_file . "\n";
	if (defined $switch_name) {
		print LOGFILE "switch_name = " . $switch_name . "\n";
	}

	$VMDestinationPath =~ tr{/}{\\};

	# Load XML
	my $parser = XML::LibXML->new();

	# Convert XML to UTF-8, Hyper-V agent stores it as UTF-16
	my $data = `xmllint --encode UTF-8 $XMLPathname`;

	if (length $data < 3) {
		print LOGFILE "Failed to convert $XMLPathname to UTF-8\n";
		die "Failed to convert $XMLPathname to UTF-8.\n";
	}
	
	my $doc = $parser->parse_string($data) || die "Failed to parse XML from $XMLPathname.\n";

        if ($isCluster != 0) {
		update_status($status_file, 'running', 10, 'Get Cluster Owner');
                # call get_owner_node
                my ($status, $return_string) = get_owner_node($hostname, "vmlocation", $VMDestinationPath);
                if ($status != 0) {
                        # The $owner variable will have error message.
                        print LOGFILE "Cannot obtain owner of $VMDestinationPath" . $return_string;
			return 1, "Get Cluster Owner failed";
                } else {
                        # substitute current owner node of the cluster for server
                        $hostname = $return_string;
                }
        }
	print LOGFILE "hostname = " . $hostname . "\n";

	update_status($status_file, 'running', 15, 'Get VM Configuration Data');
	my $nic_count = get_nic_count($doc);

	my $scsi_count = create_SCSI_controller_list($doc);

	my ($mem_startup, $mem_min, $mem_max, $mem_dynamic) = get_memory_config($doc);

	my $cpu_count = get_cpu_count($doc);

	update_status($status_file, 'running', 20, 'Create VM');

	if ($gen == 2) {
		$efi = 1;
	}

	my $command = '';
	if ($mem_dynamic eq "True") {
        	$command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task create-ir -vmname '$newvmname' -efi $efi -vmlocation '$VMDestinationPath' -vmnics $nic_count -cpu $cpu_count -memory $mem_startup -vmminram $mem_min -vmmaxram $mem_max -vmdynmem";
	} else {
        	$command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task create-ir -vmname '$newvmname' -efi $efi -vmlocation '$VMDestinationPath' -vmnics $nic_count -cpu $cpu_count -memory $mem_startup";
	}

	if (defined $switch_name && $audit ne $AUDIT) {
		$command = $command . " -vmswitch '$switch_name'";
	}

	my $vm_guid = undef;
	my ($ret, $return_string) = run_command($command);
	if ($ret == 0) {
		$vm_guid = $return_string;
		print LOGFILE "Create VM successful, GUID = " . $vm_guid . "\n";
	} else {
		print LOGFILE "Create VM failed, error message = " . $return_string . "\n";
		return 1, "Create VM failed";
	}

	record_hyperv_info($status_file, $hostname, $isCluster, $vm_guid, $audit, $newvmname, $disksource);

	update_status($status_file, 'running', 25, 'Create Disks');
	my @values = split(',', $diskpathlist);
	print LOGFILE "diskpathlist array:\n";
	foreach my $val (@values) {
		my ($path, $controller, $disk, $guid) = split(':', $val);
		$guid =~ tr/_//d;
		$val = "//" . $disksource . $path;
		$val =~ tr{/}{\\};
		print LOGFILE "$val\n";
		my $controller_type = undef;
		if ($guid =~ /$IDE_GUID/i)
		{
			$controller_type = "ide";
		} else {
			$controller_type = "scsi";
			foreach my $scsi_index (0 .. @scsi_controllers) {
				if ($scsi_controllers[$scsi_index] =~ /$guid/i) {
					$controller = $scsi_index;
					last;
				}
			}
		}

		my $disk_location = $controller_type . ":" . $controller . ":" .  $disk;
		my $disk_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task create-disk -vmlocation '$val' -vmguid $vm_guid -controller $disk_location";
		my ($ret, $return_string) = run_command($disk_command);
		if ($ret == 0) {
			print LOGFILE "Build Disk successful\n";
		} else {
			print LOGFILE "Build Disk  failed, error message = " . $return_string . "\n";
			return 1, "Create Disks failed";
		}
	}
	print LOGFILE "End diskpathlist array\n";

	if ($power != 0) {
		update_status($status_file, 'running', 30, 'Power On VM');
		my $power_on_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task poweron -vmguid $vm_guid";
		my ($ret, $return_string) = run_command($power_on_command);
		if ($ret == 0) {
			print LOGFILE "Power on was successful\n";
		} else {
			print LOGFILE "Power on failed, error message = " . $return_string . "\n";
			return 1, "Power On VM failed";
		}
	}

	if ($audit eq $AUDIT) {
		# Audit mode
		print LOGFILE "Audit mode\n";
	} else {
		# Live mode
		print LOGFILE "Starting Live mode\n";
		update_status($status_file, 'running', 35, 'Configure Switch');
		my $network_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task enable-network -vmguid $vm_guid";
		my ($ret, $return_string) = run_command($network_command);
		if ($ret == 0) {
			print LOGFILE "enable-network request was successful\n";
		} else {
			print LOGFILE "enable-network request failed, error message = " . $return_string . "\n";
			return 1, "Configure Switch failed";
		}

		update_status($status_file, 'running', 40, 'Move Disks');
		my $move_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task move-disks -vmguid $vm_guid";
		($ret, $return_string) = run_command($move_command);
		if ($ret == 0) {
			print LOGFILE "Initiate move-disks was successful\n";
		} else {
			print LOGFILE "Initiate move-disks failed, error message = " . $return_string . "\n";
			return 1, "Move Disks failed";
		}
		# Running|Failed|Not Started|Blocked|Stopped|No Migration:Percent Complete
		my $percent_complete = 0;
		my $starting_percent = 40;
		my $ending_percent = 90;
		my $move_percent = $ending_percent - $starting_percent;
		my $total_complete = 0;
		my $result = '';
		my $move_status_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task move-status -vmguid $vm_guid -dontsavelog";
		while ($percent_complete < 100) {
			my ($ret, $return_string) = run_command($move_status_command);
			if ($ret != 0) {
				print LOGFILE "move-status failed, error message = " . $return_string . "\n";
				return 1, "Move Status failed";
			}
			($result, $percent_complete) = split(':', $return_string);

			if ($result eq "Running" or $result eq "Completed") {
				$total_complete = ($starting_percent + int (($move_percent * $percent_complete)/100));
				update_status($status_file, 'available', $total_complete, 'Move Disks');
				if ($result eq "Running") {
					sleep 15;
				}
			} elsif ($result eq "Stopped") {
				print LOGFILE "move-status returned " . $result . "\n";
				return 1, "Move Disks canceled";
			} elsif ($result eq "Failed" or $result eq "Not Started" or $result eq "Blocked" or $result eq "No Migration") {
				print LOGFILE "move-status returned " . $result . "\n";
				print LOGFILE "move-status returned " . $percent_complete . "\n";
				return 1, $percent_complete;
			} else {
				print LOGFILE "move-status, invalid result, result = " . $result . "\n";
				return 1, "Move Status failed";
			}
		}
	}

	return $ret, $vm_guid;
}

sub delete_vm {
	my $hostname = '';
	my $vm_guid = '';
	my $disk_source = '';
	my $isCluster = 0;
	my $removevm = 0;
	my $ret = 0;
	my $message = 'SUCCESS';
	my $isPresent = 0;

	Getopt::Long::Configure( qw(pass_through) );
	GetOptions('hostname=s' => \$hostname,
		   'vmguid=s' => \$vm_guid,
		   'isCluster=i' => \$isCluster,
		   'disksource=s' => \$disk_source,
		   'removevm=i' => \$removevm);
	print LOGFILE "hostname = " . $hostname . "\n";
	print LOGFILE "vm_guid = " . $vm_guid . "\n";
	print LOGFILE "isCluster = " . $isCluster . "\n";
	print LOGFILE "removevm = " . $removevm . "\n";
	print LOGFILE "disk_source = " . $disk_source . "\n";

        if ($isCluster != 0) {
		# call get_owner_node
		($ret, $message) = get_owner_node($hostname, "vmguid", $vm_guid);
		if ($ret != 0) {
			# The $message variable will have error message.
			print LOGFILE "Cannot obtain owner of $vm_guid" . $message;
			return 0, $message;
		} else {
			# substitute current owner node of the cluster for server
			$hostname = $message;
		}
	}

	my $isPresent_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task ispresent -vmguid $vm_guid -vmlocation '\\\\$disk_source\\'";
	($ret, $isPresent) = run_command($isPresent_command);
	if ($ret != 0) {
		print LOGFILE "ispresent failed, error message = " . $isPresent . "\n";
		return 1, "ispresent failed";
	}

	if ($isPresent == 0) {
		$ret = 0;
		$message = $isPresent;
	} elsif ($removevm == 0) {
		$ret = 1;
		$message = "User must delete the VM";
	} else {
		my $delete_command = "$BPUTIL --wincmd $hostname RunCScript.bat wir_hyperv.ps1 -task delete -vmguid $vm_guid";
		($ret, $message) = run_command($delete_command);
	}
	return $ret, $message;
}

###### main exec block

sub main {
	my $create = 0;
	my $delete = 0;
	my $test = 0;
	my $status = 0;
	my $message = undef;

	setup_logging();
	print LOGFILE "Entering main\n";
	Getopt::Long::Configure( qw(pass_through) );
	GetOptions('create' => \$create, 'delete' => \$delete);
	print LOGFILE "After GetOpeions\n";
	if ($create == 1) {
       		print LOGFILE "create = " . $create . "\n";
		($status, $message) = create_vm();
	} elsif ($delete == 1) {
       		print LOGFILE "delete = " . $delete . "\n";
		($status, $message) = delete_vm();
	} else {
		$message = "Invalid command";
       		print LOGFILE $message . "\n";
		$status = 255;
	}

	if ($status) {
		print STDERR $message;
	} else {
		print STDOUT $message;
	}
	print LOGFILE "main - return data = " . $message . "\n";
	print LOGFILE "main - return status = " . $status . "\n";
	return $status;
}

exit main();
