<?php


class Archive
{
    private $BP;

    public function __construct($BP)
    {
        $this->BP = $BP;
        $this->FUNCTIONS = new Functions($BP);
    }

    public function get($which, $data, $sid, $systems) {
        switch($which[0]) {
            case 'media':
                switch($which[1]) {
                    case 'connected':
                        $status = $this->BP->get_connected_archive_media($sid);
                        if($status !== false) {
                            $status = array('data' => $status);
                        }
                        break;
                    case 'candidates':
                        $status = $this->BP->get_possible_archive_media($sid);
                        if($status !== false) {
                            $status = array('data' => $status);
                        }
                        break;
                    case 'sets':
                        $slots = "";
                        if(isset($data['slots'])) {
                            $slots = $data['slots'];
                        }
                        $sets = $this->BP->get_media_archive_sets(urldecode($which[2]), $slots, $sid);
                        $status['sets'] = array();
                        if($sets !== false) {
                            foreach($sets as $set) {
                                $outSet['date'] = $this->FUNCTIONS->formatDateTime($set['timestamp']);
                                $outSet['description'] = $set['description'];
                                $outSet['needs_import'] = $set['needs_import'];
                                $status['sets'][] = $outSet;
                            }
                        } else {
                            $status = $sets;
                        }
                        break;
                }
                break;
            case 'sets':
                $allSets = array();
                if(!isset($which[1])) {
                    foreach($systems as $sid => $name) {
                        $sets = $this->getSets($data, $sid);
                        for($i = 0; $i < count($sets); $i++) {
                            $sets[$i]['id'] = $sets[$i]['archive_set_id'];
                            unset($sets[$i]['archive_set_id']);
                            $sets[$i]['date'] = $this->FUNCTIONS->formatDateTime($sets[$i]['timestamp']);
                            unset($sets[$i]['timestamp']);
                            $sets[$i]['sid'] = $sid;
                            $sets[$i]['system_name'] = $name;
                        }
                        $allSets = array_merge($allSets, $sets);
                    }
                } else {
                    $sets = $this->getSets($data, $sid);
                    if($sets != false) {
                        foreach($sets as $set) {
                            if(is_numeric($which[1]) and $which[1] != $set['archive_set_id']) {
                                continue;
                            } else {
                                $setInfo = $this->BP->get_archive_set_info($set['archive_set_id'], $sid);
                                $setInfo['id'] = $set['archive_set_id'];
                                $setInfo['elapsed_time'] = $this->FUNCTIONS->formatTimeDelta($setInfo['elapsed_secs']);
                                unset($setInfo['elapsed_secs']);
                                $setInfo['date'] = $this->FUNCTIONS->formatDateTime($setInfo['timestamp']);

                                //profile info
                                if($setInfo['profile']['range_end'] == 0) {
                                    if($setInfo['profile']['range_size'] != 0) {
                                        $setInfo['profile']['start_date'] = $this->FUNCTIONS->formatDateTime($setInfo['timestamp'] - $setInfo['profile']['range_size']);
                                    } else {
                                        $setInfo['profile']['start_date'] = 0;
                                    }
                                    $setInfo['profile']['end_date'] = 0;
                                } else {
                                    if($setInfo['profile']['range_size'] != 0) {
                                        $setInfo['profile']['start_date'] = $this->FUNCTIONS->formatDateTime($setInfo['profile']['range_end'] - $setInfo['profile']['range_size']);
                                    } else {
                                        $setInfo['profile']['start_date'] = 0;
                                    }
                                    $setInfo['profile']['end_date'] = $this->FUNCTIONS->formatDateTime($setInfo['profile']['range_end']);
                                }
                                unset($setInfo['profile']['range_end']);
                                unset($setInfo['profile']['range_size']);
                                unset($setInfo['timestamp']);
                                if(isset($setInfo['profile']['clients'])) {
                                    $clients = array();
                                    foreach($setInfo['profile']['clients'] as $client) {
                                        $clientArr = array();
                                        $clientArr['id'] = $client;
                                        $clientInfo = $this->BP->get_client_info($client, $sid);
                                        $clientArr['name'] = $clientInfo['name'];
                                        $clients[] = $clientArr;
                                    }
                                    $setInfo['profile']['clients'] = $clients;
                                }
                                if(isset($setInfo['profile']['instances'])) {
                                    $instances = array();
                                    foreach($setInfo['profile']['instances'] as $instance) {
                                        $instanceArr = array();
                                        $instanceArr['id'] = $instance;
                                        $instanceInfo = $this->BP->get_appinst_info($instance, $sid);
                                        $instanceArr['primary_name'] = $instanceInfo[$instance]['primary_name'];
                                        if(isset($instanceInfo[$instance]['secondary_name'])) {
                                            $instanceArr['secondary_name'] = $instanceInfo[$instance]['secondary_name'];
                                        }
                                        $instances[] = $instanceArr;
                                    }
                                    $setInfo['profile']['instances'] = $instances;
                                }
                                if(isset($setInfo['profile']['client_objects'])) {
                                    $setInfo['profile']['objects'] = $setInfo['profile']['client_objects'];
                                    unset($setInfo['profile']['client_objects']);
                                }
                                $typeArr = array();
                                foreach($setInfo['profile']['types'] as $type) {
                                    $displayType = $this->FUNCTIONS->getBackupTypeDisplayName($type);
                                    if(!in_array($displayType, $typeArr)) {
                                        $typeArr[] = $displayType;
                                    }
                                }
                                $setInfo['profile']['types'] = $typeArr;
                                $allSets[] = $setInfo;
                            }
                        }
                    }
                }
                if($allSets !== false) {
                    $status = array('sets' => $allSets);
                } else {
                    $status = $allSets;
                }
                break;
            case 'catalog':
                //get archive status first
                $cid = isset($_GET['cid']) ? (int)$_GET['cid'] : false;
                $iid = isset($_GET['iid']) ? (int)$_GET['iid'] : false;
                $startDate = isset($_GET['start_date']) ? strtotime($_GET['start_date']) : strtotime(Constants::DATE_ONE_WEEK_AGO);
                $endDate = isset($_GET['end_date']) ? $this->FUNCTIONS->formatEndDate($_GET['end_date']) : strtotime('now');
                $startBackupDate = isset($_GET['start_backup_date']) ? strtotime($_GET['start_backup_date']) : false;
                $endBackupDate = isset($_GET['end_backup_date']) ? strtotime($_GET['start_backup_date']) : false;
                $view = isset($_GET['view']) ? $_GET['view'] : "system";

                $result_format['job_start_time'] = $startDate;
                $result_format['job_end_time'] = $endDate;

                if($cid){
                    $result_format['client_id'] = $cid;
                }
                if($iid) {
                    $result_format['instance_id'] = $iid;
                }
                if($startBackupDate) {
                    $result_format['backup_start_time'] = $startBackupDate;
                }
                if($endBackupDate) {
                    $result_format['backup_end_time'] = $endBackupDate;
                }

                if ($sid == false) {
                    $systems = $this->FUNCTIONS->selectSystems();
                    $data["catalog"] = array();
                    foreach ($systems as $sid => $name) {
                        $result_format['system_id'] = $sid;
                        if ($view !== false){
                            $data["catalog"] = $this->merge_view($data["catalog"], $this->getArchivesByView($view, $result_format, $sid), $view);
                        }
                    }
                } else {
                    $result_format['system_id'] = $sid;
                    if ($view !== false){
                        $data["catalog"] = $this->getArchivesByView($view, $result_format, $sid);
                    }
                }
                if ($view == 'day') {
                    // If day view, sort all days in the catalog.
                    foreach ($data["catalog"] as $key => $row) {
                        $orderByDate[$key] = strtotime($row['day']);
                    }
                    array_multisort($orderByDate, SORT_DESC, $data["catalog"]);
                }
                $status = $data;
                break;
            case 'files':
                $start = isset($_GET['dir']) ? $_GET['dir'] : "";
                $last = isset($_GET['last']) ? $_GET['last'] : "";
                $count = isset($_GET['count']) ? (int)$_GET['count'] : 0;
                $files = $this->BP->get_archive_files((int)$which[1], $start, $last, $count, $sid);
                if($files != false) {
                    for($i = 0; $i < count($files); $i++) {
                        unset($files[$i]['parent']);
                        unset($files[$i]['size']);
                        unset($files[$i]['date']);
                        if(array_key_exists('has_children', $files[$i])) {
                            $files[$i]['branch'] = $files[$i]['has_children'];
                            unset($files[$i]['has_children']);
                        }
                    }
                    $status = array();
                    $status['data'] = $files;
                    $status['count'] = $count;
                    $status['total'] = count($files);
                } else {
                    $status = $files;
                }
                break;
            case 'related':
                if(is_numeric($which[1])) {
                    if(isset($_GET['instance']) and is_numeric($_GET['instance'])) {
                        $pitArchiveID = array();
                        $pitArchiveID[] = (int)$which[1];
                        $instance = (int)$_GET['instance'];
                        $filter = array('instance_id' => $instance, 'archive_ids' => $pitArchiveID);
                        //get status for first archive id - backup time is needed
                        $archiveInfo = $this->BP->get_archive_status($filter, $sid);
                        if($archiveInfo !== false) {
                            if(count($archiveInfo) > 0) {
                                //if a full, then that's all we need and can send back just this id as the group
                                if(strtolower($this->FUNCTIONS->getBackupTypeDisplayName($archiveInfo[0]['type'])) == "full") {
                                    $status = array("ids" => (string)$which[1]);
                                } else {
                                    //else use instance and backup time to get archives prior to point in time
                                    unset($filter['archive_ids']);
                                    //$filter['instance_id'] = $instance;
                                    $backupEndTime = $archiveInfo[0]['backup_time'];
                                    $filter['backup_end_time'] = $backupEndTime;
                                    $allArchiveStatus = $this->BP->get_archive_status($filter, $sid);
                                    if($allArchiveStatus !== false) {
                                        //find most recent full
                                        $mostRecentFullID = -1;
                                        $mostRecentFullBackupTime = 0;
                                        for($i = 0; $i < count($allArchiveStatus); $i++) {
                                            if($allArchiveStatus[$i]['backup_time'] < $backupEndTime) {
                                                if(strtolower($this->FUNCTIONS->getBackupTypeDisplayName($allArchiveStatus[$i]['type'])) == "full" and $allArchiveStatus[$i]['backup_time'] > $mostRecentFullBackupTime) {
                                                    $mostRecentFullID = $allArchiveStatus[$i]['archive_id'];
                                                    $mostRecentFullBackupTime = $allArchiveStatus[$i]['backup_time'];
                                                }
                                            }
                                        }
                                        if($mostRecentFullID != -1) {
                                            //get all archive ids between the given id and the most recent full
                                            $archiveIDTime = array($which[1] => $backupEndTime, $mostRecentFullID => $mostRecentFullBackupTime);
                                            for($i = 0; $i < count($allArchiveStatus); $i++) {
                                                if($allArchiveStatus[$i]['backup_time'] > $mostRecentFullBackupTime and $allArchiveStatus[$i]['backup_time'] < $backupEndTime) {
                                                    $archiveIDTime[$allArchiveStatus[$i]['archive_id']] = $allArchiveStatus[$i]['backup_time'];
                                                }
                                            }
                                            //order archive ids from earliest to latest
                                            asort($archiveIDTime);
                                            $status = "";
                                            foreach($archiveIDTime as $id => $time) {
                                                $status .= (string)$id . ", ";
                                            }
                                            $status = array("ids" => substr($status, 0, -2));
                                        } else {
                                            //no full found
                                            $status = array();
                                            $status['error'] = 500;
                                            $status['message'] = "No 'Full' backup copy found for instance " . $instance;
                                        }
                                    } else {
                                        $status = $allArchiveStatus;
                                    }
                                }
                            } else {
                                $status = array();
                                $status['error'] = 500;
                                $status['message'] = "No backup copy was found with id " . $which[1] . " and instance id " . $instance;
                            }
                        } else {
                            $status = $archiveInfo;
                        }
                    } else {
                        $status = array();
                        $status['error'] = 500;
                        $status['message'] = "An instance id must be provided";
                    }
                } else {
                    $status = array();
                    $status['error'] = 500;
                    $status['message'] = "An archive id must be provided";
                }
                break;
        }
        return $status;
    }

    private function getSets($data, $sid) {
        $inputParams = array();
        $start = false;
        $end = false;
        if(isset($data['start_day'])) {
            $inputParams['job_start_time'] = $this->FUNCTIONS->dateToTimestamp($data['start_day']);
            $start = true;
        }
        if(isset($data['end_day'])) {
            $inputParams['job_end_time'] = $this->FUNCTIONS->dateToTimestamp($data['end_day']);
            $end = true;
        }
        if($start and $end and $data['start_day'] === $data['end_day']) {
            $inputParams['job_end_time'] += 86400;
        }
        if(isset($data['media'])) {
            $inputParams['target'] = $data['media'];
        }
        $sets = $this->BP->get_archive_sets($inputParams, $sid);
        return $sets;
    }

    public function delete($which, $sid) {
        switch($which[0]) {
            case 'catalog':
                $status = $this->BP->purge_archive_catalog((int)$which[1], $sid);
                break;
        }
        return $status;
    }

    public function put($which, $data, $sid) {
        if(!is_array($which) and ($which == "" or $which == "check")) {
            //dealing with archive now or archive check
            $inputParams = array();

            if(isset($data['description'])) {
                $inputParams['description'] = $data['description'];
            } else {
                $inputParams['description'] = "";
            }

            $inputParams['target'] = $data['target'];

            if($data['end_date'] == 0) {
                $inputParams['range_end'] = 0;
            } else {
                $inputParams['range_end'] = $this->FUNCTIONS->formatEndDate($data['end_date']);
            }
            if($data['start_date'] == 0) {
                $inputParams['range_size'] = 0;
            } else {
                if($inputParams['range_end'] == 0) {
                    $inputParams['range_size'] = time() - $this->FUNCTIONS->dateToTimestamp($data['start_date']);
                } else {
                    $inputParams['range_size'] = $inputParams['range_end'] - $this->FUNCTIONS->dateToTimestamp($data['start_date']);
                }
            }

            if(isset($data['clients'])) {
                $inputParams['clients'] = $data['clients'];
                $appType = "file-level";
                $types = array();
                foreach($data['types'] as $type) {
                    $types[] = $this->FUNCTIONS->getBackupTypeCoreNameBackupNow($type, $appType);
                }
                $inputParams['types'] = $types;
            } elseif(isset($data['instances'])) {
                $inputParams['instances'] = $data['instances'];
                $appTypes = array();
                $instances = implode(",", $data['instances']);
                $instanceInfo = $this->BP->get_appinst_info($instances, $sid);
                //get unique app types
                foreach($instanceInfo as $info) {
                    $appType = $info['app_type'];
                    if(!in_array($appType, $appTypes)) {
                        $appTypes[] = $appType;
                    }
                }
                //get all backup types based on user-input types and unique app types
                $types = array();
                foreach($data['types'] as $type) {
                    foreach($appTypes as $appType) {
                        $types[] = $this->FUNCTIONS->getBackupTypeCoreNameBackupNow($type, $appType);
                    }
                }
                $inputParams['types'] = $types;
            } elseif(isset($data['localdirs'])) {
                $inputParams['localdirs'] = $data['localdirs'];
            }

            $options = array();

            $options['append'] = isset($data['options']['append']) ? $data['options']['append'] : true;
            $options['purge'] = isset($data['options']['purge']) ? $data['options']['purge'] : false;
            $options['compress'] = false;
            $options['encrypt'] = isset($data['options']['encrypt']) ? $data['options']['encrypt'] : false;
            $options['deduplicate'] = false;
            $options['email_report'] = isset($data['options']['email_report']) ? $data['options']['email_report'] : false;
            $options['retention_days'] = isset($data['options']['retention_days']) ? $data['options']['retention_days'] : 0;

            $inputParams['options'] = $options;

            if($which == "check") {
                $status = $this->BP->check_archive_space($inputParams, $sid);
                //get "fit" paramater
                $mediaInfo = $this->BP->get_connected_archive_media($sid);
                foreach($mediaInfo as $media) {
                    if($media['name'] == $inputParams['target']) {
                        if($media['is_infinite']) {
                            $fitString = "The archive will fit as the media is cloud storage and can have infinite size";
                        } else {
                            $sizeFree = $media['mb_free'];
                            $freeGB = round($sizeFree/1024, 2);
                            if((int)$sizeFree - (int)$status['likely'] > 0) {
                                $fitString = "The archive will fit as the media has " . $freeGB . " GB remaining";
                            } elseif((int)$sizeFree - (int)$status['likely'] < 0) {
                                $fitString = "The archive will not fit as the media has only " . $freeGB . " GB remaining";
                            } else {
                                $fitString = "The archive might fit as the media has " . $freeGB . " GB remaining";
                            }
                        }
                        $status['fit'] = $fitString;
                    }
                }
                $status = array("result" => array($status));
            } else {
                $id = $this->BP->archive_now($inputParams, $sid);
                if($id !== false) {
                    $status['id'] = $id;
                    $status = array("result" => array($status));
                } else {
                    $status = $id;
                }

            }
        } else {
            switch($which[0]) {
                case 'media':
                    switch($which[1]) {
                        case 'mount':
                            $status = $this->BP->mount_archive_media(urldecode($which[2]), $sid);
                            break;
                        case 'unmount':
                            $status = $this->BP->unmount_archive_media(urldecode($which[2]), $sid);
                            break;
                        case 'prepare':
                            $label = "";
                            $slots = "";
                            if(isset($data['label'])) {
                                $label = $data['label'];
                            }
                            if(isset($data['slots'])) {
                                $slots = $data['slots'];
                            }
                            $status = $this->BP->prepare_archive_media(urldecode($which[2]), $label, $slots, $sid);
                            break;
                    }
                    break;
            }
        }
        return $status;
    }

    public function post($which, $data, $sid) {
        switch($which[0]) {
            case 'catalog':
                $slots = "";
                $force = false;
                $name = $data['name'];
                if(isset($data['slots'])) {
                    $slots = $data['slots'];
                }
                if(isset($data['force'])) {
                    $force = $data['force'];
                }
                $status = $this->BP->import_archive_catalog($name, $slots, $force, $sid);
                break;
            case 'search':
                $clientName = $data['client_name'];
                unset($data['client_name']);
                if(isset($data['max_count'])) {
                    $max = $data['max_count'];
                    unset($data['max_count']);
                } else {
                    $max = 1000;
                }
                if(isset($data['start_date'])) {
                    $data['start_date'] = $this->FUNCTIONS->dateToTimestamp($data['start_date']);
                }
                if(isset($data['end_date'])) {
                    $data['end_date'] = $this->FUNCTIONS->formatEndDate($data['end_date']);
                }
                $status = $this->BP->search_archive_files($data, $clientName, $max, $sid);
                if($status !== false) {
                    for($i = 0; $i < count($status); $i++) {
                        $status[$i]['id'] = $status[$i]['archive_id'];
                        unset($status[$i]['archive_id']);
                        $status[$i]['set_id'] = $status[$i]['archive_set_id'];
                        unset($status[$i]['archive_set_id']);
                        $status[$i]['date'] = $this->FUNCTIONS->formatDateTime($status[$i]['date']);
                    }
                    $status = array("files" => $status);
                }
                break;
            case 'status':
                $inputParams = array();
                if(isset($which[1]) and is_numeric($which[1])) {
                    $inputParams['archive_set_id'] = (int)$which[1];
                }
                if(isset($data['client_id'])) {
                    $inputParams['client_id'] = $data['client_id'];
                    $appType = "file-level";
                }
                if(isset($data['instance_id'])) {
                    $instanceInfo = $this->BP->get_appinst_info($data['instance_id'], $sid);
                    $appType = $instanceInfo[$data['instance_id']]['app_type'];
                    if(!isset($inputParams['client_id'])) {
                        $inputParams['client_id'] = $instanceInfo[$data['instance_id']]['client_id'];
                    }
                }
                if(isset($data['type'])) {
                    $typeArr = array();
                    foreach($data['type'] as $type) {
                        if(isset($appType)) {
                            $typeArr[] = $this->FUNCTIONS->getBackupTypeCoreNameBackupNow($type, $appType);
                        } else {
                            $typeArr = array_merge($typeArr, $this->FUNCTIONS->getArchiveTypesByDisplayType($type));
                        }
                    }
                    $inputParams['type'] = $typeArr;
                }
                if(isset($data['start_time_archive'])) {
                    $inputParams['job_start_time'] = $this->FUNCTIONS->dateToTimestamp($data['start_time_archive']);
                }
                if(isset($data['end_time_archive'])) {
                    $timestamp = $this->FUNCTIONS->dateToTimestamp($data['end_time_archive']);
                    if(isset($inputParams['job_start_time']) and $inputParams['job_start_time'] === $timestamp) {
                        $timestamp += 86400;
                    }
                    $inputParams['job_end_time'] = $timestamp;
                }
                if(isset($data['start_time_backup'])) {
                    $inputParams['backup_start_time'] = $this->FUNCTIONS->dateToTimestamp($data['start_time_backup']);
                }
                if(isset($data['end_time_backup'])) {
                    $timestamp = $this->FUNCTIONS->dateToTimestamp($data['end_time_backup']);
                    if(isset($inputParams['backup_start_time']) and $inputParams['backup_start_time'] === $timestamp) {
                        $timestamp += 86400;
                    }
                    $inputParams['backup_end_time'] = $timestamp;
                }
                if(isset($data['archive_ids'])) {
                    $inputParams['archive_ids'] = $data['archive_ids'];
                }
                $archive = $this->BP->get_archive_status($inputParams, $sid);
                if($archive !== false) {
                    $status = array();
                    for($i = 0; $i < count($archive); $i++) {
                        if(!isset($data['instance_id']) or (isset($data['instance_id']) and $data['instance_id'] === $archive[$i]['instance_id'])) {
                            $archive[$i]['id'] = $archive[$i]['archive_id'];
                            unset($archive[$i]['archive_id']);
                            $archive[$i]['set_id'] = $archive[$i]['archive_set_id'];
                            $archive[$i]['date'] = $this->FUNCTIONS->formatDateTime($archive[$i]['archive_time']);
                            unset($archive[$i]['archive_time']);
                            $archive[$i]['elapsed'] = $this->FUNCTIONS->formatTimeDelta($archive[$i]['elapsed_secs']);
                            unset($archive[$i]['elapsed_secs']);
                            $archive[$i]['type'] = $this->FUNCTIONS->getBackupTypeDisplayName($archive[$i]['type']);
                            $archive[$i]['backup_date'] = $this->FUNCTIONS->formatDateTime($archive[$i]['backup_time']);
                            unset($archive[$i]['backup_time']);
                            $archive[$i]['storage'] = $this->getStorage($archive[$i]['set_id'], $sid);
                            $status[] = $archive[$i];
                        }
                    }
                    $status = array("status" => $status);
                } else {
                    $status = $archive;
                }
                break;
        }
        return $status;
    }

    public function restore($which, $data, $sid) {
        $inputParams = array();
        $archiveIDs = explode(", ", $data['archive_ids']);
        if(isset($data['storage'])) {
            $inputParams['device_id'] = $data['storage'];
        }
        $inputParams['association'] = $data['association'];
        if(isset($data['instance_id'])) {
            if($data['association'] == "client") {
                $instanceInfo = $this->BP->get_appinst_info($data['instance_id']);
                $inputParams['association_id'] = $instanceInfo[$data['instance_id']]['client_id'];
            } elseif($data['association'] == "instance") {
                $inputParams['association_id'] = $data['instance_id'];
            }
        } else {
            //restore to original location
            $input = array("archive_ids" => array($archiveIDs[0]));
            $archiveStatus = $this->BP->get_archive_status($input, $sid);
            if($archiveStatus != false) {
                if($data['association'] == "client") {
                    $inputParams['association_id'] = $archiveStatus[0]['client_id'];
                } elseif ($data['association'] == "instance") {
                    $inputParams['association_id'] = $archiveStatus[0]['instance_id'];
                }
            }
        }
        if($which[0] == "files") {
            //check for inclusions and exclusions
            if(isset($data['inclusions'])) {
                $inputParams['inclusions'] = $data['inclusions'];
            }
            if(isset($data['exclusions'])) {
                $inputParams['exclusions'] = $data['exclusions'];
            }
        }
        $jobIDs = array();
        foreach($archiveIDs as $archiveID) {
            $ID = $this->BP->restore_archived_files($archiveID, $inputParams, $sid);
            $jobIDs[] = $ID;
        }
        $status = array();
        $status['id'] = implode(", ", $jobIDs);
        return $status;
    }

    function merge_view($a1, $a2, $view){
        $result = array();
        if (is_array($a1) && count($a1) > 0 && is_array($a2) && count($a2) > 0) {
            if ($view != 'day') {
                $result = array_merge($a1, $a2);
            } else {
                // merge days that overlap, and add others that do not.
                foreach ($a1 as $a1_item) {
                    $day1 = $a1_item['day'];
                    foreach ($a2 as $a2_item) {
                        $day2 = $a2_item['day'];
                        if (strcmp($day1, $day2) == 0) {
                            $merged = array_merge($a1_item['instances'], $a2_item['instances']);
                            $result[] = array('day' => $day2, 'instances' => $merged);
                        } else {
                            $result[] = array('day' => $day2, 'instances' => $a2_item['instances']);
                        }
                    }
                    $result[] = array('day' => $day1, 'instances' => $a1_item['instances']);
                }
            }
        } elseif (is_array($a1) && count($a1) > 0) {
            $result = $a1;
        } elseif (is_array($a2) && count($a2) > 0) {
            $result = $a2;
        }
        return ($result);
    }

    function getArchivesByView($view, $result_format, $sid){

        switch($view){
            case "day":
                $data = $this->catalogByDay($result_format, $sid);
                break;
            case "instance":
                $data = $this->catalogByInstance($result_format, $sid);
                break;
            case "storage":
                $data = $this->catalogByStorage($result_format, $sid);
                break;
            case "system" :
                $data = $this->catalogBySystem($result_format, $sid);
                break;

        }
        return $data;
    }

    function catalogByDay($result_format, $sid){
        $data = false;

        $archives = $this->BP->get_archive_status($result_format, $sid);

        if ($archives !== false) {
            $data = array();
            $items = array();
            foreach ($archives as $archive) {
                if(!isset($result_format['instance_id']) or (isset($result_format['instance_id']) and $result_format['instance_id'] === $archive['instance_id'])) {
                    $day = isset($archive['archive_time']) ? $this->FUNCTIONS->formatDate($archive['archive_time']) : null;
                    $appName = isset($archive['app_name']) ? $archive['app_name'] : "";
                    $clientID = isset($archive['client_id']) ? $archive['client_id'] : null;
                    $clientName = isset($archive['client_name']) ? $archive['client_name'] : null;
                    $instanceID = isset($archive['instance_id']) ? $archive['instance_id'] : null;
                    $instanceName = isset($archive['instance_id']) ? $this->BP->get_appinst_name($archive['instance_id']) : "";

                    $archiveItem = $archive;
                    $archiveItem['id'] = $archiveItem['archive_id'];
                    unset($archiveItem['archive_id']);
                    $archiveItem['set_id'] = $archiveItem['archive_set_id'];
                    unset($archiveItem['archive_set_id']);
                    $archiveItem['elapsed'] = $this->FUNCTIONS->formatTimeDelta($archiveItem['elapsed_secs']);
                    unset($archiveItem['elapsed_secs']);
                    $archiveItem['start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['backup_time']);
                    unset($archiveItem['backup_time']);
                    $archiveItem['archive_start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['archive_time']);
                    unset($archiveItem['archive_time']);
                    $archiveItem['type'] = $this->FUNCTIONS->getBackupTypeDisplayName($archiveItem['type']);
                    if($archiveItem['success']) {
                        $archiveItem['error_string'] = "";
                    }

                    $index = $this->encode_index($appName, $clientID, $clientName, $instanceID, $instanceName, $sid);
                    $items[$day][$index][] = $archiveItem;
                }
            }

            foreach ($items as $day => $indices) {
                $instances = array();
                foreach ($indices as $index => $archives) {
                    $instance = $this->create_instance($index);
                    $instance['archives'] = $archives;
                    $instances[] = $instance;
                }
                $data[] = array('day' => $day, 'instances' => $instances);
            }
        } else {
            global $Log;
            $Log->writeError($this->BP->getError(), true);
        }
        return $data;
    }

    function catalogByInstance($result_format, $sid){
        $data = false;

        $archives = $this->BP->get_archive_status($result_format, $sid);

        if ($archives !== false) {
            $items = array();
            foreach ($archives as $archive) {
                if(!isset($result_format['instance_id']) or (isset($result_format['instance_id']) and $result_format['instance_id'] === $archive['instance_id'])) {
                    $day = isset($archive['archive_time']) ? $this->FUNCTIONS->formatDate($archive['archive_time']) : null;
                    $appName = isset($archive['app_name']) ? $archive['app_name'] : "";
                    $clientID = isset($archive['client_id']) ? $archive['client_id'] : null;
                    $clientName = isset($archive['client_name']) ? $archive['client_name'] : null;
                    $instanceID = isset($archive['instance_id']) ? $archive['instance_id'] : null;
                    $instanceName = isset($archive['instance_id']) ? $this->BP->get_appinst_name($archive['instance_id']) : "";
                    if ($instanceName == "") {
                        $instanceName = $clientName;
                    }

                    $archiveItem = $archive;
                    $archiveItem['id'] = $archiveItem['archive_id'];
                    unset($archiveItem['archive_id']);
                    $archiveItem['set_id'] = $archiveItem['archive_set_id'];
                    unset($archiveItem['archive_set_id']);
                    $archiveItem['elapsed'] = $this->FUNCTIONS->formatTimeDelta($archiveItem['elapsed_secs']);
                    unset($archiveItem['elapsed_secs']);
                    $archiveItem['start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['backup_time']);
                    unset($archiveItem['backup_time']);
                    $archiveItem['archive_start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['archive_time']);
                    unset($archiveItem['archive_time']);
                    $archiveItem['type'] = $this->FUNCTIONS->getBackupTypeDisplayName($archiveItem['type']);
                    if($archiveItem['success']) {
                        $archiveItem['error_string'] = "";
                    }
                    $index = $this->encode_index($appName, $clientID, $clientName, $instanceID, $instanceName, $sid);
                    $items[$day][$index][] = $archiveItem;
                }
            }

            $data = array();
            foreach ($items as $instanceID => $indices) {
                $instances = array();
                foreach ($indices as $index => $archives) {
                    $instance = $this->create_instance($index);
                    $instance['archives'] = $archives;
                    $instances[] = $instance;
                }
                $data = array_merge($data, $instances);
            }
        } else {
            global $Log;
            $Log->writeError($this->BP->getError(), true);
        }
        return $data;
    }

    function catalogByStorage($result_format, $sid){
        $data = false;

        $archives = $this->BP->get_archive_status($result_format, $sid);

        if ($archives !== false) {
            $data = array();
            $items = array();
            foreach ($archives as $archive) {
                if(!isset($result_format['instance_id']) or (isset($result_format['instance_id']) and $result_format['instance_id'] === $archive['instance_id'])) {
                    // $day = isset($backup['start_time']) ? $this->functions->formatDate($backup['start_time']) : null;
                    $appName = isset($archive['app_name']) ? $archive['app_name'] : "";
                    $clientID = isset($archive['client_id']) ? $archive['client_id'] : null;
                    $clientName = isset($archive['client_name']) ? $archive['client_name'] : null;
                    $setid = isset($archive['archive_set_id']) ? $archive['archive_set_id'] : null;
                    $storage = $this->getStorage($setid, $sid);
                    $instanceID = isset($archive['instance_id']) ? $archive['instance_id'] : null;
                    $instanceName = isset($archive['instance_id']) ? $this->BP->get_appinst_name($archive['instance_id']) : "";

                    $index = $this->encode_index($appName, $clientID, $clientName, $instanceID, $instanceName, $sid);
                    $archiveItem = $archive;
                    $archiveItem['id'] = $archiveItem['archive_id'];
                    unset($archiveItem['archive_id']);
                    $archiveItem['set_id'] = $archiveItem['archive_set_id'];
                    unset($archiveItem['archive_set_id']);
                    $archiveItem['elapsed'] = $this->FUNCTIONS->formatTimeDelta($archiveItem['elapsed_secs']);
                    unset($archiveItem['elapsed_secs']);
                    $archiveItem['start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['backup_time']);
                    unset($archiveItem['backup_time']);
                    $archiveItem['archive_start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['archive_time']);
                    unset($archiveItem['archive_time']);
                    $archiveItem['type'] = $this->FUNCTIONS->getBackupTypeDisplayName($archiveItem['type']);
                    if($archiveItem['success']) {
                        $archiveItem['error_string'] = "";
                    }
                    $items[$storage][$index][] = $archiveItem;
                }
            }

            foreach ($items as $storage => $indices) {
                $instances = array();
                foreach ($indices as $index => $archives) {
                    $instance = $this->create_instance($index);
                    $instance['archives'] = $archives;
                    $instances[] = $instance;
                }
                $data[] = array('storage' => $storage, 'instances' => $instances);
            }
        } else {
            global $Log;
            $Log->writeError($this->BP->getError(), true);
        }
        return $data;
    }

    function catalogBySystem($result_format, $sid){
        $data = false;

        $archives = $this->BP->get_archive_status($result_format, $sid);
        $systemName = $this->getSystemName($sid);

        if ($archives !== false) {
            $data = array();
            $items = array();
            foreach ($archives as $archive) {
                if(!isset($result_format['instance_id']) or (isset($result_format['instance_id']) and $result_format['instance_id'] === $archive['instance_id'])) {
                    // $day = isset($archive['archive_time']) ? $this->FUNCTIONS->formatDate($archive['archive_time']) : null;
                    $appName = isset($archive['app_name']) ? $archive['app_name'] : "";
                    $clientID = isset($archive['client_id']) ? $archive['client_id'] : null;
                    $clientName = isset($archive['client_name']) ? $archive['client_name'] : null;
                    $instanceID = isset($archive['instance_id']) ? $archive['instance_id'] : null;
                    $instanceName = isset($archive['instance_id']) ? $this->BP->get_appinst_name($archive['instance_id']) : "";

                    $archiveItem = $archive;
                    $archiveItem['id'] = $archiveItem['archive_id'];
                    unset($archiveItem['archive_id']);
                    $archiveItem['set_id'] = $archiveItem['archive_set_id'];
                    unset($archiveItem['archive_set_id']);
                    $archiveItem['elapsed'] = $this->FUNCTIONS->formatTimeDelta($archiveItem['elapsed_secs']);
                    unset($archiveItem['elapsed_secs']);
                    $archiveItem['start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['backup_time']);
                    unset($archiveItem['backup_time']);
                    $archiveItem['archive_start_date'] = $this->FUNCTIONS->formatDateTime($archiveItem['archive_time']);
                    unset($archiveItem['archive_time']);
                    $archiveItem['type'] = $this->FUNCTIONS->getBackupTypeDisplayName($archiveItem['type']);
                    if($archiveItem['success']) {
                        $archiveItem['error_string'] = "";
                    }
                    $index = $this->encode_index($appName, $clientID, $clientName, $instanceID, $instanceName, $sid);
                    $items[$systemName][$index][] = $archiveItem;
                }
            }

            foreach ($items as $systemName => $indices) {
                $instances = array();
                foreach ($indices as $index => $archives) {
                    $instance = $this->create_instance($index);
                    $instance['archives'] = $archives;
                    $instances[] = $instance;
                }
                $data[] = array('System' => $systemName, 'instances' => $instances);
            }
        } else {
            global $Log;
            $Log->writeError($this->BP->getError(), true);
        }
        return $data;
    }

    function encode_index($appName, $clientID, $clientName, $instanceID, $instanceName, $sid) {
        return $appName . '^' . $clientID . '^' . $clientName . '^' . $instanceID . '^' . $instanceName . '^' . $sid;
    }

    function create_instance($index) {
        $instance = array();
        $a = explode('^', $index);
        $instance = array(
            'app_name' => $a[0],
            'client_id' => $a[1],
            'client_name' => $a[2],
            'instance_id' => $a[3],
            'instance_name' => $a[4],
            'system_id' => $a[5],
            'system_name' => $this->getSystemName($a[5])
        );
        return $instance;
    }

    function getSystemName($sid) {
        $systemName = NULL;
        $systems = $this->BP->get_system_list();
        foreach ($systems as $id => $name) {
            if ($sid == $id) {
                $systemName = $name;
                break;
            }
        }
        return $systemName;
    }

    function getStorage($setid, $sid){
        $storageName = "";
        $setInfo = $this->BP->get_archive_set_info($setid, $sid);
        if($setInfo !== false) {
            $storageName = isset($setInfo['profile']['target']) ? $setInfo['profile']['target'] : "";
        }
        return $storageName;
    }

}
?>

