<?php

class Encryption
{
    private $BP;

    const OFF          = "off";
    const PERSIST      = "persist";
    const ON           = "on";

    public function __construct($BP)
    {
        $this->BP = $BP;
    }

    public function get($which, $systems)
    {
        $encryption = array();

        if($which == -1) {
            $encryption = $this->get_encryption($systems);
        } else {
            if ($which === "support") {
                foreach ($systems as $systemID => $systemName) {
                    $cryptSupport = $this->BP->encryption_supported($systemID);
                    if($cryptSupport !== -1) {
                        $temp_support = array();
                        $temp_support['system_id'] = $systemID;
                        $temp_support['system_name'] = $systemName;
                        $temp_support['EncryptionSupported'] = $cryptSupport === true ? "1" : "0";

                        $encryption['data'][] = $temp_support;
                    }
                }
            }
        }

        return $encryption;
    }

    private function get_encryption($systems)
    {
        $cryptInfo = array();
        foreach ($systems as $systemID => $systemName) {
            //first make sure the crypto daemon is running
            $this->BP->save_crypt_info(array('active' => true), $systemID);
            $info = $this->BP->get_crypt_info($systemID);
            if($info !== false) {
                $temp_encryption = array();
                $temp_encryption['system_id'] = $systemID;
                $temp_encryption['system_name'] = $systemName;
                if($info['active'] && $info['enabled'] && !$info['persistent']) {
                    $temp_encryption['state'] = Encryption::ON;
                } elseif($info['active'] && $info['enabled'] && $info['persistent']) {
                    $temp_encryption['state'] = Encryption::PERSIST;
                } elseif(!$info['enabled']) {
                    $temp_encryption['state'] = Encryption::OFF;
                }
                $temp_encryption['has_passphrase'] = $info['passphrase_set'];

                $cryptInfo['data'][] = $temp_encryption;
            }
        }
        return $cryptInfo;
    }

    public function put($which, $data, $sid)
    {
        $result = array();
        if (is_string($which[0])) {
            switch ($which[0]) {
                case 'enable':
                    $currentState = $this->getEncryptionState($sid);
                    if($currentState !== "") {
                        if(array_key_exists('passphrase', $data)) {
                            $cryptInfo = array('active' => true,
                                                'current_passphrase' => $data['passphrase']);
                            $result = $this->BP->save_crypt_info($cryptInfo, $sid);
                        } else {
                            $result = "Passphrase must be specified";
                        }
                    }
                    break;
                case 'disable':
                    $cryptInfo = array('active' => false);
                    $result = $this->BP->save_crypt_info($cryptInfo, $sid);
                    break;
                case 'persistent':
                    $currentState = $this->getEncryptionState($sid);
                    if($currentState !== "") {
                        if (array_key_exists('passphrase', $data)) {
                            $cryptInfo = array('persistent' => true,
                                'current_passphrase' => $data['passphrase']);
                            $result = $this->BP->save_crypt_info($cryptInfo, $sid);
                        } else {
                            $result = "Passphrase must be specified";
                       }
                    }
                    break;
                case 'not-persistent':
                    $cryptInfo = array('persistent' => false);
                    $result = $this->BP->save_crypt_info($cryptInfo, $sid);
                    break;
                case 'passphrase':
                    if (array_key_exists('current_passphrase', $data) && array_key_exists('new_passphrase', $data)) {
                        $cryptInfo = array('current_passphrase' => $data['current_passphrase'],
                            'new_passphrase' => $data['new_passphrase']);
                        $result = $this->BP->save_crypt_info($cryptInfo, $sid);
                    }
                    break;
            }
        }
        return $result;
    }

    private function getEncryptionState($sid)
    {
        $state = "";
        $cryptInfo = $this->BP->get_crypt_info($sid);
        if($cryptInfo !== false) {
            if($cryptInfo['active'] && $cryptInfo['enabled'] && !$cryptInfo['persistent']) {
                $state = Encryption::ON;
            } elseif($cryptInfo['active'] && $cryptInfo['enabled'] && $cryptInfo['persistent']) {
                $state = Encryption::PERSIST;
            } elseif(!$cryptInfo['enabled']) {
                $state = Encryption::OFF;
            }
        }
        return $state;
    }

    public function post($which, $data, $sid)
    {
        $result = array();
        switch($which) {
            case 'keys':
                $result = $this->BP->backup_crypt_keyfile();
                break;
            case 'passphrase':
                if (array_key_exists('new_passphrase', $data)) {
                    $cryptInfo = array('new_passphrase' => $data['new_passphrase']);
                    $result = $this->BP->save_crypt_info($cryptInfo, $sid);
                }
                break;
        }
        return $result;
    }



}

?>