<?php

class Hostname
{
    private $BP;

    public function __construct($BP, $sid)
    {
        $this->BP = $BP;
        $this->now = time();
        $this->sid = $sid;

        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);

    }


    public function get($which, $sid)
    {
        $data = array();
        switch ($which){
            case "-1":  //never pass in request[1]?
                if ($sid !== false) {
                    $data["data"] = $this->buildOutput($sid);
                } else {
                    $systems = $this->BP->get_systems();
                    foreach($systems as $sid => $name){
                        $data["data"][] = $this->buildOutput($sid);
                    }
                }
            break;
        }

        return $data;
    }

    public function update($which, $inputArray){
 ;
    }

    public function put($which, $data, $sid) {
        global $Log;
        //check if the system is replicating
        $result = ($sid == $this->BP->get_local_system_id()) ? $this->BP->local_system_is_replicating() : $this->BP->is_replication_configured($sid);
        if($result === true or is_string($result)) {
            $status = array();
            $status['error'] = 405;
            $status['message'] = "The given system is replicating, therefore its hostname cannot be changed";
        } else {
            if(isset($data['name'])) {
                //first verify that the name provided does not contain only numbers
                if(!ctype_digit($data['name'])) {
                    //get current hostname in case we have to roll back
                    $oldHostInfo = $this->BP->get_hostname($sid);
                    if($oldHostInfo !== false) {
                        $originalName = $oldHostInfo['name'];
                        if(isset($oldHostInfo['long_name'])) {
                            $longName = $oldHostInfo['long_name'];
                        } else {
                            $longName = false;
                        }
                        $inputParams = array();
                        if(isset($data['name'])) {
                            $name = $data['name'];
                        }
                        if(isset($data['long_name'])) {
                            $inputParams['long_name'] = $data['long_name'];
                        }
                        if(isset($data['keep_alias'])) {
                            $inputParams['keep_alias'] = $data['keep_alias'];
                        }
                        $status = $this->BP->change_hostname($name, $inputParams, $sid);

                        if($status !== false) {
                            $resultClient = $this->updateClientName($sid, $originalName, $name);
                            if($resultClient === false) {
                                $hostOptions = array();
                                if ($longName !== false) {
                                    $hostOptions['long_name'] = $longName;
                                }
                                $hostResult = $this->BP->change_hostname($originalName, $hostOptions, $sid);
                                $status = array();
                                if($hostResult !== false) {
                                    $status['error'] = 500;
                                    $status['message'] = "Unable to change the hostname because: " . $this->BP->getError();
                                    $Log->writeVariable("Unable to change system client to " . $name . ". Leaving the hostname as " . $originalName . ".");
                                } else {
                                    $status['error'] = 500;
                                    $status['message'] = "Unable to change system client to " . $name . ". Unable to revert hostname back to " . $originalName . " because: " . $this->BP->getError();
                                    $Log->writeVariable("Unable to change system client to " . $name . ". Leaving the hostname as " . $originalName . ".");
                                }
                            } else {
                                //update system name
                                $status = $this->updateSystemName($sid, $originalName, $name);
                                if($status === false) {
                                    $status = array();
                                    $status[]['message'] = "Hostname successfully changed. System name failed to update to match. Try to update using PUT /systems/{id}";
                                    $status = array("result" => $status);
                                }
                            }
                        }
                    } else {
                        $status = $oldHostInfo;
                    }
                } else {
                    $status = array();
                    $status['error'] = 500;
                    $status['message'] = "Hostname cannot contain only numbers";
                }
            } else {
                $status = array();
                $status['error'] = 500;
                $status['message'] = "Name is required";
            }

        }
        return $status;
    }

    public function updateClientName($dpuID, $originalName, $newName) {

        $result = false;
        $clientID = $this->getClientID($originalName, $dpuID);
        if ($clientID != -1) {
            $clientInfo = $this->BP->get_client_info($clientID, $dpuID);
            $clientUpdate = array();
            if ($clientInfo !== false) {
                $clientUpdate['id'] = $clientInfo['id'];
                $clientUpdate['name'] = $newName;
                if ( $clientInfo['os_type_id'] == '40' && isset($clientInfo['generic_property']) ) {
                    $result = $this->BP->save_rae_client_info($clientUpdate, $dpuID);
                } else {
                    $result = $this->BP->save_client_info($clientUpdate, $dpuID);
                }
            }
        }
        return $result;
    }

    public function getClientID($clientName, $dpuID) {
        $clientID = -1;
        $clients = $this->BP->get_client_list($dpuID);
        foreach ($clients as $id => $name) {
            if ($clientName == $name) {
                $clientID = $id;
                break;
            }
        }
        return $clientID;
    }

    public function updateSystemName($dpuID, $originalName, $newName) {
        $result = false;
        $systemUpdate = array();
        $systemInfo = $this->BP->get_system_info($dpuID);
        if ($systemInfo !== false) {
            $systemUpdate['id'] = $systemInfo['id'];
            $systemUpdate['name'] = $newName;
            $result = $this->BP->save_system_info($systemUpdate);
        }
        return $result;
    }


    function buildOutput($sid){

        $result = $this->BP->get_hostname($sid);

        if ($result !== false){
            $name = isset($result['name']) ? $result['name'] : "";
            $longName = isset($result['long_name']) ? $result['long_name'] : "";
            $sname = $this->functions->getSystemNameFromID($sid);

            $data = array(
                'name' => $name,
                'long_name' => $longName,
                'system_name' => $sname,
                'system_id' => $sid
            );
        } else {
            $data = false;
        }

        return $data;
    }
}

?>
