<?php
//IN PROGRESS
class Inventory
{
    private $BP;

    const INVENTORY_GET_TYPE_TREE      = 0;
    const INVENTORY_GET_TYPE_SETTINGS  = 1;
    const INVENTORY_GET_TYPE_STATUS    = 2;

    const INVENTORY_NODE_SEPARATOR = "_";
    const INVENTORY_NODE_ID_MY_PHYSICAL_WORLD   = "0_0";
    const INVENTORY_NODE_ID_MY_WINDOWS          = "0_0_0";
    const INVENTORY_NODE_ID_MY_VIRTUAL_WORLD    = "0_1";
    const INVENTORY_NODE_ID_MY_HYPER_V          = "0_1_0";
    const INVENTORY_NODE_ID_MY_VMWARE           = "0_1_1";

    const INVENTORY_GET_ALL         = "all";
    const INVENTORY_GET_CHILDREN    = "children";

    const INVENTORY_STATUS_SUCCESS          = 0;
    const INVENTORY_STATUS_WARNINGS         = 1;
    const INVENTORY_STATUS_FAILURE          = 2;



    public function __construct($BP)
    {
        $this->BP = $BP;
    }

    public function get($which, $data, $sid, $systems)
    {
        $returnArray = array();
        $returnArray['inventory'] = false;

        if ($which == -1)
        {
            // GET /api/inventory/?sid={sid}
            $returnArray['inventory'] = $this->getInventoryTree( $sid, $data , $systems);
        }
        else //if (is_string($which[0]))
        {
            switch ($which[0])
            {
                case 'settings':
                    // GET /api/inventory/settings/?sid={sid}
                    $returnArray['inventory'] = $this->getInventorySettings( $sid );
                    break;
                case 'status':
                    // GET /api/inventory/status/?sid={sid}
                    $returnArray['inventory'] = $this->getInventoryStatus( $sid );
                    break;
                case 'sync':
                    // GET /api/inventory/sync/?sid={sid}
                    $returnArray = $this->getInventorySync( $sid );
                    break;
                case 'instance':
                    // GET /api/inventory/instance/?sid={sid}&instances=(a,b,c,d)
                    $returnArray = $this->getInventoryInstance( $sid );
                    break;
                default:
                    // GET /api/inventory/{id}/...
                    $nodeID = $which[0];
                    if ( count($which) > 1 )
                    {
                        switch ($which[1])
                        {
                            case Inventory::INVENTORY_GET_ALL:
                                $returnArray['inventory'] = $this->getInventoryTree( $sid, $data, $systems, $nodeID, Inventory::INVENTORY_GET_ALL );
                                break;
                            case Inventory::INVENTORY_GET_CHILDREN:
                                $returnArray['inventory'] = $this->getInventoryTree( $sid, $data, $systems, $nodeID, Inventory::INVENTORY_GET_CHILDREN );
                                break;
                            case 'settings':
                                // GET /api/inventory/{id}/settings?sid={sid}
                                $returnArray['inventory'] = $this->getInventorySettings( $sid, $nodeID ,$systems);
                                break;
                            case 'status':
                                // GET /api/inventory/{id}/status/?sid={sid}
                                $returnArray['inventory'] = $this->getInventoryStatus( $sid, $nodeID );
                                break;
                        }
                    }
                    else
                    {
                        $returnArray['inventory'] = $this->getInventoryTree( $sid, $data, $systems, $nodeID, Inventory::INVENTORY_GET_ALL );
                    }
                    break;
            }

        }
        if ( $returnArray['inventory'] === false )
        {
            $returnArray = false;
        }
        return($returnArray);
    }

    public function put($which, $data) {
        $status = false;
        $filter = NULL;
        if ($which != -1 and is_string($which[0]))
        {
            // PUT /api/inventory/{app}/?sid={id}
            if ( $which[0] == 'VMware' or $which[0] == 'Hyper-V' )
            {
                $filter = array($which[0]);
            }
            /* Not needed for first out
            if (isset($which[1]) and is_numeric($which[1]))
            {
                // PUT /api/inventory/{app}/{id}/?sid={id}
            }*/
        }
        if (isset($data['sid']))
        {
            $sysID = $data['sid'];
            if($this->BP->put_inventory($filter, $sysID))
            {
                $status = true;
            }
        }
        else
        {
            if($this->BP->put_inventory($filter))
            {
                $status = true;
            }
        }
        return $status;
    }

    private function getInventoryTree( $sid = false , $data = NULL, $systems = NULL, $node_id = NULL, $returnChildrenOrAll = Inventory::INVENTORY_GET_ALL )
    {
        $relevantClients = $this->getRelevantClients($sid, $node_id, $systems);
        $inventoryTreeArray = array();
        $returnTreeArray = false;

        $getDisks = false;  //whether or not to get info about VM disks for VMware
        if ($data !== NULL and array_key_exists('disks', $data)) {
            $getDisks = ((int)$data['disks'] === 1);
        }

        $specifiedClientID = NULL;
        $specifiedApplicationID = NULL;
        $specifiedInstanceID = NULL;
        $specifiedEsxServerUUID = NULL;
        $specifiedVMwareKey = NULL;
        $specifiedDiskID = NULL;
        if ($node_id !== NULL)
        {
            // Because getRelevantClients filters on $systemID, it is not needed here
            // Because getRelevantClients does not throw out any applications, $applicationID is still needed
            $inventoryNodeIDArray = $this->getNodeIDArrayFromInventoryNodeID($node_id);
            if (array_key_exists('clientID', $inventoryNodeIDArray))
            {
                $specifiedClientID = $inventoryNodeIDArray['clientID'];
                if (array_key_exists('applicationID', $inventoryNodeIDArray))
                {
                    $specifiedApplicationID = $inventoryNodeIDArray['applicationID'];
                    if (array_key_exists('instanceID', $inventoryNodeIDArray))
                    {
                        $specifiedInstanceID = $inventoryNodeIDArray['instanceID'];
                    }
                    elseif (array_key_exists('esxServerUUID', $inventoryNodeIDArray))
                    {
                        $specifiedEsxServerUUID = $inventoryNodeIDArray['esxServerUUID'];
                        if (array_key_exists('instanceID', $inventoryNodeIDArray))
                        {
                            $specifiedInstanceID = $inventoryNodeIDArray['instanceID'];
                            if (array_key_exists('diskID', $inventoryNodeIDArray))
                            {
                                $specifiedDiskID = $inventoryNodeIDArray['diskID'];
                            }
                        } elseif
                        (array_key_exists('VMwareKey', $inventoryNodeIDArray))
                        {
                            $specifiedVMwareKey = $inventoryNodeIDArray['VMwareKey'];
                        }
                    }
                }
            }
        }

        foreach ($relevantClients as $systemID => $system)
        {
            $inventoryTreeArray[$systemID] = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                Constants::HV_UNKNOWN,
                $systemID,
                (string)$system['systemName'],
                //Not really sure what Node type a system is, but I'm calling it a datacenter for the present
                Constants::NODE_TYPE_DATACENTER);

            if (count($system['windowsClients']) > 0)
            {
                foreach ($system['windowsClients'] as $windowsClient) {
                    if ($specifiedClientID === NULL or $specifiedClientID === $windowsClient['id'])
                    {
                        $inventoryTreeArray[$systemID]['nodes'][$windowsClient['id']] = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                            Constants::HV_WINDOWS,
                            $this->buildInventoryNodeID($systemID, $windowsClient['id']),
                            $windowsClient['name'],
                            Constants::NODE_TYPE_COMPUTER);

                        foreach ($windowsClient['applications'] as $applicationID => $windowsApplication)
                        {
                            if ($specifiedApplicationID === NULL or $specifiedApplicationID === $applicationID)
                            {
                                // The check to see that it is a Hyper-V is only necessary if $specifiedApplicationID is NULL
                                if ($windowsApplication['type'] == Constants::APPLICATION_TYPE_NAME_HYPER_V)
                                {
                                    $hyperVArray = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                        Constants::HV_HYPER_V,
                                        $this->buildInventoryNodeID($systemID, $windowsClient['id'], $applicationID),
                                        $windowsClient['name'] . "'s " . $windowsApplication['name'],
                                        Constants::NODE_TYPE_HOST);

                                    $hyperVInfo = $this->BP->get_hyperv_info($windowsClient['id'], true, $systemID);
                                    // Sort VMs alphabetically
                                    $hyperVInfo = $this->sort($hyperVInfo);

                                    foreach ($hyperVInfo as $hyperVVM)
                                    {
                                        //Is instance_id the right value or should it be guid?
                                        if ($specifiedInstanceID === NULL or $specifiedInstanceID === $hyperVVM['instance_id'])
                                        {
                                            $hyperVVMArray = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                                Constants::HV_HYPER_V,
                                                $this->buildInventoryNodeID($systemID, $windowsClient['id'], $applicationID, $hyperVVM['instance_id']),
                                                $hyperVVM['name'],
                                                Constants::NODE_TYPE_VM);
                                            $hyperVArray['nodes'][] = $hyperVVMArray;
                                            if ($specifiedInstanceID !== NULL)
                                            {
                                                $returnTreeArray = $hyperVVMArray;
                                            }
                                        }
                                    }
                                    if ($specifiedApplicationID === NULL)
                                    {
                                        $inventoryTreeArray[$systemID]['nodes'][$windowsClient['id']]['nodes'][] = $hyperVArray;
                                    }
                                    elseif ($specifiedInstanceID === NULL)
                                    {
                                        if ($returnChildrenOrAll === Inventory::INVENTORY_GET_CHILDREN)
                                        {
                                            $returnTreeArray = $hyperVArray['nodes'];
                                        }
                                        else
                                        {
                                            $returnTreeArray = $hyperVArray;
                                        }
                                    }
                                }
                            }
                        }
                        if ($specifiedClientID !== NULL and $returnTreeArray === false)
                        {
                            if ($returnChildrenOrAll === Inventory::INVENTORY_GET_CHILDREN)
                            {
                                $returnTreeArray = $inventoryTreeArray[$systemID]['nodes'][$windowsClient['id']]['nodes'];
                            }
                            else
                            {
                                $returnTreeArray = $inventoryTreeArray[$systemID]['nodes'][$windowsClient['id']];
                            }
                        }
                    }
                }
                $inventoryTreeArray[$systemID]['nodes'] = array_values($inventoryTreeArray[$systemID]['nodes']);
            }

            if (count($system['VMwareClients']) > 0)
            {
                foreach ($system['VMwareClients'] as $VMwareClient)
                {
                    foreach ($VMwareClient['applications'] as $applicationID => $VMwareApplication)
                    {
                        if ($specifiedApplicationID === NULL or $specifiedApplicationID === $applicationID)
                        {
                            if ($VMwareApplication['type'] === Constants::APPLICATION_TYPE_NAME_VMWARE)
                            {
                                foreach ($VMwareApplication['servers'] as $esxServerUUID => $esxServerName)
                                {
                                    if ($specifiedEsxServerUUID === NULL or $specifiedEsxServerUUID === $esxServerUUID)
                                    {
                                        $esxServer = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                            Constants::HV_VMWARE,
                                            $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, NULL, $esxServerUUID),
                                            $esxServerName,
                                            Constants::NODE_TYPE_HOST);
                                        if ($specifiedInstanceID !== NULL)
                                        {
                                            $vmList = $this->BP->get_vm_info($esxServerUUID, (string)$specifiedInstanceID, true, $getDisks, $systemID);
                                        }
                                        else
                                        {
                                            $vmList = $this->BP->get_vm_info($esxServerUUID, NULL, true, $getDisks, $systemID);
                                            $resourcePools = $this->BP->get_resource_pool_info($esxServerUUID, $systemID);
                                            $vApps = $this->BP->get_vApp_info($esxServerUUID, $systemID);

                                            $resourcePoolAndVAppArray = array();
                                            if (count($resourcePools) > 0)
                                            {
                                                foreach ($resourcePools as $resourcePool)
                                                {
                                                    $resourcePoolAndVAppArray[$resourcePool['key']] = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                                        Constants::HV_VMWARE,
                                                        $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, NULL, $esxServerUUID, $resourcePool['key']),
                                                        $resourcePool['name'],
                                                        Constants::NODE_TYPE_RESOURCE_POOL);
                                                    $resourcePoolAndVAppArray[$resourcePool['key']]['parentKey'] = $resourcePool['parentKey'];
                                                    $resourcePoolAndVAppArray[$resourcePool['key']]['parentType'] = $resourcePool['parentType'];
                                                    $resourcePoolAndVAppArray[$resourcePool['key']]['resourcePoolOrVApp'] = array();
                                                }
                                            }
                                            if (count($vApps) > 0)
                                            {
                                                foreach ($vApps as $vApp)
                                                {
                                                    $resourcePoolAndVAppArray[$vApp['key']] = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                                        Constants::HV_VMWARE,
                                                        $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, NULL, $esxServerUUID, $vApp['key']),
                                                        $vApp['name'],
                                                        Constants::NODE_TYPE_VAPP);
                                                    $resourcePoolAndVAppArray[$vApp['key']]['parentKey'] = $vApp['parentKey'];
                                                    $resourcePoolAndVAppArray[$vApp['key']]['parentType'] = $vApp['parentType'];
                                                    $resourcePoolAndVAppArray[$vApp['key']]['resourcePoolOrVApp'] = array();
                                                }
                                            }
                                        }
                                        // Sorts VMs alphabetically
                                        $vmList = $this->sort($vmList);
                                        if (count($vmList) > 0)
                                        {
                                            foreach ($vmList as $vm)
                                            {
                                                if ($specifiedInstanceID === NULL or $specifiedInstanceID === $vm['instance_id'])
                                                {
                                                    if ($vm['template'] === true)
                                                    {
                                                        $esxServer['nodes'][] = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                                            Constants::HV_VMWARE,
                                                            $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, $vm['instance_id'], $esxServerUUID),
                                                            $vm['name'],
                                                            Constants::NODE_TYPE_TEMPLATE);
                                                    }
                                                    else
                                                    {
                                                        $tempNode = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                                            Constants::HV_VMWARE,
                                                            $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, $vm['instance_id'], $esxServerUUID),
                                                            $vm['name'],
                                                            Constants::NODE_TYPE_VM);
                                                        if ($getDisks === true and array_key_exists('disks', $vm) and count($vm['disks']) > 0)
                                                        {
                                                            foreach ($vm['disks'] as $disk)
                                                            {
                                                                if ($specifiedDiskID === NULL or $specifiedDiskID === $disk['disk_id'])
                                                                {
                                                                    $tempNode['nodes'][] = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_TREE,
                                                                        Constants::HV_VMWARE,
                                                                        $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, $vm['instance_id'], $esxServerUUID, NULL, $disk['disk_id']),
                                                                        $disk['name'],
                                                                        Constants::NODE_TYPE_DISK);
                                                                }
                                                            }
                                                        }
                                                        if ($specifiedInstanceID === NULL)
                                                        {
                                                            switch ($vm['parentType'])
                                                            {
                                                                case 0: //ESXhost
                                                                    if ($specifiedVMwareKey === NULL)
                                                                    {
                                                                        $esxServer['nodes'][] = $tempNode;
                                                                        break;
                                                                    }
                                                                case 1: //resourcePool
                                                                case 2: //vApp
                                                                    /*$resourcePoolAndVAppArray[$vm['parentKey']]['VMs'][$vm['instance_id']] = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_TREE,
                                                                        Constants::HV_VMWARE,
                                                                        $this->buildInventoryNodeID( $systemID, $VMwareClient['id'], $applicationID, $vm['instance_id'], $esxServerUUID ),
                                                                        $vm['name'],
                                                                        Constants::NODE_TYPE_VM );*/
                                                                    $resourcePoolAndVAppArray[$vm['parentKey']]['nodes'][] = $tempNode;
                                                                    break;
                                                                case (-1): //template
                                                                    // Nothing should ever come here
                                                                    break;
                                                            }
                                                        }
                                                        else
                                                        {
                                                            if ($specifiedDiskID !== NULL or $returnChildrenOrAll === Inventory::INVENTORY_GET_CHILDREN)
                                                            {
                                                                $returnTreeArray = $tempNode['nodes'];
                                                            }
                                                            else
                                                            {
                                                                $returnTreeArray = $tempNode;
                                                            }
                                                        }
                                                    }
                                                }
                                            }

                                            $esxServerResourcePoolOrVApp = array();
                                            if ($specifiedInstanceID === NULL and count($resourcePoolAndVAppArray) > 0)
                                            {
                                                $nestedResourcePoolOrVAppArray = array();
                                                foreach ($resourcePoolAndVAppArray as $resourcePoolOrVAppKey => $resourcePoolOrVApp)
                                                {
                                                    if ($resourcePoolOrVApp['parentType'] === 0) //ESX is the Parent
                                                    {
                                                        if ($specifiedVMwareKey === NULL or $specifiedVMwareKey === $resourcePoolOrVAppKey)
                                                        {
                                                            unset($resourcePoolOrVApp['parentKey']);
                                                            unset($resourcePoolOrVApp['parentType']);
                                                            $esxServerResourcePoolOrVApp[$resourcePoolOrVAppKey] = $resourcePoolOrVApp;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        unset($resourcePoolOrVApp['parentType']);
                                                        if ($specifiedVMwareKey !== NULL and $specifiedVMwareKey === $resourcePoolOrVAppKey)
                                                        {
                                                            unset($resourcePoolOrVApp['parentKey']);
                                                            $esxServerResourcePoolOrVApp[$resourcePoolOrVAppKey] = $resourcePoolOrVApp;
                                                        }
                                                        else
                                                        {
                                                            $nestedResourcePoolOrVAppArray[$resourcePoolOrVAppKey] = $resourcePoolOrVApp;
                                                        }
                                                    }
                                                }
                                                if (count($nestedResourcePoolOrVAppArray) > 0)
                                                {
                                                    $returnedArray = $this->recursiveResourcePoolAndVAppAssignment($esxServerResourcePoolOrVApp, $nestedResourcePoolOrVAppArray);
                                                    if ($specifiedVMwareKey === NULL)
                                                    {
                                                        $esxServer['nodes'] = array_values(array_merge($returnedArray['parents'], $esxServer['nodes']));
                                                    }
                                                    else
                                                    {
                                                        if ($returnChildrenOrAll === Inventory::INVENTORY_GET_CHILDREN) {
                                                            $returnTreeArray = array_values($returnedArray['parents'][$specifiedVMwareKey]['nodes']);
                                                        }
                                                        else
                                                        {
                                                            $returnTreeArray = array_values($returnedArray['parents']);
                                                        }
                                                    }
                                                }
                                                elseif (count($esxServerResourcePoolOrVApp) > 0)
                                                {
                                                    $newNodesArray = array();
                                                    foreach ( $esxServerResourcePoolOrVApp as $esxServerNodeArray )
                                                    {
                                                        unset($esxServerNodeArray['resourcePoolOrVApp']);
                                                        $newNodesArray[] = $esxServerNodeArray;
                                                    }
                                                    $esxServer['nodes'] = array_values(array_merge($newNodesArray, $esxServer['nodes']));
                                                }
                                            }
                                        }
                                        if ($specifiedEsxServerUUID === NULL)
                                        {
                                            $inventoryTreeArray[$systemID]['nodes'][] = $esxServer;
                                        }
                                        elseif ($specifiedInstanceID === NULL and $specifiedVMwareKey === NULL)
                                        {
                                            if ($returnChildrenOrAll === Inventory::INVENTORY_GET_CHILDREN)
                                            {
                                                $returnTreeArray = $esxServer['nodes'];
                                            }
                                            else
                                            {
                                                $returnTreeArray = $esxServer;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            $inventoryTreeArray[$systemID]['nodes'] = $this->sort($inventoryTreeArray[$systemID]['nodes']);
        }
        if ($returnTreeArray === false)
        {
            $returnTreeArray = array_values($inventoryTreeArray);
        }
        return $returnTreeArray;
    }

    private function getInventorySettings( $sid = false , $node_id = NULL , $systems = NULL)
    {
        $relevantClients = $this->getRelevantClients($sid, $node_id , $systems);
        $settingsReturnArray = array();


        $specifiedApplicationID = NULL;
        $specifiedInstanceID = NULL;
        $specifiedEsxServerUUID = NULL;
        $specifiedVMwareKey = NULL;
        if ($node_id !== NULL)
        {
            // Because getRelevantClients filters on $systemID and $clientID, they are not needed here
            // Because getRelevantClients does not throw out any applications, $applicationID is still needed
            $inventoryNodeIDArray = $this->getNodeIDArrayFromInventoryNodeID($node_id);
            if (array_key_exists('applicationID', $inventoryNodeIDArray))
            {
                $specifiedApplicationID = $inventoryNodeIDArray['applicationID'];
                if (array_key_exists('instanceID', $inventoryNodeIDArray))
                {
                    $specifiedInstanceID = $inventoryNodeIDArray['instanceID'];
                }
                elseif (array_key_exists('esxServerUUID', $inventoryNodeIDArray))
                {
                    $specifiedEsxServerUUID = $inventoryNodeIDArray['esxServerUUID'];
                    if (array_key_exists('instanceID', $inventoryNodeIDArray))
                    {
                        $specifiedInstanceID = $inventoryNodeIDArray['instanceID'];
                    }
                    elseif (array_key_exists('VMwareKey', $inventoryNodeIDArray))
                    {
                        $specifiedVMwareKey = $inventoryNodeIDArray['VMwareKey'];
                    }
                }
            }
        }

        foreach ( $relevantClients as $systemID => $system )
        {
            if ( count($system['windowsClients']) > 0 )
            {
                foreach ( $system['windowsClients'] as $windowsClient )
                {
                    if ( $specifiedApplicationID === NULL )
                    {
                        $tempSettingsInventoryNodeArray = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_SETTINGS,
                            Constants::HV_WINDOWS,
                            $this->buildInventoryNodeID( $systemID, $windowsClient['id'] ),
                            $windowsClient['name'],
                            Constants::NODE_TYPE_COMPUTER );
                        $tempSettingsInventoryNodeArray['credentials'] = isset($windowsClient['credentials']) ? $windowsClient['credentials'] : array();
                        $settingsReturnArray[] = $tempSettingsInventoryNodeArray;
                    }

                    foreach ($windowsClient['applications'] as $applicationID => $windowsApplication)
                    {
                        if ( $specifiedApplicationID === NULL or $specifiedApplicationID === $applicationID )
                        {
                            if ( $windowsApplication['type'] == Constants::APPLICATION_TYPE_NAME_HYPER_V )
                            {
                                /* //Don't need Hyper-V Hosts at the moment
                                $tempSettingsInventoryNodeArray = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_SETTINGS,
                                    Constants::HV_HYPER_V,
                                    $this->buildInventoryNodeID( $systemID, $hyperVClient['id'], $applicationID ),
                                    $hyperVClient['name']."'s ".$hyperVApplication['name'],
                                    Constants::NODE_TYPE_HOST );
                                $tempSettingsInventoryNodeArray['retention'] = $this->getRetentionType( $systemID, $hyperVClient['id'], $applicationID) ;
                                $settingsReturnArray[] = $tempSettingsInventoryNodeArray; */

                                $hyperVInfo = $this->BP->get_hyperv_info($windowsClient['id'], true, $systemID);
                                // Sort VMs alphabetically
                                $hyperVInfo = $this->sort($hyperVInfo);

                                foreach ( $hyperVInfo as $hyperVVM )
                                {
                                    //Is instance_id the right value or should it be guid?
                                    if ( $specifiedInstanceID === NULL or $specifiedInstanceID === $hyperVVM['instance_id'] )
                                    {
                                        $tempSettingsInventoryNodeArray = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_SETTINGS,
                                            Constants::HV_HYPER_V,
                                            $this->buildInventoryNodeID( $systemID, $windowsClient['id'], $applicationID, $hyperVVM['instance_id'] ),
                                            $hyperVVM['name'],
                                            Constants::NODE_TYPE_VM );
                                        $tempSettingsInventoryNodeArray['is_standalone'] = $hyperVVM['is_standalone'];
                                        $tempSettingsInventoryNodeArray['cluster_name'] = $hyperVVM['cluster_name'];
                                        $settingsReturnArray[] = $tempSettingsInventoryNodeArray;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if ( count($system['VMwareClients']) > 0 )
            {
                foreach ( $system['VMwareClients'] as $VMwareClient )
                {
                    foreach ( $VMwareClient['applications'] as $applicationID => $VMwareApplication )
                    {
                        if ( $specifiedApplicationID === NULL or $specifiedApplicationID === $applicationID )
                        {
                            if ( $VMwareApplication['type'] == Constants::APPLICATION_TYPE_NAME_VMWARE )
                            {
                                foreach ( $VMwareApplication['servers'] as $esxServerUUID => $esxServerName )
                                {
                                    if ($specifiedEsxServerUUID === NULL or $specifiedEsxServerUUID === $esxServerUUID)
                                    {
                                        $relevantResourcePoolAndVAppKeys = NULL;
                                        if ( $specifiedVMwareKey !== NULL )
                                        {
                                            $relevantResourcePoolAndVAppKeys = array();
                                            $notRelevantYet = array();
                                            $resourcePoolAndVAppArray = $this->BP->get_resource_pool_info($esxServerUUID, $systemID);
                                            $resourcePoolAndVAppArray = array_merge( $resourcePoolAndVAppArray, $this->BP->get_vApp_info($esxServerUUID, $systemID) );
                                            foreach ( $resourcePoolAndVAppArray as $resourcePoolAndVApp )
                                            {
                                                if ( $resourcePoolAndVApp['key'] === $specifiedVMwareKey or $resourcePoolAndVApp['parentKey'] === $specifiedVMwareKey  )
                                                {
                                                    $relevantResourcePoolAndVAppKeys[] = $resourcePoolAndVApp['key'];
                                                }
                                                elseif ( $resourcePoolAndVApp['parentType'] !== 0 )
                                                {
                                                    $notRelevantYet[] = $resourcePoolAndVApp;
                                                }
                                            }
                                            $changesMade = (count($notRelevantYet) > 0);
                                            $iterator = 0;
                                            while ( $changesMade === true or $iterator !== 0 )
                                            {
                                                if( $iterator === 0)
                                                {
                                                    $changesMade = false;
                                                    $workingArray = $notRelevantYet;
                                                    unset($notRelevantYet);
                                                    $notRelevantYet = array();
                                                }
                                                if ( in_array($workingArray[$iterator]['parentKey'], $relevantResourcePoolAndVAppKeys)  )
                                                {
                                                    $relevantResourcePoolAndVAppKeys[] = $workingArray[$iterator]['key'];
                                                    $changesMade = true;
                                                }
                                                else
                                                {
                                                    $notRelevantYet[] = $workingArray[$iterator];
                                                }
                                                $iterator = $iterator + 1;
                                                if ( $iterator === count($workingArray) )
                                                {
                                                    $iterator = 0;
                                                    $changesMade = ($changesMade and (count($notRelevantYet) > 0));
                                                }
                                            }
                                        }
                                        // Only need settings info for VMs, not disks, vApps, nor Resource Pools
                                        if ($specifiedInstanceID !== NULL)
                                        {
                                            //$get_disks is false because disks aren't needed
                                            $vmList = $this->BP->get_vm_info($esxServerUUID, (string)$specifiedInstanceID, true, false, $systemID);
                                        } else
                                        {
                                            //$get_disks is false because disks aren't needed
                                            $vmList = $this->BP->get_vm_info($esxServerUUID, NULL, true, false, $systemID);
                                        }
                                        // Sorts VMs alphabetically
                                        $vmList = $this->sort($vmList);
                                        if (count($vmList) > 0)
                                        {
                                            foreach ($vmList as $vm)
                                            {
                                                if ( $relevantResourcePoolAndVAppKeys === NULL or in_array($vm['parentKey'], $relevantResourcePoolAndVAppKeys)  )
                                                {
                                                    $tempSettingsInventoryNodeArray = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_SETTINGS,
                                                        Constants::HV_VMWARE,
                                                        $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, $vm['instance_id'], $esxServerUUID),
                                                        $vm['name'],
                                                        Constants::NODE_TYPE_VM);
                                                    $tempSettingsInventoryNodeArray['credentials'] = $vm['credentials'];
                                                    $tempSettingsInventoryNodeArray['app_aware'] = $vm['app_aware'];
                                                    $tempSettingsInventoryNodeArray['template'] = $vm['template'];
                                                    $settingsReturnArray[] = $tempSettingsInventoryNodeArray;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return $settingsReturnArray;
    }

    //$systemID is actually optional, but at the moment it doesn't make sense to not require it
    //REQUIRED: one of $client_id, $app_id, $uuid, $instance_id must not be NULL, check is in the BPL
    private function getRetentionType( $systemID, $client_id = NULL, $app_id = NULL, $uuid = NULL, $instance_id = NULL )
    {
        $filter = array();
        if ( $client_id !== NULL )
        {
            $filter['client_id'] = $client_id;
        }
        if ( $app_id !== NULL )
        {
            $filter['app_id'] = $app_id;
        }
        if ( $uuid !== NULL )
        {
            $filter['uuid'] = $uuid;
        }
        if ( $instance_id !== NULL )
        {
            $filter['instance_id'] = $instance_id;
        }
        $retentionSettingsArray= $this->BP->get_retention_settings($filter, $systemID);

        //Default in the Unitrends Recovery Series and Unitrends Enterprise Backup is that Retention is unset
        $retentionType = Constants::RETENTION_NONE;
        if ( (int)$retentionSettingsArray['legal_hold'] !== 0 )
        {
            $retentionType = Constants::RETENTION_LEGAL_HOLD;
        }
        //If legal_hold is set, then retention_min and retention_max will also have values, so legal_hold takes precedent
        elseif ( (int)$retentionSettingsArray['retention_min'] !== 0 or  (int)$retentionSettingsArray['retention_max'] !== 0 )
        {
            $retentionType = Constants::RETENTION_CUSTOM;
        }
        //Note: Constants::RETENTION_TYPICAL doesn't have meaning at the moment for Unitrends Recovery Series and Unitrends Enterprise Backup

        return $retentionType;

    }

    private function getInventoryStatus( $sid = NULL , $node_id = NULL, $systems = NULL)
    {
        $relevantClients = $this->getRelevantClients($sid, $node_id, $systems, true);
        $statusReturnArray = array();

        $specifiedClientID = NULL;
        $specifiedApplicationID = NULL;
        $specifiedInstanceID = NULL;
        $specifiedEsxServerUUID = NULL;
        $specifiedVMwareKey = NULL;
        if ($node_id !== NULL)
        {
            // Because getRelevantClients filters on $systemID, it is not needed here
            // Because bp_get_backup_status takes a client ID, $clientID is still needed
            // Because getRelevantClients does not throw out any applications, $applicationID is still needed
            $inventoryNodeIDArray = $this->getNodeIDArrayFromInventoryNodeID($node_id);
            if (array_key_exists('clientID', $inventoryNodeIDArray))
            {
                $specifiedClientID = $inventoryNodeIDArray['clientID'];
                if (array_key_exists('applicationID', $inventoryNodeIDArray))
                {
                    $specifiedApplicationID = $inventoryNodeIDArray['applicationID'];
                    if (array_key_exists('instanceID', $inventoryNodeIDArray))
                    {
                        $specifiedInstanceID = $inventoryNodeIDArray['instanceID'];
                    }
                    elseif (array_key_exists('esxServerUUID', $inventoryNodeIDArray))
                    {
                        $specifiedEsxServerUUID = $inventoryNodeIDArray['esxServerUUID'];
                        if (array_key_exists('instanceID', $inventoryNodeIDArray))
                        {
                            $specifiedInstanceID = $inventoryNodeIDArray['instanceID'];
                        }
                        elseif (array_key_exists('VMwareKey', $inventoryNodeIDArray))
                        {
                            $specifiedVMwareKey = $inventoryNodeIDArray['VMwareKey'];
                        }
                    }
                }
            }
        }

        $backup_status_result_format = array();
        $backup_status_result_format['type'] = array( Constants::BACKUP_TYPE_MASTER,
            Constants::BACKUP_TYPE_DIFFERENTIAL,
            Constants::BACKUP_TYPE_INCREMENTAL,
            Constants::BACKUP_TYPE_BAREMETAL,
            Constants::BACKUP_TYPE_SELECTIVE,
            Constants::BACKUP_TYPE_VMWARE_FULL,
            Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL,
            Constants::BACKUP_TYPE_VMWARE_INCREMENTAL,
            Constants::BACKUP_TYPE_HYPER_V_FULL,
            Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL,
            Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL,
            Constants::BACKUP_TYPE_SECURESYNC_MASTER,
            Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL,
            Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL,
            Constants::BACKUP_TYPE_SECURESYNC_BAREMETAL,
            Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL,
            Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL,
            Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL,
            Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL,
            Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL,
            Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL);
        $backup_status_result_format['start_time'] = strtotime('today-6days');
        if ( $specifiedClientID === NULL )
        {
            $backup_status_result_format['grandclients'] = false;
        }

        $archive_status_result_format = array();
        $archive_status_result_format['type'] = array( Constants::BACKUP_TYPE_MASTER,
            Constants::BACKUP_TYPE_DIFFERENTIAL,
            Constants::BACKUP_TYPE_INCREMENTAL,
            Constants::BACKUP_TYPE_BAREMETAL,
            Constants::BACKUP_TYPE_SELECTIVE,
            Constants::BACKUP_TYPE_VMWARE_FULL,
            Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL,
            Constants::BACKUP_TYPE_VMWARE_INCREMENTAL,
            Constants::BACKUP_TYPE_HYPER_V_FULL,
            Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL,
            Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL);
        $archive_status_result_format['job_start_time'] =  $backup_status_result_format['start_time'];

        foreach ( $relevantClients as $systemID => $system )
        {
            if ( $specifiedClientID !== NULL )
            {
                $backup_status_result_format['client_id'] = $specifiedClientID;
                $archive_status_result_format['client_id'] = $specifiedClientID;
            }

            $backup_status_result_format['system_id'] = $systemID;
            $backup_status_result_array = $this->BP->get_backup_status($backup_status_result_format);

            foreach ($backup_status_result_array as $backup)
            {
                    switch ( $backup['type'] )
                    {
                        case Constants::BACKUP_TYPE_MASTER:
                        case Constants::BACKUP_TYPE_DIFFERENTIAL:
                        case Constants::BACKUP_TYPE_INCREMENTAL:
                        case Constants::BACKUP_TYPE_BAREMETAL:
                        case Constants::BACKUP_TYPE_SELECTIVE:
                            if ( array_key_exists( $backup['client_id'], $system['windowsClients'] ) )
                            {
                                if ( array_key_exists('status', $backup) )
                                {
                                    switch ( $backup['status'] )
                                    {
                                        case Constants::BACKUP_STATUS_SUCCESS:
                                            $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['successes']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['successes']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_WARNINGS:
                                            $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['warnings']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['warnings']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_FAILURE:
                                            $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['failures']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['failures']['ids'][] = $backup['id'];
                                            break;
                                    }
                                }
                                else
                                {
                                    $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['incomplete']['count'] += 1;
                                    $system['windowsClients'][$backup['client_id']]['client_backups'][date("w", $backup['start_time'])]['incomplete']['ids'][] = $backup['id'];
                                }
                            }
                            break;
                        case Constants::BACKUP_TYPE_VMWARE_FULL:
                        case Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL:
                        case Constants::BACKUP_TYPE_VMWARE_INCREMENTAL:
                            if ( array_key_exists( $backup['client_id'], $system['VMwareClients'] ) and array_key_exists( 'instance_id', $backup ) )
                            {
                                if ( !array_key_exists( $backup['instance_id'],  $system['VMwareClients'][$backup['client_id']]['vm_backups'] ) )
                                {
                                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                    $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['VMwareClients'][$backup['client_id']]['vm_archives'][$backup['instance_id']] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                                }

                                if ( array_key_exists('status', $backup) )
                                {
                                    switch ( $backup['status'] )
                                    {
                                        case Constants::BACKUP_STATUS_SUCCESS:
                                            $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['count'] += 1;
                                            $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_WARNINGS:
                                            $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['count'] += 1;
                                            $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_FAILURE:
                                            $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['count'] += 1;
                                            $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['ids'][] = $backup['id'];
                                            break;
                                    }
                                }
                                else
                                {
                                    $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['count'] += 1;
                                    $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['ids'][] = $backup['id'];
                                }
                            }
                            break;
                        case Constants::BACKUP_TYPE_HYPER_V_FULL:
                        case Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL:
                        case Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL:
                            if ( array_key_exists( $backup['client_id'], $system['windowsClients'] ) and array_key_exists( 'instance_id', $backup ) )
                            {
                                if ( !array_key_exists( $backup['instance_id'],  $system['windowsClients'][$backup['client_id']]['vm_backups'] ) )
                                {
                                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                    $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['windowsClients'][$backup['client_id']]['vm_archives'][$backup['instance_id']] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                                }

                                if ( array_key_exists('status', $backup) )
                                {
                                    switch ( $backup['status'] )
                                    {
                                        case Constants::BACKUP_STATUS_SUCCESS:
                                            $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_WARNINGS:
                                            $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_FAILURE:
                                            $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['ids'][] = $backup['id'];
                                            break;
                                    }
                                }
                                else
                                {
                                    $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['count'] += 1;
                                    $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['ids'][] = $backup['id'];
                                }
                            }
                            break;
                        case Constants::BACKUP_TYPE_SECURESYNC_MASTER:
                        case Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL:
                        case Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL:
                        case Constants::BACKUP_TYPE_SECURESYNC_BAREMETAL:
                            if ( array_key_exists( $backup['client_id'], $system['windowsClients'] ) )
                            {
                                if ( array_key_exists('status', $backup) )
                                {
                                    switch ( $backup['status'] )
                                    {
                                        case Constants::BACKUP_STATUS_SUCCESS:
                                            $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['successes']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['successes']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_WARNINGS:
                                            $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['warnings']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['warnings']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_FAILURE:
                                            $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['failures']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['failures']['ids'][] = $backup['id'];
                                            break;
                                    }
                                }
                                else
                                {
                                    $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['incomplete']['count'] += 1;
                                    $system['windowsClients'][$backup['client_id']]['client_replications'][date("w", $backup['start_time'])]['incomplete']['ids'][] = $backup['id'];
                                }
                            }
                            break;
                        case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL:
                        case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL:
                        case Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL:
                            if ( array_key_exists( $backup['client_id'], $system['VMwareClients'] ) and array_key_exists( 'instance_id', $backup ) )
                            {
                                if ( !array_key_exists( $backup['instance_id'],  $system['VMwareClients'][$backup['client_id']]['vm_replications'] ) )
                                {
                                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                    $system['VMwareClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['VMwareClients'][$backup['client_id']]['vm_archives'][$backup['instance_id']] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                                }

                                if ( array_key_exists('status', $backup) )
                                {
                                    switch ( $backup['status'] )
                                    {
                                        case Constants::BACKUP_STATUS_SUCCESS:
                                            $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['count'] += 1;
                                            $system['VMwareClients'][$backup['client_id']]['vm_replications'][date("w", $backup['start_time'])]['successes']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_WARNINGS:
                                            $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['count'] += 1;
                                            $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_FAILURE:
                                            $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['count'] += 1;
                                            $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['ids'][] = $backup['id'];
                                            break;
                                    }
                                }
                                else
                                {
                                    $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['count'] += 1;
                                    $system['VMwareClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['ids'][] = $backup['id'];
                                }
                            }
                            break;
                        case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL:
                        case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL:
                        case Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL:
                            if ( array_key_exists( $backup['client_id'], $system['windowsClients'] ) and array_key_exists( 'instance_id', $backup ) )
                            {
                                if ( !array_key_exists( $backup['instance_id'],  $system['windowsClients'][$backup['client_id']]['vm_replications'] ) )
                                {
                                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                    $system['windowsClients'][$backup['client_id']]['vm_backups'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['windowsClients'][$backup['client_id']]['vm_archives'][$backup['instance_id']] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                                }

                                if ( array_key_exists('status', $backup) )
                                {
                                    switch ( $backup['status'] )
                                    {
                                        case Constants::BACKUP_STATUS_SUCCESS:
                                            $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['successes']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_WARNINGS:
                                            $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['warnings']['ids'][] = $backup['id'];
                                            break;
                                        case Constants::BACKUP_STATUS_FAILURE:
                                            $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['count'] += 1;
                                            $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['failures']['ids'][] = $backup['id'];
                                            break;
                                    }
                                }
                                else
                                {
                                    $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['count'] += 1;
                                    $system['windowsClients'][$backup['client_id']]['vm_replications'][$backup['instance_id']][date("w", $backup['start_time'])]['incomplete']['ids'][] = $backup['id'];
                                }
                            }
                            break;

                    }
            }

            $archive_status_result_array = $this->BP->get_archive_status( $archive_status_result_format, $systemID );

            foreach ( $archive_status_result_array as $archive )
            {
                if ( array_key_exists('app_id', $archive) )
                {
                    switch ( $archive['app_id'] )
                    {
                        case Constants::APPLICATION_ID_FILE_LEVEL:

                            if ( array_key_exists('os_type_id', $archive) )
                            {
                                if (   $archive['os_type_id'] == Constants::OS_WIN_16
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_95
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_2000
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_XP
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_2003
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_VISTA
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_2008
                                    or $archive['os_type_id'] == Constants::OS_WIN_2008_R2
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_7
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_8
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_2012
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_8_1
                                    or $archive['os_type_id'] == Constants::OS_WINDOWS_2012_R2 )
                                {
                                    if ( array_key_exists( $archive['client_id'], $system['windowsClients'] ) )
                                    {
                                        if ( $archive['success'] === true )
                                        {
                                            $system['windowsClients'][$archive['client_id']]['client_archives'][date("w", $archive['archive_time'])]['successes']['count'] += 1;
                                            $system['windowsClients'][$archive['client_id']]['client_archives'][date("w", $archive['archive_time'])]['successes']['ids'][] = $archive['archive_id'];
                                        }
                                        else
                                        {
                                            $system['windowsClients'][$archive['client_id']]['client_archives'][date("w", $archive['archive_time'])]['failures']['count'] += 1;
                                            $system['windowsClients'][$archive['client_id']]['client_archives'][date("w", $archive['archive_time'])]['failures']['ids'][] = $archive['archive_id'];
                                        }
                                    }
                                }
                            }
                            break;
                        case Constants::APPLICATION_ID_VMWARE:
                            if ( array_key_exists( $archive['client_id'], $system['VMwareClients'] ) and array_key_exists( 'instance_id', $archive ) )
                            {
                                if ( !array_key_exists( $archive['instance_id'],  $system['VMwareClients'][$archive['client_id']]['vm_archives'] ) )
                                {
                                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                    $system['VMwareClients'][$archive['client_id']]['vm_backups'][$archive['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['VMwareClients'][$archive['client_id']]['vm_replications'][$archive['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['VMwareClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                                }

                                if ( $archive['success'] === true )
                                {
                                    $system['VMwareClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['successes']['count'] += 1;
                                    $system['VMwareClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['successes']['ids'][] = $archive['archive_id'];
                                }
                                else
                                {
                                    $system['VMwareClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['failures']['count'] += 1;
                                    $system['VMwareClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['failures']['ids'][] = $archive['archive_id'];
                                }
                            }
                            break;
                        case Constants::APPLICATION_ID_HYPER_V_2008_R2:
                        case Constants::APPLICATION_ID_HYPER_V_2012:
                            if ( array_key_exists( $archive['client_id'], $system['windowsClients'] ) and array_key_exists( 'instance_id', $archive ) )
                            {
                                if ( !array_key_exists( $archive['instance_id'],  $system['windowsClients'][$archive['client_id']]['vm_archives'] ) )
                                {
                                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                    $system['windowsClients'][$archive['client_id']]['vm_backups'][$archive['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['windowsClients'][$archive['client_id']]['vm_replications'][$archive['instance_id']] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                                    $system['windowsClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                                }

                                if ( $archive['success'] === true )
                                {
                                    $system['windowsClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['successes']['count'] += 1;
                                    $system['windowsClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['successes']['ids'][] = $archive['archive_id'];
                                }
                                else
                                {
                                    $system['windowsClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['failures']['count'] += 1;
                                    $system['windowsClients'][$archive['client_id']]['vm_archives'][$archive['instance_id']][date("w", $archive['archive_time'])]['failures']['ids'][] = $archive['archive_id'];
                                }
                            }
                            break;

                    }
                }
            }



            if ( count($system['windowsClients']) > 0 )
            {
                foreach ( $system['windowsClients'] as $windowsClient )
                {
                    if ( $specifiedApplicationID === NULL )
                    {
                        $tempStatusInventoryNodeArray = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_STATUS,
                            Constants::HV_WINDOWS,
                            $this->buildInventoryNodeID( $systemID, $windowsClient['id'] ),
                            $windowsClient['name'],
                            Constants::NODE_TYPE_COMPUTER );

                        $tempStatusInventoryNodeArray['last_backups'] = array();
                        foreach ( $windowsClient['client_backups'] as $summary_day => $backup_summary )
                        {
                            $backup_summary['day'] = $summary_day;
                            $tempStatusInventoryNodeArray['last_backups'][] = $backup_summary;
                        }

                        $tempStatusInventoryNodeArray['last_replications'] = array();
                        foreach ( $windowsClient['client_replications'] as $summary_day => $replication_summary )
                        {
                            $replication_summary['day'] = $summary_day;
                            $tempStatusInventoryNodeArray['last_replications'][] = $replication_summary;
                        }

                        $tempStatusInventoryNodeArray['last_archives'] = array();
                        foreach ( $windowsClient['client_archives'] as $summary_day => $archive_summary )
                        {
                            $archive_summary['day'] = $summary_day;
                            $tempStatusInventoryNodeArray['last_archives'][] = $archive_summary;
                        }

                        $statusReturnArray[] = $tempStatusInventoryNodeArray;
                    }

                    foreach ($windowsClient['applications'] as $applicationID => $windowsApplication)
                    {
                        if ( $specifiedApplicationID === NULL or $specifiedApplicationID === $applicationID )
                        {
                            // The check to see that it is a Hyper-V is only necessary if $specifiedApplicationID is NULL
                            if ( $windowsApplication['type'] == Constants::APPLICATION_TYPE_NAME_HYPER_V )
                            {
                                /* //Don't need Hyper-V Hosts at the moment
                                $tempSettingsInventoryNodeArray = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_SETTINGS,
                                    Constants::HV_HYPER_V,
                                    $this->buildInventoryNodeID( $systemID, $hyperVClient['id'], $applicationID ),
                                    $hyperVClient['name']."'s ".$hyperVApplication['name'],
                                    Constants::NODE_TYPE_HOST );
                                $tempSettingsInventoryNodeArray['retention'] = $this->getRetentionType( $systemID, $hyperVClient['id'], $applicationID) ;
                                $settingsReturnArray[] = $tempSettingsInventoryNodeArray; */

                                $hyperVInfo = $this->BP->get_hyperv_info($windowsClient['id'], true, $systemID);
                                // Sort VMs alphabetically
                                $hyperVInfo = $this->sort($hyperVInfo);

                                foreach ( $hyperVInfo as $hyperVVM )
                                {
                                    //Is instance_id the right value or should it be guid?
                                    if ( $specifiedInstanceID === NULL or $specifiedInstanceID === $hyperVVM['instance_id'] )
                                    {
                                        $tempStatusInventoryNodeArray = $this->buildInventoryNodeArray( Inventory::INVENTORY_GET_TYPE_STATUS,
                                            Constants::HV_HYPER_V,
                                            $this->buildInventoryNodeID( $systemID, $windowsClient['id'], $applicationID, $hyperVVM['instance_id'] ),
                                            $hyperVVM['name'],
                                            Constants::NODE_TYPE_VM );

                                        $tempStatusInventoryNodeArray['last_backups'] = array();
                                        $tempStatusInventoryNodeArray['last_replications'] = array();
                                        $tempStatusInventoryNodeArray['last_archives'] = array();

                                        if ( array_key_exists($hyperVVM['instance_id'], $windowsClient['vm_backups']) )
                                        {
                                            foreach ( $windowsClient['vm_backups'][$hyperVVM['instance_id']] as $summary_day => $backup_summary )
                                            {
                                                $backup_summary['day'] = $summary_day;
                                                $tempStatusInventoryNodeArray['last_backups'][] = $backup_summary;
                                            }

                                            foreach ( $windowsClient['vm_replications'][$hyperVVM['instance_id']] as $summary_day => $replication_summary )
                                            {
                                                $replication_summary['day'] = $summary_day;
                                                $tempStatusInventoryNodeArray['last_replications'][] = $replication_summary;
                                            }

                                            foreach ( $windowsClient['vm_archives'][$hyperVVM['instance_id']] as $summary_day => $archive_summary )
                                            {
                                                $archive_summary['day'] = $summary_day;
                                                $tempStatusInventoryNodeArray['last_archives'][] = $archive_summary;
                                            }
                                        }
                                        else
                                        {
                                            $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                            $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                            for ( $i = 0 ; $i < 7 ; $i++ )
                                            {
                                                $empty_day['day'] = $i;
                                                $tempStatusInventoryNodeArray['last_backups'][] = $empty_day;
                                                $tempStatusInventoryNodeArray['last_replications'][] = $empty_day;

                                                $empty_day_archive['day'] = $i;
                                                $tempStatusInventoryNodeArray['last_archives'][] = $empty_day_archive;
                                            }
                                        }

                                        $statusReturnArray[] = $tempStatusInventoryNodeArray;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if ( count($system['VMwareClients']) > 0 )
            {
                foreach ( $system['VMwareClients'] as $VMwareClient )
                {
                    foreach ( $VMwareClient['applications'] as $applicationID => $VMwareApplication )
                    {
                        if ( $specifiedApplicationID === NULL or $specifiedApplicationID === $applicationID )
                        {
                            if ( $VMwareApplication['type'] == Constants::APPLICATION_TYPE_NAME_VMWARE )
                            {
                                foreach ( $VMwareApplication['servers'] as $esxServerUUID => $esxServerName )
                                {
                                    if ($specifiedEsxServerUUID === NULL or $specifiedEsxServerUUID === $esxServerUUID)
                                    {
                                        $relevantResourcePoolAndVAppKeys = NULL;
                                        if ( $specifiedVMwareKey !== NULL )
                                        {
                                            $relevantResourcePoolAndVAppKeys = array();
                                            $notRelevantYet = array();
                                            $resourcePoolAndVAppArray = $this->BP->get_resource_pool_info($esxServerUUID, $systemID);
                                            $resourcePoolAndVAppArray = array_merge( $resourcePoolAndVAppArray, $this->BP->get_vApp_info($esxServerUUID, $systemID) );
                                            foreach ( $resourcePoolAndVAppArray as $resourcePoolAndVApp )
                                            {
                                                if ( $resourcePoolAndVApp['key'] === $specifiedVMwareKey or $resourcePoolAndVApp['parentKey'] === $specifiedVMwareKey  )
                                                {
                                                    $relevantResourcePoolAndVAppKeys[] = $resourcePoolAndVApp['key'];
                                                }
                                                elseif ( $resourcePoolAndVApp['parentType'] !== 0 )
                                                {
                                                    $notRelevantYet[] = $resourcePoolAndVApp;
                                                }
                                            }
                                            $changesMade = (count($notRelevantYet) > 0);
                                            $iterator = 0;
                                            while ( $changesMade === true or $iterator !== 0 )
                                            {
                                                if( $iterator === 0)
                                                {
                                                    $changesMade = false;
                                                    $workingArray = $notRelevantYet;
                                                    unset($notRelevantYet);
                                                    $notRelevantYet = array();
                                                }
                                                if ( in_array($workingArray[$iterator]['parentKey'], $relevantResourcePoolAndVAppKeys)  )
                                                {
                                                    $relevantResourcePoolAndVAppKeys[] = $workingArray[$iterator]['key'];
                                                    $changesMade = true;
                                                }
                                                else
                                                {
                                                    $notRelevantYet[] = $workingArray[$iterator];
                                                }
                                                $iterator = $iterator + 1;
                                                if ( $iterator === count($workingArray) )
                                                {
                                                    $iterator = 0;
                                                    $changesMade = ($changesMade and (count($notRelevantYet) > 0));
                                                }
                                            }
                                        }
                                        // Only need settings info for VMs, not disks, vApps, nor Resource Pools
                                        if ($specifiedInstanceID !== NULL)
                                        {
                                            //$get_disks is false because disks aren't needed
                                            $vmList = $this->BP->get_vm_info($esxServerUUID, (string)$specifiedInstanceID, true, false, $systemID);
                                        } else
                                        {
                                            //$get_disks is false because disks aren't needed
                                            $vmList = $this->BP->get_vm_info($esxServerUUID, NULL, true, false, $systemID);
                                        }
                                        // Sort VMs alphabetically
                                        $vmList = $this->sort($vmList);
                                        if (count($vmList) > 0)
                                        {
                                            foreach ($vmList as $vm)
                                            {
                                                if ( $relevantResourcePoolAndVAppKeys === NULL or in_array($vm['parentKey'], $relevantResourcePoolAndVAppKeys)  )
                                                {
                                                    $tempStatusInventoryNodeArray = $this->buildInventoryNodeArray(Inventory::INVENTORY_GET_TYPE_STATUS,
                                                        Constants::HV_VMWARE,
                                                        $this->buildInventoryNodeID($systemID, $VMwareClient['id'], $applicationID, $vm['instance_id'], $esxServerUUID),
                                                        $vm['name'],
                                                        Constants::NODE_TYPE_VM);

                                                    $tempStatusInventoryNodeArray['last_backups'] = array();
                                                    $tempStatusInventoryNodeArray['last_replications'] = array();
                                                    $tempStatusInventoryNodeArray['last_archives'] = array();

                                                    if ( array_key_exists($vm['instance_id'], $VMwareClient['vm_backups']) )
                                                    {
                                                        foreach ( $VMwareClient['vm_backups'][$vm['instance_id']] as $summary_day => $backup_summary )
                                                        {
                                                            $backup_summary['day'] = $summary_day;
                                                            $tempStatusInventoryNodeArray['last_backups'][] = $backup_summary;
                                                        }

                                                        foreach ( $VMwareClient['vm_replications'][$vm['instance_id']] as $summary_day => $replication_summary )
                                                        {
                                                            $replication_summary['day'] = $summary_day;
                                                            $tempStatusInventoryNodeArray['last_replications'][] = $replication_summary;
                                                        }

                                                        foreach ( $VMwareClient['vm_archives'][$vm['instance_id']] as $summary_day => $archive_summary )
                                                        {
                                                            $archive_summary['day'] = $summary_day;
                                                            $tempStatusInventoryNodeArray['last_archives'][] = $archive_summary;
                                                        }
                                                    }
                                                    else
                                                    {
                                                        $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                                                        $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                                                        for ( $i = 0 ; $i < 7 ; $i++ )
                                                        {
                                                            $empty_day['day'] = $i;
                                                            $tempStatusInventoryNodeArray['last_backups'][] = $empty_day;
                                                            $tempStatusInventoryNodeArray['last_replications'][] = $empty_day;

                                                            $empty_day_archive['day'] = $i;
                                                            $tempStatusInventoryNodeArray['last_archives'][] = $empty_day_archive;
                                                        }
                                                    }

                                                    $statusReturnArray[] = $tempStatusInventoryNodeArray;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return $statusReturnArray;
    }

    private function getInventorySync( $sid )
    {
        require_once('summary.php');
        $summary = new Summary( $this->BP, $sid );
        return $summary->get("current", "");
    }

    private function buildInventoryNodeArray( $inventoryGetType, $hv, $id, $name, $type )
    {
        $inventoryNodeArray = array();
        $inventoryNodeArray['hv']    = $hv;
        $inventoryNodeArray['id']    = $id;
        $inventoryNodeArray['name']  = $name;
        $inventoryNodeArray['type']  = $type;
        switch ($inventoryGetType)
        {
            case (Inventory::INVENTORY_GET_TYPE_TREE):
                $inventoryNodeArray['nodes']  = array();
                break;
            case (Inventory::INVENTORY_GET_TYPE_SETTINGS):
                $inventoryNodeArray['credentials']  = array();  //TBD = NULL;
                $inventoryNodeArray['is_standalone']  = false;
                $inventoryNodeArray['app_aware']  = false;
                $inventoryNodeArray['template']  = false;
                $inventoryNodeArray['cluster_name'] = "";
                break;
            case (Inventory::INVENTORY_GET_TYPE_STATUS):
                $inventoryNodeArray['archive_jobs']  = array(); //Waiting on Job Order Stuff
                $inventoryNodeArray['last_archives']  = array();

                $inventoryNodeArray['backup_jobs']  = array(); //Waiting on Job Order Stuff
                $inventoryNodeArray['last_backups']  = array();
                //$inventoryNodeArray['backup_next_run']  = "--"; //Waiting on Job Order Stuff

                $inventoryNodeArray['replication_jobs']  = array();//Waiting on Job Order Stuff
                $inventoryNodeArray['last_replications']  = array();
                break;
        }

        return $inventoryNodeArray;
    }


    //Three options for a NodeID - VMware Resource Pool or vApp, VMware vm, or not VMware
    //Not VMware option: $systemID_$clientID_$applicationID_$instanceID
    //VMware VM option: $systemID_$clientID_$applicationID_$esxServerUUID_$instanceID_$diskID
    //VMware Resource Pool or vApp option: $systemID_$clientID_$applicationID_$esxServerUUID_$VMwareKey
    //Do NOT give both a $VMwareKey and an $instanceID
    private function getNodeIDArrayFromInventoryNodeID( $inventoryNodeId )
    {
        $returnArray = array();
        if ( $inventoryNodeId !== NULL )
        {
            $tempArray = explode( Inventory::INVENTORY_NODE_SEPARATOR, $inventoryNodeId );
            if ( count($tempArray) > 0 and is_numeric($tempArray[0]) )
            {
                $returnArray['systemID'] = (int)$tempArray[0];
                if ( count($tempArray) > 1 and is_numeric($tempArray[1]) )
                {
                    $returnArray['clientID'] = (int)$tempArray[1];
                    if ( count($tempArray) > 2 and is_numeric($tempArray[2]) )
                    {
                        $returnArray['applicationID'] = (int)$tempArray[2];
                        if ( count($tempArray) > 3 )
                        {
                            if ( $returnArray['applicationID'] === Constants::APPLICATION_ID_VMWARE )
                            {
                                $returnArray['esxServerUUID'] = (string)$tempArray[3];
                                if ( count($tempArray) > 4 )
                                {
                                    //VMware VM option: $systemID_$clientID_$applicationID_$esxServerUUID_$instanceID_$diskID
                                    if ( is_numeric($tempArray[4]) )
                                    {
                                        $returnArray['instanceID'] = (int)$tempArray[4];
                                        if ( count($tempArray) >5  and is_numeric($tempArray[5]) )
                                        {
                                            $returnArray['diskID'] = (int)$tempArray[5];
                                        }
                                    }
                                    //VMware Resource Pool or vApp option: $systemID_$clientID_$applicationID_$esxServerUUID_$VMwareKey
                                    //Make sure that $VMwareKey are never numeric
                                    else
                                    {
                                        $returnArray['VMwareKey'] = (string)$tempArray[4];
                                    }
                                }
                            }
                            //Not VMware option: $systemID_$clientID_$applicationID_$instanceID
                            else
                            {
                                $returnArray['instanceID'] = (int)$tempArray[3];
                            }
                        }
                    }
                }
            }
        }
        return $returnArray;
    }

    //Three options for a NodeID - VMware Resource Pool or vApp, VMware vm, or not VMware
    //Not VMware option: $systemID_$clientID_$applicationID_$instanceID
    //VMware VM option: $systemID_$clientID_$applicationID_$esxServerUUID_$instanceID_$diskID
    //VMware Resource Pool or vApp option: $systemID_$clientID_$applicationID_$esxServerUUID_$VMwareKey
    //Do NOT give both a $VMwareKey and an $instanceID
    private function buildInventoryNodeIDFromNodeIDArray( $NodeIDArray )
    {
        $returnID = "";
        //Function no longer works and is not in use, so commenting out for now
        /*
        if ( $NodeIDArray !== NULL  and array_key_exists('systemID', $NodeIDArray) )
        {
            $returnID = (string)$NodeIDArray['systemID'];
            if ( array_key_exists('clientID', $NodeIDArray) )
            {
                $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$NodeIDArray['clientID'];
                if ( array_key_exists('applicationID', $NodeIDArray) )
                {
                    $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$NodeIDArray['applicationID'];
                    if ( array_key_exists('instanceID', $NodeIDArray) )
                    {
                        $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$NodeIDArray['instanceID'];
                        if (  array_key_exists('diskID', $NodeIDArray) )
                        {
                            $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$NodeIDArray['diskID'];
                        }
                    }
                }
            }
        }*/
        return $returnID;
    }

    //Three options for a NodeID - VMware Resource Pool or vApp, VMware vm, or not VMware
    //Not VMware option: $systemID_$clientID_$applicationID_$instanceID
    //VMware VM option: $systemID_$clientID_$applicationID_$esxServerUUID_$instanceID_$diskID
    //VMware Resource Pool or vApp option: $systemID_$clientID_$applicationID_$esxServerUUID_$VMwareKey
    //Do NOT give both a $VMwareKey and an $instanceID
    private function buildInventoryNodeID( $systemID, $clientID = NULL, $applicationID = NULL, $instanceID = NULL, $esxServerUUID = NULL, $VMwareKey = NULL ,$diskID = NULL )
    {
        $returnID = "";
        if ( $systemID !== NULL )
        {
            $returnID = (string)$systemID;
            if ( $clientID !== NULL )
            {
                $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$clientID;
                if ( $applicationID !== NULL )
                {
                    $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$applicationID;
                    if ( (int)$applicationID === Constants::APPLICATION_ID_VMWARE and $esxServerUUID !== NULL )
                    {
                        $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$esxServerUUID;
                        //VMware VM option: $systemID_$clientID_$applicationID_$esxServerUUID_$instanceID_$diskID
                        if ( $instanceID !== NULL )
                        {
                            $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$instanceID;
                            if (  $diskID !== NULL )
                            {
                                $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$diskID;
                            }
                        }
                        //VMware Resource Pool or vApp option: $systemID_$clientID_$applicationID_$esxServerUUID_$VMwareKey
                        elseif ( $VMwareKey !== NULL )
                        {
                            $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$VMwareKey;
                        }
                    }
                    //Not VMware option: $systemID_$clientID_$applicationID_$instanceID
                    elseif ( $instanceID !== NULL )
                    {
                        $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$instanceID;
                        if (  $diskID !== NULL )
                        {
                            $returnID = $returnID.(Inventory::INVENTORY_NODE_SEPARATOR).(string)$diskID;
                        }
                    }
                }
            }
        }
        return $returnID;
    }

    // $do_status_arrays adds empty arrays to be filled in by GET /inventory/status
    private function getRelevantClients( $sid = false , $node_id = NULL, $systems= NULL, $do_status_arrays = false )
    {
        $specifiedSystemID = NULL;
        $specifiedClientID = NULL;
        if ( $node_id !== NULL )
        {
            $inventoryNodeIDArray = $this->getNodeIDArrayFromInventoryNodeID($node_id);
            if ( array_key_exists('systemID', $inventoryNodeIDArray) )
            {
                $specifiedSystemID = $inventoryNodeIDArray['systemID'];
                if ( array_key_exists('clientID', $inventoryNodeIDArray) )
                {
                    $specifiedClientID = $inventoryNodeIDArray['clientID'];
                }
            }
        }
        if ( $sid !== false  )
        {
            //Add system Name - Do this
            $systemList = array($sid => '');
        }
        elseif ( $specifiedSystemID !== NULL )
        {
            //Add system Name - Do this
            $systemList = array($specifiedSystemID => '');
        }
        else
        {
            $systemList = $systems;
        }
        foreach ($systemList as $systemID => $systemName)
        {
            $systems[$systemID] = array();
            //$systems[$systemID]['clients'] = array();  //Not needed at the moment, used if underlying function needs a list of clients
            $systems[$systemID]['systemName'] = $systemName;
            $systems[$systemID]['windowsClients'] = array();
            $systems[$systemID]['VMwareClients'] = array();

            if ( $specifiedClientID !== NULL )
            {
                $clients = array($specifiedClientID => '');
            }
            else
            {
                $clients = $this->BP->get_client_list($systemID);
            }
            foreach ($clients as $clientID => $clientName)
            {
                //$isClientRelevant = false;
                $clientInfo = $this->BP->get_client_info($clientID, $systemID);
                if ( $do_status_arrays )
                {
                    $empty_day = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()), 'warnings' => array('count' => 0, 'ids' => array()), 'incomplete' => array('count' => 0, 'ids' => array()) );
                    $empty_day_archive = array( 'successes' => array('count' => 0, 'ids' => array()), 'failures' => array('count' => 0, 'ids' => array()) );
                    $clientInfo['client_backups'] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                    $clientInfo['client_replications'] = array( 0=>$empty_day, 1=>$empty_day, 2=>$empty_day, 3=>$empty_day, 4=>$empty_day, 5=>$empty_day, 6=>$empty_day );
                    $clientInfo['client_archives'] = array( 0=>$empty_day_archive, 1=>$empty_day_archive, 2=>$empty_day_archive, 3=>$empty_day_archive, 4=>$empty_day_archive, 5=>$empty_day_archive, 6=>$empty_day_archive );
                    $clientInfo['vm_backups'] = array();
                    $clientInfo['vm_replications'] = array();
                    $clientInfo['vm_archives'] = array();
                }

                if (   $clientInfo['os_type_id'] == Constants::OS_WIN_16
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_95
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2000
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_XP
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2003
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_VISTA
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2008
                    or $clientInfo['os_type_id'] == Constants::OS_WIN_2008_R2
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_7
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_8
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2012
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_8_1
                    or $clientInfo['os_type_id'] == Constants::OS_WINDOWS_2012_R2 )
                {
                    $systems[$systemID]['windowsClients'][$clientID] = $clientInfo;
                    //$isClientRelevant = true;
                }
                //This is unnecessary because hyperv clusters are all Windows Clients
                elseif ( $clientInfo['is_hyperv_cluster'] === true )
                {
                    $systems[$systemID]['windowsClients'][$clientID] = $clientInfo;
                    //$isClientRelevant = true;
                }
                else
                {
                    foreach ($clientInfo['applications'] as $applicationID => $applicationInfo)
                    {
                        if ( $applicationInfo['type'] === Constants::APPLICATION_TYPE_NAME_VMWARE )
                        {
                            $systems[$systemID]['VMwareClients'][$clientID] = $clientInfo;
                            //$isClientRelevant = true;
                            break;
                        }
                    }
                }

                /*if ( $isClientRelevant == true )
                {
                    $systems[$systemID]['clients'][] = $clientID;
                }*/
            }
        }
        return($systems);

    }

    //Double check this logic with multi tiers of recursion.
    private function recursiveResourcePoolAndVAppAssignment($parents, $children)
    {
        $newChildren = array();
        if ( count($children) > 0 )
        {
            foreach ( $children as $childKey => $child )
            {
                if ( array_key_exists($child['parentKey'], $parents) )
                {
                    $parents[$child['parentKey']]['resourcePoolOrVApp'][$childKey] = $child;
                    unset($parents[$child['parentKey']]['resourcePoolOrVApp'][$childKey]['parentKey']);
                }
                else
                {
                    $newChildren[$childKey] = $child;
                }
            }
            if ( count($newChildren) > 0 )
            {
                foreach ( $parents as $parentKey => $parent )
                {
                    if ( count($parent['resourcePoolOrVApp']) > 0 )
                    {
                        $returnedArray = $this->recursiveResourcePoolAndVAppAssignment( $parent['resourcePoolOrVApp'], $newChildren );
                        $parents[$parentKey]['resourcePoolOrVApp'] = $returnedArray['parents'];
                        $newChildren = $returnedArray['children'];
                    }
                    /*else
                    {
                        unset($parents[$parentKey]['resourcePoolOrVApp']);
                    }*/
                }
            }
            else
            {
                foreach ( $parents as $parentKey => $parent )
                {
                    foreach ( $parent['resourcePoolOrVApp'] as $childKey => $child )
                    {
                        $parents[$parentKey]['resourcePoolOrVApp'][$childKey]['nodes'] = array_values(array_merge($parents[$parentKey]['resourcePoolOrVApp'][$childKey]['resourcePoolOrVApp'], $parents[$parentKey]['resourcePoolOrVApp'][$childKey]['nodes']));
                        unset($parents[$parentKey]['resourcePoolOrVApp'][$childKey]['resourcePoolOrVApp']);
                    }
                }
            }
        }
        foreach ( $parents as $parentKey => $parent )
        {
            $parents[$parentKey]['nodes'] = array_values(array_merge($parents[$parentKey]['resourcePoolOrVApp'], $parents[$parentKey]['nodes']));
            unset($parents[$parentKey]['resourcePoolOrVApp']);
        }

        return( array('parents' => $parents, 'children' => $newChildren) );
    }

    private function getInventoryInstance( $sid ){
        $instances = $_GET['instances'];
        $instancesArray = explode(",", $instances);
        foreach($instancesArray as $key => $instance){
            $Instanceinfo = $this->BP->get_appinst_info($instance,$sid);
            foreach($Instanceinfo as &$val){
                if($val['app_type'] === "VMware") {
                    $val['client_name'] = $val['primary_name'];
                    $val['vm_name'] = $val['secondary_name'];
                    unset($val['primary_name']);
                    unset($val['secondary_name']);
                }
                elseif($val['app_type'] === "Hyper-V"){
                    $val['vm_name'] = $val['primary_name'];
                    unset($val['primary_name']);
                    unset($val['secondary_name']);
                }
            }
          $info['instances'][] = $Instanceinfo;

        }
        return $info;
    }

    // Sorts the Array alphabetically
    private function sort($Array) {
        $Info = $Array;
        $orderByName = array();
        foreach ($Info as $key => $row) {
            $orderByName[$key] = strtolower($row['name']);
        }
        array_multisort($orderByName, SORT_STRING, $Info);
        return $Info;
    }

}

?>