<?php

class JobsHistory
{
    private $BP;

    public function __construct($BP, $sid, $Functions)
    {
        $this->BP = $BP;
        $this->now = time();
		$this->sid = $sid;
	    $this->itemCount = 0;

        $this->functions = $Functions;

        require_once('jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get_jobs_history($which, $filter, $sid)
    {
        $allJobs = array();
        $startDate = isset($_GET['start_date']) ? strtotime($_GET['start_date']) : strtotime(Constants::DATE_ONE_WEEK_AGO);
        $endDate = isset($_GET['end_date']) ? $this->functions->formatEndDate($_GET['end_date']) : strtotime('now');
        // $endDate = isset($_GET['end_date']) ? $this->calcEndDate(($_GET['end_date'])) : strtotime('now');
        $failed = isset($_GET['result']) ? $_GET['result'] : null;
        $includeOnDemand = isset($_GET['include_on_demand']) == 'true' ? true : false;
        $includeDisabled = isset($GET['include_disabled']) === 'true' ? true : false;

        //did they pass in sid, cid, or iid?
        if ($sid !== false) {
            $historyFilter = array(
                'start_time' => $startDate,
                'end_time' => $endDate,
                'include_on_demand' => $includeOnDemand,
                'include_disabled' => $includeDisabled
            );

            $name = $this->functions->getSystemNameFromID($sid);
            $jobHistory = $this->BP->get_schedule_history($historyFilter, $sid);

            foreach ($jobHistory as $history) {
                $data = $this->buildHistoryOutput($history, $name, $sid, null, $which, $startDate, $failed);
                $filtered = $this->buildReturn($data, $filter);
                if ($filtered != NULL) {
                    $allJobs[] = $filtered;
                }
            }
        } else {
            $systems = $this->functions->selectSystems();
            foreach ($systems as $sid => $name) {
                $historyFilter = array(
                    'start_time' => $startDate,
                    'end_time' => $endDate,
                    'include_on_demand' => $includeOnDemand,
                    'include_disabled' => $includeDisabled
                );

                $jobHistory = $this->BP->get_schedule_history($historyFilter, $sid);
                $systemHistory = array();
                foreach ($jobHistory as $history) {

                    $data = $this->buildHistoryOutput($history, $name, $sid, null, $which, $startDate, $failed);
                    if (!empty($data)) {
                        $systemHistory[] = $data;
                    }
                }
                $allJobs = array_merge($allJobs, $systemHistory);
            }
        }
        return $allJobs;
    }

    public function buildHistoryOutput($jobData, $sysName, $sid, $clientName,$which, $historyStartDate = null, $failed = null)
    {
        $data = array();
        $hasRun = false;

        $scheduleName = isset($jobData['schedule_name']) ? $jobData['schedule_name'] : null;
        // Do this as the core is not returning duration.  Format duration as a string.
        $duration = isset($jobData['start_time']) ? $this->functions->formatTimeDelta($this->now - $jobData['start_time']) : "n/a";
        $appName = isset($jobData['application_name']) ? $jobData['application_name'] : "";
        $appType = isset($jobData['app_type']) ? $jobData['app_type'] : "";
        $scheduleDescription = isset($jobData['schedule_description']) ? $jobData['schedule_description'] : "";
        $failures = isset($jobData['failures']) ? $jobData['failures'] : "";
        $status = ((int)($failures) > 0) ? 'Error' : 'Successful';
        $historyStartDate = $this->functions->formatDateTime($historyStartDate);
        $backups = isset($jobData['backups']) ? $this->jobsFunctions->buildBackupsOutput($jobData['backups'], $failed) : "";
        if($backups){
            $hasRun = true;
        }

        if ($failed == null or ($failed == "failed" and $failures !== 0)) {
            if ($appName == "on_demand") {
                $clientName = "";
            } else {
                $clientName = isset($jobData['client_name']) ? $jobData['client_name'] : $this->jobsFunctions->getClientNameFromBackup($backups);
            }

            if ($hasRun) {
                $data = array(
                    'client_name' => $clientName,
                    'app_name' => $appName == "Archive" ? "Backup Copy" : $appName,
                    'description' => $scheduleDescription,
                    'id' => $sid . '.' . $this->itemCount++,
                    'duration' => $duration,
                    'type' => $appType == "Archive" ? "Backup Copy" : "Backup",
                    'status' => $status,
                    'failures' => $failures,
                    'name' => $scheduleName,
                    'start_date' => $this->jobsFunctions->getStartDateFromBackup($backups, $historyStartDate),
                    'system_name' => $sysName,
                    'system_id' => $sid,
                    'backups' => $backups
                );
            }
        }

        return $data;
    }

    //this needs to be moved job functions as it's used in both active and history
    function buildReturn($data, $filter){
        $skip = false;

        if ($filter != "") {
            switch ($filter) {

                // get them all from jobs list.
                case 'active':
                    break;
                // see if type matches user selection.  If not, skip it.
                default:
                    if ($data['type'] != $filter) {
                        $skip = true;
                    }
                    break;
            }
        }
        if ($skip) {
            $data = NULL;
        }

        return $data;
    }

} //end JobsHistory class


