<?php

$dir = '/api/includes/reports';

class LegalHoldReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_legal_hold_report($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();
        
        $start_time = false;
        if ( array_key_exists('start_date', $data) )
        {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if ( array_key_exists('end_date', $data) )
        {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }

        foreach ($systems as $system_id => $system_name)
        {
            $legalHoldSupported = $this->BP->legalhold_supported($system_id);
            if ($legalHoldSupported === true)
            {
                $clients = $this->BP->get_client_list($system_id);
                $relevantBackupIDs = array();
                $tempReportData = array();

                $legalhold_backups_list = $this->BP->get_legalhold_backups( array('system_id' => $system_id) );

                if ( $legalhold_backups_list !== false and count($legalhold_backups_list) > 0 )
                {
                    foreach ( $legalhold_backups_list as $legalhold_backup )
                    {
                        if ( $legalhold_backup['is_on_hold'] === true )
                        {
                            $relevantBackupIDs[] = $legalhold_backup['backup_no'];

                            $temp_legalhold_backup = array();
                            $temp_legalhold_backup['id'] = $legalhold_backup['backup_no'];
                            $temp_legalhold_backup['app_name'] = '';
                            $temp_legalhold_backup['client_name'] = (array_key_exists($legalhold_backup['client_id'], $clients) ? $clients[$legalhold_backup['client_id']] :'');
                            $temp_legalhold_backup['database_name'] = '';
                            $temp_legalhold_backup['instance_name'] = '';
                            $temp_legalhold_backup['system_name'] = $system_name;
                            $temp_legalhold_backup['type'] = '';
                            $temp_legalhold_backup['size'] = 0;
                            $temp_legalhold_backup['start_date'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $legalhold_backup['start_time']);
                            if ( $legalhold_backup['hold_days_backup'] === -1 )
                            {
                                $temp_legalhold_backup['hold_end_date'] = 'Never';
                                $temp_legalhold_backup['hold_days'] = 'Forever';
                            }
                            else
                            {
                                $temp_legalhold_backup['hold_end_date'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $legalhold_backup['hold_expire_time']);
                                $temp_legalhold_backup['hold_days'] = $legalhold_backup['hold_days_backup'];
                            }
                            $tempReportData[ $legalhold_backup['backup_no'] ] = $temp_legalhold_backup;
                        }
                    }
                }

                $backup_status_filter = array( 'system_id' => $system_id, 'backup_ids' => $relevantBackupIDs );
                if ( $start_time !== false )
                {
                    $backup_status_filter['start_time'] = $start_time;
                }
                if ( $end_time !== false )
                {
                    $backup_status_filter['end_time'] = $end_time;
                }

                $backup_status_result_array = $this->BP->get_backup_status( $backup_status_filter );

                if ( $backup_status_result_array !== false and count($backup_status_result_array) > 0 )
                {
                    foreach ( $backup_status_result_array as $backup_status )
                    {
                        $currentReportData = $tempReportData[ $backup_status['id'] ];

                        $currentReportData['app_name'] = (array_key_exists('app_name', $backup_status) ? $backup_status['app_name'] :'');
                        $currentReportData['database_name'] = (array_key_exists('database_name', $backup_status) ? $backup_status['database_name'] : '');
                        $currentReportData['instance_name'] = (array_key_exists('server_instance_name', $backup_status) ? $backup_status['server_instance_name'] : ((array_key_exists('sharepoint_guid', $backup_status) ? $backup_status['sharepoint_guid'] :'')));
                        $currentReportData['type'] = $this->functions->getBackupTypeString($backup_status['type']);
                        $currentReportData['size'] = $backup_status['size'];

                        $report_data[] = $currentReportData;
                        $report_count += 1;
                    }
                }

            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;
    }

}