<?php

class PDF
{
    private $BP;

    // If this is ever changed to be more than 10, then line 43 ($temporary_report_number = substr($new_temporary_html_filename, -6, 1);) needs to be changed to handle double digits.
    const NUMBER_OF_TEMPORARY_REPORT_FILES = 4;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }


    // Returns full path to PDF file upon success
    // Returns false upon failure or if PDF is not to be created
    function toPDF( $reportData, $reportName ) {
        $array = $this->convertArrayToHtml($reportData, true);
        $new_temporary_html_filename = '';
        $new_temporary_pdf_filename = '';
        $temporary_report_html_files = glob('/var/www/html/tempPDF/tempReportHtml*.html');
        $count_of_temporary_report_html_files = count($temporary_report_html_files);
        if ( $temporary_report_html_files !== false and $count_of_temporary_report_html_files < PDF::NUMBER_OF_TEMPORARY_REPORT_FILES )
        {
            $new_temporary_html_filename = '/var/www/html/tempPDF/tempReportHtml'.($count_of_temporary_report_html_files+1).'.html';
            $new_temporary_pdf_filename = '/var/www/html/tempPDF/tempReportPDF'.($count_of_temporary_report_html_files+1).'.pdf';
        }
        else
        {
            // Initialize the oldest time to one hour from now, so that any time is older than it.
            $oldest_time = strtotime('+1hour');
            foreach ($temporary_report_html_files as $html_filename)
            {
                $html_file_time = filemtime($html_filename);
                if ($html_file_time < $oldest_time)
                {
                    $oldest_time = $html_file_time;
                    $new_temporary_html_filename = $html_filename;
                }
            }
            $temporary_report_number = substr($new_temporary_html_filename, -6, 1);
            $new_temporary_pdf_filename = '/var/www/html/tempPDF/tempReportPDF' . $temporary_report_number . '.pdf';
        }
        file_put_contents( $new_temporary_html_filename, $array);
        $result = $this->createPDF( 1, $new_temporary_html_filename, $new_temporary_pdf_filename );
        return $result!==false ? $new_temporary_pdf_filename : $result ;
    }


    // Returns full path to PDF file upon success
    // Returns false upon failure or if PDF is not to be created
    function createPDF($systemID, $htmlFilePath, $pdfFilename) {
        // load pdf conversion class
        $rootdir = "/var/www/html/recoveryconsole";
        $pdfScript = $rootdir . '/system/pdf_conversion.php';
        // if this method is called more than one time - it will fail without require_once
        require_once $pdfScript;
        $pdfConversion = new PDFConversion();

        // create PDF file if necessary
        // needs to create a cookie
        $result = $pdfConversion->createPDF($systemID, $htmlFilePath, $pdfFilename, false, true);
        return $result;
    }


    // A heavily modified/adapted version of Aidan Lister's array2table version 1.3.2 http://aidanlister.com/2004/04/converting-arrays-to-human-readable-tables/
    function convertArrayToHtml( $PHP_data_array, $use_style_sheet = false )
    {
        if ( empty($PHP_data_array) === true or is_array($PHP_data_array) !== true )
        {
            return false;
        }

        if ( !isset($PHP_data_array[0]) or !is_array($PHP_data_array[0]) )
        {
            $PHP_data_array = array($PHP_data_array);
        }

        // Take the keys from the first row as the starting point for headings
        $headers = array_keys($PHP_data_array[0]);

        $table_rows_array = array();

        // The body
        foreach ($PHP_data_array as $row)
        {
            $nested_arrays = array();
            $row_tr_html_string = "\t<tr>";

            foreach ( $headers as $array_key )
            {
                if ( array_key_exists($array_key, $row) )
                {
                    // Cast objects to arrays
                    if ( is_object($row[$array_key]) )
                    {
                        $row[$array_key] = (array) $row[$array_key];
                    }

                    if ( is_array($row[$array_key]) )
                    {
                        // This is a nested array.  Save it away to be handled by recursion later
                        $nested_arrays[$array_key] = $row[$array_key];
                    }
                    else
                    {
                        if ( is_bool($row[$array_key]) )
                        {
                            if ( $row[$array_key] === true )
                            {
                                $row[$array_key] = 'Yes';
                            }
                            else
                            {
                                $row[$array_key] = 'No';
                            }
                        }
                        if ( strlen($row[$array_key]) > 0 )
                        {
                            $row_tr_html_string .= "<td>".( htmlspecialchars((string) $row[$array_key]) )."</td>";
                        }
                        else
                        {
                            //Add a blank <td> tag here to keep data lined up in the correct column
                            $row_tr_html_string .= "<td></td>";
                        }
                    }

                    unset($row[$array_key]);
                }
                // The $array_key isn't present, so it must be an optional column
                else
                {
                    //Add a blank <td> tag here to keep data lined up in the correct column
                    $row_tr_html_string .= "<td></td>";
                }
            }

            foreach ( $row as $new_header => $new_cell)
            {
                // The $array_key hasn't been used yet, so it must be an optional column
                // Add this optional column to all future rows in the table
                $headers[] = $new_header;
                // Add this optional column to all previously added row
                foreach ( $table_rows_array as $table_row_key => $table_row )
                {
                    //Add a blank <td> tag here to each row to ensure all rows have the same number of columns
                    $table_rows_array[$table_row_key]['row_string'] .= "<td></td>";
                }

                // Cast objects to arrays
                if ( is_object($new_cell) )
                {
                    $new_cell = (array) $new_cell;
                }

                if ( is_array($new_cell) and count($new_cell) > 0 )
                {
                    // This is a nested array.  Save it away to be handled by recursion later
                    $nested_arrays[$new_header] = $new_cell;
                }
                else
                {
                    if ( is_bool($new_cell) )
                    {
                        if ( $new_cell === true )
                        {
                            $new_cell = 'Yes';
                        }
                        else
                        {
                            $new_cell = 'No';
                        }
                    }
                    if ( strlen($new_cell) > 0 )
                    {
                        $row_tr_html_string .= "<td>".( htmlspecialchars((string) $new_cell) )."</td>";
                    }
                    else
                    {
                        //Add a blank <td> tag here to keep data lined up in the correct column
                        $row_tr_html_string .= "<td></td>";
                    }
                }
            }

            // Save away this row and its associated nested arrays until final calculations can be made
            $table_rows_array[] = array( 'row_string'=>$row_tr_html_string, 'nested_arrays'=>$nested_arrays  );
        }

        $table_rows_html_string = "";
        $do_header_row = false;
        foreach ( $table_rows_array as $table_row )
        {
            // Do a check to see if any rows exist at all or if it is just nested arrays
            $do_empty_cell = ("\t<tr>" !== $table_row['row_string']);

            // This loop ensures that we have the correct number of headers
            foreach ( $table_row['nested_arrays'] as $nested_array_header => $nested_array )
            {
                // Don't use any nested array column headings as regular column headings
                $header_key = array_search( $nested_array_header, $headers, true );
                if ( $header_key !== false )
                {
                    unset( $headers[$header_key] );
                }
            }

            $nested_table_column_span = count($headers);
            $do_header_row = ($nested_table_column_span > 0);
            if ($do_empty_cell === true)
            {
                $nested_table_column_span -= 1;
            }
            if ( $nested_table_column_span <= 0 )
            {
                $nested_table_column_span = 1;
            }

            // Put the actual row information into the string
            $table_rows_html_string .= $table_row['row_string']."</tr>\n";

            // This loop processes each
            foreach ( $table_row['nested_arrays'] as $nested_array_header => $nested_array )
            {
                $table_rows_html_string .= "\t<tr>";

                // If it is not just nested arrays, add an empty cell that acts as a buffer to view data more easily
                if ($do_empty_cell === true)
                {
                    $table_rows_html_string .= "<td rowspan='2'></td>";
                }

                $table_rows_html_string .= "<th colspan='".$nested_table_column_span."'>".$nested_array_header."</th></tr>\n\t<tr>";

                //Recursively define each $nested_array
                $table_rows_html_string .= "<td colspan='".$nested_table_column_span."'>\n".$this->convertArrayToHtml($nested_array)."\n</td></tr>\n";
            }
        }

        $table = "";
        if ( $use_style_sheet === true )
        {
            $table .= "<head>\n\t<link rel='stylesheet' type='text/css' href='pdf-style-sheet.css'>\n</head>\n";
        }
        // Start the table
        $table .= "<table>\n";

        if ( $do_header_row === true )
        {
            // The header
            $table .= "\t<tr>";

            foreach ( $headers as $heading )
            {
                $table .= '<th>' . $heading . '</th>';
            }
            $table .= "</tr>\n";
        }

        $table .= $table_rows_html_string;

        $table .= '</table>';
        return $table;
    }

}



?>