<?php

class Systems
{
    private $BP;

    public function __construct($BP, $sid)
    {
        $this->BP = $BP;
        $this->now = time();
        $this->sid = $sid;

        require_once('function.lib.php');
        $this->functions = new Functions($this->BP);

        define("PROCESSOR_INFORMATION", "Processor Information");
        define("MEMORY_INFORMATION", "Memory Usage");
        define("MAX_BACKUPS", "Maximum Backups");
    }

    public function get($which, $filter, $sid)
    {
        $data = array();
        switch($which){
            case "details":
                $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
                $systemInfo = $this->BP->get_system_info($sid);

                if ($systemInfo !== false){
                    $data['appliance'] = $this->processDetails($systemInfo, $sid);
                }
                break;
            default:
                $systems = $sid == false ? $this->functions->getSystems(true) : $this->functions->getSystems(true, $sid);
                foreach ($systems as $system) {
                    $sid = $system['id'];
                    $data["appliance"][] = $this->buildOutput($system, $sid);
                }
                break;
        }
        return $data;
    }

    function processDetails($system, $sid){
        $this->determineOSVersion($OS);
        $commandName = PROCESSOR_INFORMATION;
        $commandOutput = $this->BP->run_command($commandName, "", $sid);
        $bVirtual = $this->BP->is_virtual($sid);
        if ($commandOutput !== false) {
            $cores = $this->parseProcessorCommandOutput($commandOutput, $bVirtual, $ProcessorType, $ProcessorCores, $ProcessorCache, $ProcessorFrequency);
        }

        $commandName = MEMORY_INFORMATION;
        $commandOutput = $this->BP->run_command($commandName, "", $sid);
        if ($commandOutput !== false) {
            $memory = $this->parseMemoryCommandOutput($commandOutput, $MemorySize, $MemoryMB);
        }
        $commandName = MAX_BACKUPS;
        $maxBackups = $this->BP->run_command($commandName, "", $sid);

            $data = array(
                'name' => isset($system['name']) ? $system['name'] : "",
                'asset_tag' => $assetTag = $this->BP->get_asset_tag($sid),
                'version' => isset($system['version']) ? $system['version'] : "",
                'os_version' => $OS = preg_replace("~[\r\n]~", "",$OS),
                'processors' => $cores,
                'memory' => $memory,
                'max_backup' => $maxBackups
            );

        return $data;
    }

    function determineOSVersion(&$OS){
        ob_start();
        $result = shell_exec('cat /etc/redhat-release');
        if (!is_null($result)) {
            $OS =  $result;
        }
        ob_end_clean();
    }

    function parseProcessorCommandOutput($output, $bVirtual, &$ProcessorType, &$ProcessorCores, &$ProcessorCache, &$ProcessorFrequency) {
        $outputArray = explode("\n", $output);

        $cores = 0;
        for ($i = 0; $i < count($outputArray); $i++) {
            $line = $outputArray[$i];
            $processorArray = explode(':', $line);
            if (count($processorArray) > 1) {
                if (strstr($processorArray[0], "model name")) {
                    $ProcessorType = $processorArray[1];
                } else if (strstr($processorArray[0], "cpu cores")) {
                    $ProcessorCores = $processorArray[1];
                } else if (strstr($processorArray[0], "cache size")) {
                    $ProcessorCache = $processorArray[1];
                } else if (strstr($processorArray[0], "cpu MHz")) {
                    $frequency = (float)$processorArray[1] / 1000.0;		// convert MHz to GHz
                    $ProcessorFrequency = $frequency . " GHz";
                } else if (strstr($processorArray[0], "processor")) {
                    $cores++;
                }
            }
        }
        // Number of processors is total core count.
        $ProcessorCores = $cores;

        return $ProcessorCores;
    }

    function parseMemoryCommandOutput($output, &$MemorySize, &$MemoryMB)
    {
        $outputArray = explode("\n", $output);
        for ($i = 0; $i < count($outputArray); $i++) {
            $line = $outputArray[$i];
            $processorArray = explode(' ', $line);
            if (count($processorArray) > 2) {
                if (strstr($processorArray[0], "Mem:")) {
                    for ($j = 1; $j < count($processorArray); $j++) {
                        if ($processorArray[$j] != " " && $processorArray[$j] != "") {
                            $MemoryMB = (float)$processorArray[$j]; 			// size in MB.
                            $memory = $MemoryMB / 1000.0; 						// convert to GB
                            $MemorySize = $memory . " GB";
                            break;
                        }
                    }
                }
            }
        }
        return $MemorySize;
    }


    function buildOutput($system, $sid)
    {
        $host = "";
        $name = isset($system['name']) ? $system['name'] : false;

        $hostInfo = $this->BP->get_host_info($name, $sid);
        if ($hostInfo !== false) {
            $host = isset($hostInfo['ip']) ? $hostInfo['ip'] : false;
        }

        $local = $sid == $this->BP->get_local_system_id() ? true : false;
        $id = isset($system['id']) ? $system['id'] : false;
        $hostname = isset($system['host']) ? $system['host'] : "";  //need localhost available
        $role = isset($system['role']) ? $system['role'] : false;
        $version = isset($system['version']) ? $system['version'] : false;
        $version_status = isset($system['version_status']) ? $system['version_status'] : "Unknown";
        $online = isset($system['online']) ? $system['online'] : "n/a";
        $customer_id = isset($system['customer_id']) ? $system['customer_id'] : false;
        $customer_name = isset($system['customer_name']) ? $system['customer_name'] : false;
        $location_id = isset($system['location_id']) ? $system['location_id'] : false;
        $location_name = isset($system['location_name']) ? $system['location_name'] : false;
        $registered_assets = isset($system['registered_assets']) ? $system['registered_assets'] : 0;
        $replicating = $this->getReplicatingStatus($hostname, $role);
        $archiving = $this->getArchivingStatus($sid);
        $mb_size = isset($system['total_mb_size']) ? $system['total_mb_size'] : 0;
        $mb_free = isset($system['total_mb_free']) ? $system['total_mb_free'] : 0;

        $data = array(
            'id' => $id,
            'host' => $host,
            'name' => $name,
            'role' => $role,
            'version' => $version,
            'version_status' => $version_status,
            'online' => $online,
            'customer_id' => $customer_id,
            'customer_name' => $customer_name,
            'location_id' => $location_id,
            'location_name' => $location_name,
            'registered_assets' => $registered_assets,
            'replicating' => $replicating,
            'archiving' => $archiving,
            'local'=> $local,
            'total_mb_size' => $mb_size,
            'total_mb_free' => $mb_free
        );
        return $data;
    }

    function getArchivingStatus($sid)
    {
        $status = false;
        $archiveTest = $this->BP->get_archive_schedule_list($sid);
        if ($archiveTest !== false) {
            foreach ($archiveTest as $archiveSched) {
                $enabled = isset($archiveSched['enabled']) ? $archiveSched['enabled'] : "not found";
                if ($enabled == 1) {
                    $status = true;
                    continue;
                }
            }
        }
        return $status;
    }

    function getReplicatingStatus($host, $role)
    {
        $status = false;
        if ($host == "localhost" ) {
            $status = $this->BP->local_system_is_replicating();
        } else {
            $status = $role == "Replication Source" ? true : false;
        }
        return $status;
    }

    function update($data){

       $status = $this->BP->save_system_info($data);
        return $status;
    }
    //the POST/systems function
    function add($data){
        $data['location_id'] = 1;
        $status = array();
        if(isset($data['ip'])) {
            $result = $this->BP->get_host_info($data['host']);
            if ($result !== false) {  //host is in hosts file
                //is name or ip being changed?
                $originalIP = $result['ip'];
                $input = array(
                    'original_ip' => $originalIP,
                    'ip' => $data['ip'],
                    'name' => $data['name']
                );
                $result = $this->BP->save_host_info($input);
                    if ($result !== false){
                        //need the system id for the system we're modifying
                        $systems = $this->functions->getSystems();
                        foreach($systems as $id => $name){
                            if ($name == $data['host']){
                                $data['id'] = $id;
                                break;
                            }
                        }
                        $status = $this->BP->save_system_info($data);
                    }
                }else { //host is not in hosts file
                $input = array(
                    'name' => $data['name'],
                    'ip' => $data['ip']
                );
                $addToHosts = $this->BP->save_host_info($input);
                if ($addToHosts !== false) {
                    $status = $this->BP->save_system_info($data);
                }
            }
        }
         return $status;
    }
    //the delete function
    function delete_systems($which){

        $status = $this->BP->remove_system($which);
        return $status;
    }
} // end Systems class
?>
