<?php

//
// This function walks the schedule information returned from the load and
// creates the XML for each element.
//
function createScheduleXML($xml, $scheduleInfo)
{
	$iCalendarContent = $scheduleInfo['calendar'];
	if ($iCalendarContent === false) {
		$BP->buildResult($xml, false);
	} else {
		$xml->push("Calendar");
		$xml->cData($iCalendarContent);
		$xml->pop(); // Calendar	
	}
	
	$profile = $scheduleInfo['profile'];
	profileToXML($xml, $profile);
}

function profileToXML($xml, $profile)
{
	$xml->push('Profile');
	if (array_key_exists('description', $profile)) {
		$xml->element("Description", $profile['description']);
	}
	if (array_key_exists('target', $profile)) {
		$xml->element("Target", $profile['target']);
	}
	if (array_key_exists('target_slots', $profile)) {
		$xml->element("Slots", $profile['target_slots']);
	}
	if (array_key_exists('range_end', $profile)) {
		$xml->element("RangeEnd", $profile['range_end']);
	}
	if (array_key_exists('range_size', $profile)) {
		$xml->element("RangeSize", $profile['range_size']);
	}
	if (array_key_exists('instances', $profile)) {
		$instances = $profile['instances'];
		foreach ($instances as $instance) {
			$xml->element("Instance", $instance);
		}
		//$xml->element("Instances", $profile['instances']);
	}
	if (array_key_exists('clients', $profile)) {
		$clients = $profile['clients'];
		foreach ($clients as $client) {
			$xml->element("Client", $client);
		}
	}
	if (array_key_exists('types', $profile)) {
		$types = $profile['types'];
		foreach ($types as $type) {
			//$xml->element("Type", $type);
			$xml->element("Type", getBackupTypeString($type, true));
		}
	}
	if (array_key_exists('localdirs', $profile)) {
		$dirs = $profile['localdirs'];
		$stringDir = "";
		$sep = "";
		foreach ($dirs as $dir) {
			$stringDir .= $sep . $dir;
			$sep = ",";
		}
		if (count($dirs) > 0) {
			$xml->element("Dir", $stringDir);
		}
	}
	
	$optionList = $profile['options'];
	$xml->push('Options');
	if (array_key_exists('append', $optionList)) {
		$xml->element("Append", $optionList['append']);
	}
	if (array_key_exists('purge', $optionList)) {
		$xml->element("Purge", $optionList['purge']);
	}
	if (array_key_exists('compress', $optionList)) {
		$xml->element("Compress", $optionList['compress']);
	}
	if (array_key_exists('encrypt', $optionList)) {
		$xml->element("Encrypt", $optionList['encrypt']);
	}
	if (array_key_exists('dedup', $optionList)) {
		$xml->element("Deduplicate", $optionList['dedup']);
	}
	if (array_key_exists('fastseed', $optionList)) {
		$xml->element("FastSeed", $optionList['fastseed']);
	}
	if (array_key_exists('retention_days', $optionList)) {
		$xml->element("Days", $optionList['retention_days']);
	}
	if (array_key_exists('email_report', $optionList)) {
		$xml->element("EMail", $optionList['email_report'] == true ? "1" : "0");
	}
	$xml->pop(); // Options
	$xml->pop(); // Profile
}


function getProfileFromURL($lib) {
	
	$profile = array();
	$profile['description'] = $_GET['desc'];
	$profile['target'] = $_GET['target'];
	$profile['range_end'] = isset($_GET['end']) ? (int)$_GET['end'] : 0;
	$profile['range_size'] = isset($_GET['size']) ? (int)$_GET['size'] : 0;
	$profile['instances'] = $lib->getIDs($_GET['ids']);
	if (isset($_GET['dirs'])) {
		$profiles['localdirs'] = $lib->getStringArray($_GET['dirs']);
	}
	if (isset($_GET['types'])) {
		$profile['types'] = $lib->getStringArray($_GET['types']);
	}
	$profile['options'] = array();
	return $profile;
}
?>