<?php
// defines BP, xml, action, cookie
require 'header.php';

$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();

switch ($action) {				
	case "list":
		// list all the current jobs.	
		$jobList = $BP->get_job_list($dpuID);
		$taskerStatus = $BP->get_tasker_status($dpuID);
		if ($jobList === false) {
			$BP->buildResult($xml, false);
		} else {
			//			
			// Get client and device list for job info id to name mapping.
			//
			$clients = $BP->get_client_list($dpuID);
			$devices = $BP->get_device_list($dpuID);

			$xml->push('root');
			$xml->push('Jobs', array("Tasker" => $taskerStatus == 'on' ? "On" : "Off"));
			foreach ($jobList as $id => $name) {			
				$jobInfo = $BP->get_job_info($id, $dpuID);
				
				//
				// If a client ID is passed in, then display only the jobs that pertain to that client
				//
				if (isset($_GET['cid']) && ($jobInfo['client_id'] != (int)$_GET['cid'])) {
					;
				} else {
					displayJobInfo($BP, $xml, $jobInfo, $clients, $devices, $dpuID);
				}
			}
			$xml->pop(); // Jobs
			$xml->pop(); // root						
		}
		echo($xml->getXml());
		break;
					
	case "status":
	case "info":
		// Get Information about a specific job specified by its ID.
		if (isset($_GET['id'])) {
			$id = (int)$_GET['id'];				
			$jobInfo = $BP->get_job_info($id, $dpuID);
			if ($jobInfo === false) {
				$BP->buildResult($xml, false);
			} else {
				//			
				// Get client and device list for job info id to name mapping.
				//
				$clients = $BP->get_client_list($dpuID);
				$devices = $BP->get_device_list($dpuID);
				
				$xml->push('root');
				displayJobInfo($BP, $xml, $jobInfo, $clients, $devices, $dpuID);
				$xml->pop();
			}
		}
		echo($xml->getXml());
		break;
					
	case "cancel":
		// Cancel specific job specified by its ID.
		if (isset($_GET['id'])) {
			$id = (int)$_GET['id'];				
			$BP->buildResult($xml, $BP->cancel_job($id, $dpuID));
		}
		echo($xml->getXml());
		break;
					
	case "suspend":
		// Suspend specific job specified by its ID.
		if (isset($_GET['id'])) {
			$id = (int)$_GET['id'];				
			$BP->buildResult($xml, $BP->suspend_job($id, $dpuID));
		}
		echo($xml->getXml());
		break;		

	case "resume":
		// Resume specific job specified by its ID.
		if (isset($_GET['id'])) {
			$id = (int)$_GET['id'];				
			$BP->buildResult($xml, $BP->resume_job($id, $dpuID));
		}
		echo($xml->getXml());
		break;
					
	case "start":
		// Start Tasker	
		$BP->buildResult($xml, $BP->start_tasker($dpuID));
		echo($xml->getXml());
		break;
					
	case "stop":
		// Stop Tasker
		$BP->buildResult($xml, $BP->stop_tasker($dpuID));
		echo($xml->getXml());
		break;

}

//
// Build XML structure of detailed job information.
//
function displayJobInfo($BP, $xml, $jobInfo, $clients, $devices, $dpuID) {
	// If the job information was not retrievable, build a placeholder array so we do not get php warnings.
    if ($jobInfo == false) {
         $jobInfo = array();
    }
		
	$jobName = (array_key_exists('name', $jobInfo)) ? $jobInfo['name'] : "unknown";
	$jobID = (array_key_exists('id', $jobInfo)) ? $jobInfo['id'] : "unknown";
	
	$xml->push("Job", array("Name" => $jobName,
						    "ID" => $jobID));
	
						 
	if (array_key_exists('type', $jobInfo))
		$xml->element('Type', getBackupTypeString($jobInfo['type']));
	if (array_key_exists('client_id', $jobInfo)) {
		$clientID = $jobInfo['client_id'];
		
		if ($jobInfo['type'] == "virtual restore"){
			$vclientInfo = $BP->get_client_info($clientID, $dpuID);
			if ($vclientInfo !== false){
				$clientName = $vclientInfo['name'];
			}
		} else {
			$clientName = is_array($clients) ? $clients[$clientID] : "unknown";
		}
		
		$xml->emptyelement("Client", array("Name" => $clientName,
										   "ID" => $clientID,
										   "NetworkID" => $dpuID . '.' . $clientID));
	}
	if (array_key_exists('device_id', $jobInfo)) {
		$deviceID = $jobInfo['device_id'];
		$deviceName = is_array($devices) ? $devices[$deviceID] : "unknown";
		$xml->emptyelement("Device", array("Name" => $deviceName,
										   "ID" => $deviceID,
										   "NetworkID" => $dpuID . '.' . $deviceID));
	}
	if (array_key_exists('instance_id', $jobInfo)) {
		$xml->element('InstanceID', $jobInfo['instance_id']);
	}
	if (array_key_exists('instance_description', $jobInfo)) {
		$xml->element('Description', $jobInfo['instance_description']);
	
	/*	$newDesc = $jobInfo['instance_description'];
		$pieces = explode('"', $newDesc);
					
		if(count($pieces) > 1) {
			if($clientName == "vCenter-RRC"){
				$xml->element('InstanceName', $pieces[3]);
				
			} else if ($pieces[0] == "SQL Server instance ") {
				$xml->element('InstanceName', ($pieces[3]));
			} else {
				$xml->element('InstanceName', $pieces[1]);
			}
		}*/
	}
	
	if (array_key_exists('instance_short_name', $jobInfo)){
		$xml->element('InstanceName', $jobInfo['instance_short_name']);
	}
	if (array_key_exists('schedule_name', $jobInfo)) {
		$xml->element('ScheduleName', $jobInfo['schedule_name']);
	}
	if (array_key_exists('status', $jobInfo)) {
		$xml->element('Status', $jobInfo['status']);
	}
	if (array_key_exists('comment', $jobInfo)) {
		$xml->element('Comment', $jobInfo['comment']);
	}
	if (array_key_exists('start_time', $jobInfo)) {
		$xml->element('StartTime', $jobInfo['start_time']);
	}
	
	
	$xml->pop(); // job
}

$BP->destroyCookie();
?>