<?php

// class to wrap backup/schedule list functions.
class ListBase
{
	function __construct()
	{
	}
	
	//
	// This function returns an array of "attribute tag" => "value" pairs that
	// are elements of all CMC backup lists.
	//
	function getAttributes($list) // : Array
	{
		$attributes = array("Name" => $list['name'],
							"ID" => $list['id']);
		
		if (array_key_exists('family', $list)) {
			$attributes = array_merge($attributes, array("Family" => $list['family']));
		}
							
		if (array_key_exists('description', $list)) {
			$attributes = array_merge($attributes, array("Description" => $list['description']));
		}
		
		if (array_key_exists('type', $list)) {
			$attributes = array_merge($attributes, array("Type" => $this->getType($list['type'])));
		}
		
		if (array_key_exists('read_only', $list)) {
			$ro = $list['read_only'] == true ? "1" : "0";
			$attributes = array_merge($attributes, array("ReadOnly" => $ro));
		}
		
		return $attributes;
	}
	
	//
	// This function formats and returns the type string for the list.
	//
	function getType($stringType) // : String
	{
		switch ($stringType) {
			case "inclusion":
				$returnType = "Include";
				break;
			case "exclusion":
				$returnType = "Exclude";
				break;
			case "any":
				$returnType = "Any";
				break;
			case "backup":
				$returnType = "Backup";
				break;
			default:
				$returnType = $stringType;
				break;
		}
		return $returnType;
	}
	
	//
	// This function determines the ID of a list, either by the user explicitly passing the id
	// as a URL parameter, or by taking the name passed in and converting to its ID by searching
	// through the list retrieved by the given listFunction.
	//
	function getID($BP, $listFunction, $systemID ) // : int
	{
		$id = -1;
		
		if (isset($_GET['id'])) {
			$id = (int)$_GET['id'];
		} else {
			if (isset($_GET['name'])) {
				$name = $_GET['name'];
				$lists = $BP->$listFunction($systemID);
				foreach ($lists as $list) {
					if ($name == $list['name']) {
						$id = $list['id'];
						break;
					}
				}
			}
		}
		return $id;
	}
	
	//
	// This function determines the name of a list, either by the user explicitly passing the id
	// as a URL parameter, or by taking the name passed in and converting to its ID by searching
	// through the list retrieved by the given listFunction.
	//
	function getNameFromID($BP, $listFunction, $id, $systemID ) // : String
	{
		$name = "";
		$lists = $BP->$listFunction($systemID);
		foreach ($lists as $list) {
			if ($id == $list['id']) {
				$name = $list['name'];
				break;
			}
		}
		return $name;
	}
	
	
	//
	// This function build the XML of a list of calendars.
	//	
	function buildCalendarList($BP, $xml, $dpuID, $bShowRoot = true)
	{
		$calendarList = $BP->get_calendar_list($dpuID);
		if ($calendarList === false) {
			if ($bShowRoot) {
				$BP->buildResult($xml, false);
			} else {
				$message = $BP->getError();
				$xml->emptyElement("Error", array("Component" => 'calendars', "Message" => $message));
			}
		} else {
			if ($bShowRoot) {
				$xml->push("root");
			}
			$dedup = $BP->dedup_supported($dpuID);
			$incrSupported = ($dedup == 'block');
			foreach ($calendarList as $calendar) {
				if (validIncrementalItem($incrSupported, $calendar['name'], $calendar['read_only'])) {
					$attributes = $this->getAttributes($calendar);
					$xml->emptyelement("Calendar", $attributes);
				}
			}
			if ($bShowRoot) {
				$xml->pop(); //root
			}
		}
	}
	
	//
	// This function build the XML of a list of options.
	//
	function buildOptionsList($BP, $xml, $dpuID, $bShowRoot = true)
	{
		$optionLists = $BP->get_option_lists($dpuID);
		if ($optionLists === false) {
			if ($bShowRoot) {
				$BP->buildResult($xml, false);
			} else {
				$message = $BP->getError();
				$xml->emptyElement("Error", array("Component" => 'options', "Message" => $message));
			}
		} else {
			if ($bShowRoot) {
				$xml->push("root");
			}
			foreach ($optionLists as $optionList) {
				$attributes = $this->getAttributes($optionList);
				$xml->emptyelement("OptionList", $attributes);
			}
			if ($bShowRoot) {
				$xml->pop(); //root
			}
		}
	}
	
	//
	// This function build the XML of the selection lists.
	//	
	function buildSelectionLists($BP, $xml, $dpuID, $bShowRoot = true)
	{
		$selectionLists = $BP->get_selection_lists($dpuID);
		if ($selectionLists === false) {
			if ($bShowRoot) {
				$BP->buildResult($xml, false);
			} else {
				$message = $BP->getError();
				$xml->emptyElement("Error", array("Component" => 'selection lists', "Message" => $message));
			}
		} else {
			if ($bShowRoot) {
				$xml->push("root");
			}
			foreach ($selectionLists as $selectionList) {
				$attributes = $this->getAttributes($selectionList);
				$xml->emptyelement("SelectionList", $attributes);
			}
			if ($bShowRoot) {
				$xml->pop(); //root
			}
		}
	}
	
	
	//
	// This function retrieves the list of schedules and outputs an
	// XML representation of each schedule's information.
	//
	// The last_time field has 2 special values we need to check for:
	//
	// 0 - the schedule has never been run, and
	// 77 - the schedule has been reset.
	//
	function buildScheduleList($BP, $xml, $dpuID, $bShowRoot = true, $bLoadAll = false)
	{
		global $Log;
		
		$scheduleList = $BP->get_schedule_list($dpuID);

		if ($scheduleList === false) {
			if ($bShowRoot) {
				$BP->buildResult($xml, false);
			} else {
				$message = $BP->getError();
				$xml->emptyElement("Error", array("Component" => 'schedules', "Message" => $message));
			}
		} else {
			if ($bShowRoot) {
				$xml->push("root");
			}
			foreach ($scheduleList as $schedule) {
				$attributes = $this->getAttributes($schedule);	
				// These are for schedules only
		
				if(array_key_exists('id', $schedule)){
					$SchedID = $schedule['id'];
					$attributes = array_merge($attributes, array("SchedID" => $SchedID));
				}
				if (array_key_exists('enabled', $schedule)) {
					$enabled = $schedule['enabled'] == true ? "1" : "0";
					$attributes = array_merge($attributes, array("Enabled" => $enabled));
				}
				if (array_key_exists('run_only', $schedule)) {
					$runOnly = $schedule['run_only'] == true ? "1" : "0";
					$attributes = array_merge($attributes, array("RunOnly" => $runOnly));
				}

				$xml->push("Schedule", $attributes);
				// Add last_time, next_time info.
				$this->addLastNext($xml, $schedule);
				$this->addClients($BP, $xml, $schedule, $dpuID);
		
				// If getting calendar info, include it here.
				if ($bLoadAll && $runOnly != "1") {
					$scheduleInfo = $BP->get_schedule_info($SchedID, $dpuID);
					
					if ($scheduleInfo !== false) {
						if (isset($scheduleInfo['calendar'])) {
							$iCalendarContent = $BP->get_calendar($scheduleInfo['calendar'], $dpuID);
							if ($iCalendarContent !== false) {
								$xml->push("Calendar");
								$xml->cData($iCalendarContent);
								$xml->pop(); // Calendar	
							}
						}
					}
				}

				$xml->pop(); // schedule
			}
		
			if ($bShowRoot) {
				$xml->pop(); //root
			}
		}
	}
	
	
	//
	// This function processes the detailed messages returned when schedules are saved.
	// Possible return values are:
	//
	//
	//	Return: true - if saved successfully and list already exists
	//			scalar - if saved successfully and new item created
	//			false - if save failed
	//			array - if save failed (partially) with additional information about item.
	//
	function buildResult($xml, $returnValue)
	{
		global $Log;
		
		$xml->push("root");
		if ($returnValue === false) {
			$xml->element("Result", 0);
			$errorString = bp_error();
			$xml->element("ErrorString", $errorString);
			$Log->writeError($errorString, true);
		} else if (is_array($returnValue)) {
			$xml->element("Result", 0);
			$errorString = bp_error();
			$xml->element("ErrorString", $errorString);
			$Log->writeError($errorString, true);
			$xml->push("ResultArray");
			foreach ($returnValue as $element) {
				$xml->push("Result");
				if (array_key_exists('schedule_id', $element)) {
					$xml->element("ScheduleID", $element['schedule_id']);						
				}
				if (array_key_exists('client_id', $element)) {
					$xml->element("ClientID", $element['client_id']);						
				}
				if (array_key_exists('type', $element)) {
					$xml->element("Type", $element['type']);						
				}
				if (array_key_exists('option_list_id', $element)) {
					$xml->element("OptionID", $element['option_list_id']);
				}
				if (array_key_exists('selection_incl_id', $element)) {
					$xml->element("IncludeID", $element['selection_incl_id']);
				}
				if (array_key_exists('selection_excl_id', $element)) {
					$xml->element("ExcludeID", $element['selection_excl_id']);
				}
				if (array_key_exists('message', $element)) {
					$xml->element("Message", $element['message']);
				}
				// application schedules will return this.
				if (array_key_exists('appinst_id', $element)) {
					$xml->element("AppInstID", $element['appinst_id']);
				}
				if (array_key_exists('msg', $element)) {
					$xml->element("Message", $element['msg']);
				}
				$xml->pop(); // Result
			}
			$xml->pop(); // ResultArray
		} else if ($returnValue === true) {
			$xml->element("Result", 1);
		} else {
			$xml->element("Result", $returnValue);
		}
		$xml->pop();
	}
	
	//
	// This function processes the detailed messages returned when options, selection lists,
	// and calendars are saved.  Possible return values are:
	//
	//
	//	Return: true - if saved successfully and list already exists
	//			scalar - if saved successfully and new item created
	//			false - if save failed
	//			array - if save failed (partially) with additional information about item.
	//
	function buildListResult($xml, $returnValue)
	{
		global $Log;
		
		$xml->push("root");
		if ($returnValue === false) {
			$xml->element("Result", 0);
			$errorString = bp_error();
			$xml->element("ErrorString", $errorString);
			$Log->writeError($errorString, true);
		} else if (is_array($returnValue)) {
			$xml->element("Result", 0);
			$errorString = bp_error();
			$xml->element("ErrorString", $errorString);
			$Log->writeError($errorString, true);
			$xml->push("ResultArray");
			while ($elementArray = current($returnValue)) {
				$xml->push("Result");
				$xml->element("Message", key($returnValue));
				if (is_array($elementArray)) {
					foreach ($elementArray as $element) {
						$this->gatherErrorDetails($xml, $element);
					}
				}
				$xml->pop(); // Result
				next($returnValue);
			}
			$xml->pop(); // ResultArray
		} else if ($returnValue === true) {
			$xml->element("Result", 1);
		} else {
			$xml->element("Result", $returnValue);
		}
		$xml->pop();
	}
	
	//
	// Grabs specific schedule error information to better describe failures.
	//
	function gatherErrorDetails($xml, $element)
	{
		if (array_key_exists('schedule_id', $element)) {
			$xml->element("ScheduleID", $element['schedule_id']);						
		}
		if (array_key_exists('client_id', $element)) {
			$xml->element("ClientID", $element['client_id']);						
		}
		if (array_key_exists('type', $element)) {
			$xml->element("Type", $element['type']);						
		}
		if (array_key_exists('option_list_id', $element)) {
			$xml->element("OptionID", $element['option_list_id']);
		}
		if (array_key_exists('selection_incl_id', $element)) {
			$xml->element("IncludeID", $element['selection_incl_id']);
		}
		if (array_key_exists('selection_excl_id', $element)) {
			$xml->element("ExcludeID", $element['selection_excl_id']);
		}
		if (array_key_exists('message', $element)) {
			$xml->element("Message", $element['message']);
		}
	}

	//
	// Creates the XML information for the application schedule list.
	//
	function buildAppScheduleList($BP, $xml, $dpuID){
		global $Log;
	
		$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
		$aid = isset($_GET['aid']) ? (int)$_GET['aid'] : -1;
			
		$appScheduleList = $BP->get_app_schedule_list($cid, $aid, $dpuID);
		if ($appScheduleList !== false){
			foreach ($appScheduleList as $appschedule) {
				$attributes = $this->getAttributes($appschedule);	
			  	if (array_key_exists('client_id', $appschedule)) {
				  	$clientID = $appschedule['client_id'];
				 	 $attributes = array_merge($attributes, array("ClientID" => $clientID));
			  	}
				
				$clientInfo = $BP->get_client_info($clientID, $dpuID);
				if (array_key_exists('name', $clientInfo)){
					$clientName = $clientInfo['name'];
					$attributes = array_merge($attributes, array("ClientName" => $clientName));
				 }
			  
				if (array_key_exists('app_id', $appschedule)) {
					$appID = $appschedule['app_id'];
					$attributes = array_merge($attributes, array("AppID" => $appID));
			  	}
  
			  	if (array_key_exists('id', $appschedule)) {
					$schedID = $appschedule['id'];
			 	}	
  
 				$iCalendarContent = false;
				$appScheduleInfo = $BP->get_app_schedule_info($schedID, $dpuID);
				if ($appScheduleInfo === false) {
					$appScheduleInfo = $BP->get_rae_app_schedule_info($schedID, $dpuID);
				}
			  	if ($appScheduleInfo !== false) {
					$appSchedOptions = $appScheduleInfo['backup_options'];
					
					if(array_key_exists('appinst_ids', $appSchedOptions)) {
						$appInstIDs = $appSchedOptions['appinst_ids'];
						$items = count($appInstIDs);
						for ($i=0; $i < $items; $i++){
							$AppInstID = (int)$appInstIDs[$i];
							$attributes = array_merge($attributes, array("AppInstID" => $AppInstID));
							$AppInstName = $BP->get_appinst_name($AppInstID, $dpuID);
							if ($AppInstName !== false) {
								$attributes = array_merge($attributes, array("AppInstName" => $AppInstName)); 
								//need to check for existence of "|" if so take it apart							
								$strPipe = "|";
								if (strlen(strstr($AppInstName, $strPipe)) > 0) {
									 $InstPart1 = explode("|", $AppInstName);
									 $AppInstance = $InstPart1[0];
									 $AppItem = $InstPart1[1];
							  
						 			 //here is where multiple "app items" are listed.
									 $attributes = array_merge($attributes, array("AppInstance" => $AppInstance));
							 		 $attributes = array_merge($attributes, array("AppItem" => $AppItem));
								}
							}
						
					  		$AppNameArray = $BP->get_application_clients($AppInstID, $dpuID);
					  		if ($AppNameArray !== false){
								if (array_key_exists('app_name', $AppNameArray)) {
									$AppName = $AppNameArray['app_name'];
									$attributes = array_merge($attributes, array("AppName" => $AppName));
								}
							}
					  	}
					}
					$iCalendarContent = isset($appScheduleInfo['calendar']) ? $appScheduleInfo['calendar'] : false;
				}

				if (array_key_exists('enabled', $appschedule)) {
					$enabled = $appschedule['enabled'] == true ? "1" : "0";
					$attributes = array_merge($attributes, array("Enabled" => $enabled));
				}
			  
			  	$xml->push("AppSchedule", $attributes);
				// Add last_time, next_time info.
				$this->addLastNext($xml, $appschedule);
				
				if ($iCalendarContent != false) {
					$xml->push("Calendar");
					$xml->cData($iCalendarContent);
					$xml->pop(); // Calendar	
				}
				$xml->pop(); //appschedule
		  	}
		}
	}
	
	//
	// Creates the archive schedule information.
	//
	function buildArchiveScheduleList($BP, $xml, $dpuID) {
	
		global $Log;
		
		$arScheduleList = $BP->get_archive_schedule_list($dpuID);
		if ($arScheduleList !== false) {
			foreach ($arScheduleList as $schedule) {
			  $attributes = $this->getAttributes($schedule);
			  $attributes['type'] = "Archive";	
				if (array_key_exists('enabled', $schedule)) {
					$enabled = $schedule['enabled'] == true ? "1" : "0";
					$attributes['Enabled'] = $enabled;
				}
				if(array_key_exists('id', $schedule)) {
					$id = $schedule['id'];
				}
				$arScheduleInfo = $BP->get_archive_schedule_info($id, $dpuID);
							
				$iCalendarContent = isset($arScheduleInfo['calendar']) ? $arScheduleInfo['calendar'] : false;
					
				$xml->push("ArchiveSchedule", $attributes);
				// Add last_time, next_time info.
				$this->addLastNext($xml, $schedule);
				$this->addARClients($BP, $xml, $schedule, $dpuID);
				
				if ($iCalendarContent != false) {
					$xml->push("Calendar");
					$xml->cData($iCalendarContent);
					$xml->pop(); // Calendar	
				}
				$xml->pop(); // schedule
			}
			
		} else {
			$BP->buildResult($xml, false);
		}
	}
	
	//
	// Adds the options information for the option list.
	//
	function buildOptionsXML($xml, $optionList) {
		if (array_key_exists('include_new', $optionList)) {
			$xml->element("IncludeNew", $optionList['include_new'] == true ? "1" : "0");
		}
		if (array_key_exists('before_command', $optionList)) {
			$xml->element("BeforeCommand", $optionList['before_command']);
		}
		if (array_key_exists('after_command', $optionList)) {
			$xml->element("AfterCommand", $optionList['after_command']);
		}
		if (array_key_exists('email_report', $optionList)) {
			$xml->element("EMailReport", $optionList['email_report'] == true ? "1" : "0");
		}
		if (array_key_exists('failure_report', $optionList)) {
			$xml->element("FailureReport", $optionList['failure_report'] == true ? "1" : "0");
		}
	}
	
	//
	// Adds 'last_time', and 'next_time', to the XML schedule output.
	//
	//  If last_time is 0, it means that the schedule has never run.
	//  If next_time is 0, it means the schedule will not run again.
	//
	//  If last_time is 77, it means that the schedule has been "reset" (an lcs quirk).
	//  If next_time is 77, it means that the schedule has been "reset".
	//
	function addLastNext($xml, $schedule) {
	  if (array_key_exists('last_time', $schedule)) {
		  $lastTime = $schedule['last_time'];
		  if ($lastTime == 0) {
			  $dateTime = "Never";
		  } else if ($lastTime == 77) {
			  $dateTime = "Has Been Reset";
		  } else {
			  $date = date('m/d/Y', $lastTime);
			  $time = date('H:i:s', $lastTime);
			  $dateTime = $date . ' ' . $time;
		  }
		  $xml->element("LastTime", $dateTime);
	  }
	  if (array_key_exists('next_time', $schedule)) {
		  $nextTime = $schedule['next_time'];
		  if ($nextTime == 0) {
			  $dateTime = "Never";
		  } else if ($nextTime == 77) {
			  $dateTime = "Has Been Reset";
		  } else {
			  $date = date('m/d/Y', $nextTime);
			  $time = date('H:i:s', $nextTime);
			  $dateTime = $date . ' ' . $time;
		  }
		  $xml->element("NextTime", $dateTime);
	  }
	}
	
	function addClients($BP, $xml, $schedule, $dpuID){
			
		if(array_key_exists('id', $schedule)){
			$id = $schedule['id'];
		}
		$scheduleInfo = $BP->get_schedule_info($id, $dpuID);
						
		if($scheduleInfo != false){
			if(array_key_exists('clients', $scheduleInfo)){
				$xml->push("Clients");
				$clientList = $scheduleInfo['clients'];
				foreach ($clientList as $clients) {
					if(array_key_exists('id', $clients)){		
						$clientID = $clients['id'];
						$xml->element("ClientID", $clientID);
					}
				}
				$xml->pop(); //Clients
			}
		}
	}
	
	function addARClients($BP, $xml, $schedule, $dpuID){
			
		if(array_key_exists('id', $schedule)){
			$id = $schedule['id'];
		}
	
		$scheduleInfo = $BP->get_archive_schedule_info($id, $dpuID);
		
		if($scheduleInfo != false){
			if(array_key_exists('profile', $scheduleInfo)){
				$profile = $scheduleInfo['profile'];
			}
			if(array_key_exists('clients', $profile)){
				$xml->push("Clients");
				$clientList = $profile['clients'];
				foreach ($clientList as $clients => $clientID) {
					$xml->element("ClientID", $clientID);
				}
				$xml->pop(); //Clients
			}
		}
	}
}
?>