<?php
// defines BP, xml, action, cookie
require 'header.php';
require 'credentials.php';

//GLOBAL $Log;
// initialize snmp info.
$psa = array();
$bValid = true;
$bUpdate = false;
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();

$Credentials = new Credentials();

switch ($action) {
	case "list":
		$psaToolsArray = $BP->get_psa_tools($dpuID);
		if ( $psaToolsArray !== false ) {
			$toolCount = (count($psaToolsArray));
			buildPSAList($BP, $xml, $psaToolsArray, $dpuID, $Credentials);
		} else {
			$BP->buildResult($xml, false);
		}
		
		echo($xml->getXml());
		break;
		
	case "update":
			// if updating, specify id
			if (isset($_GET['configid'])) {
				$configid = (int)$_GET['configid'];
				$psa['id'] = $configid;
				$bUpdate = true;
			} else {
				$BP->buildResult($xml, false, "error: when updating an PSA configuration, the id must be specified.");
				$bValid = false;
			}
			// fall through to save.
	
	case "save":
			if ($bValid) {	
				// When saving psa config:
				// New config:  psa_tool_id, url, company_id, credential id
				// existing config: id required, other fields optional.
				
				$configID      = isset($_GET['configid'] )     ? (int)$_GET['configid'] : -1;
				$url           = isset($_GET['url'] )          ? $_GET['url']           : "";
				$companyID     = isset($_GET['companyid'] )    ? $_GET['companyid']     : "";
				$toolID        = isset($_GET['toolid'] )       ? (int)$_GET['toolid']   : "";
				$toolName      = isset($_GET['toolname'] )     ? $_GET['toolname']      : "";
				$isDefaultTool = $_GET['isdefaulttool'] == '1' ? true                   : false;
				
				$username             = isset($_GET['username'])      ? $_GET['username']             : "";
				$password             = isset($_POST['password'])     ? urldecode($_POST['password']) : "";
				$credentialID         = isset($_GET['credentialid'] ) ? (int)$_GET['credentialid']    : -1;
				$credentialName       = isset($_GET['credname'])      ? $_GET['credname']             : "";
				
				// we can save what we get either way here.  If the credentialID is -1 we will set the value for $credentialID to 
				// the returned value from the service.
				$credentialResultID = $Credentials->save( $BP, $dpuID, $credentialID, $username, $password, "", false, $credentialName );
				
				// if $credentialID was passed with -1 a new credential will be created and returned in $credentialResultID
				if ( $credentialID == -1 ) {
					$credentialID = $credentialResultID;
				}
				// otherwise - the credential ID should just be passed into the config.
				
				
				if ( $credentialResultID !== false ) {
					$psa = array (	'psa_tool_id' => $toolID,
									'url' => $url,
									'company_id' => $companyID,
									'credential_id' => $credentialID,
									'is_default' => $isDefaultTool);
					
					if ( $bUpdate ) {
						$psa[ 'id' ] = $configID;
					}
				}
					
				$BP->buildResult($xml, $BP->save_psa_config($psa, $dpuID));
			}
			echo($xml->getXml());			
			break;

	case "delete":
			// If deleting psa config, specify psa config id.
			if (isset($_GET['configid'])) {
				
				
				$id = (int)$_GET['configid'];
				$result = $BP->buildResult($xml, $BP->delete_psa_config($id, $dpuID));
				if ($result !== false) {
					$crid = (int)$_GET['credentialid'];
					$BP->delete_credentials($crid, $dpuID);
				}
			} else {
				$BP->buildResult($xml, false, "ID must be specified when removing PSA configuration.");
			}
			echo($xml->getXml());
			break;
			
	case "test":
			if (isset($_GET['configid'])) {
				$id = (int)$_GET['configid'];
			}
			$BP->buildResult($xml, $BP->send_test_psa_ticket($id, $dpuID));
			echo($xml->getXml());
			break;
			
	case "history":
			$psaArray = $BP->get_psa_history($dpuID);
			if ($psaArray !== false) {
				buildPSAHistory($xml, $psaArray);
			} else {
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());
			break;
}

//
// Creates the XML list of PSA tools and configurations .
//
function buildPSAList($BP, $xml, $psaToolsArray, $dpuID, $credential ){
	$psaConfigArray = $BP->get_psa_config($dpuID);
	$xml->push("root");
	
	foreach ($psaToolsArray as $tools) {
		$xml->push("PSAtools");
		if ( array_key_exists('psa_tool_id', $tools)) {
			$toolID = $tools['psa_tool_id'];
			$xml->element("ToolID", $tools['psa_tool_id']);
		}
		if ( array_key_exists('psa_tool_name', $tools)) {
			$xml->element("ToolName", $tools['psa_tool_name']);
		}
		
		//is tool configured?
		if($psaConfigArray !== false) {
			$xml->push("Config");
			foreach ( $psaConfigArray as $config ) {
				if (array_key_exists('psa_tool', $config)) {
					$psaTool = $config['psa_tool'];
					
					//if tool ids match, config exists
					if ($psaTool['psa_tool_id'] == $tools['psa_tool_id']) {
						//does the tool  id in the configuration match the tool id returned from bp_get_psa_tools?
						$xml->element("ConfigToolID", $psaTool['psa_tool_id']);
						if (isset($config['id'])) {
							$xml->element("ConfigID", $config['id']);
						}
						if ( array_key_exists('url', $config)) {
							$xml->element("url", $config['url']);
						}
						if ( array_key_exists('company_id', $config)) {
							$xml->element("CompanyID", $config['company_id']);
						}
						if ( array_key_exists('is_default', $config)) {
							$xml->element("IsDefaultTool", $config['is_default']);
						}
						
						// this credential format matches the credential format used by the credentials model.  
						if ( array_key_exists('credentials', $config) ) {
							
							$xml->push("Credentials");
							
							$configCredentials = $config['credentials'];
							
							$id = $configCredentials['credential_id'];
							
							$credential->get( $BP, $xml, $id, $dpuID );
							$xml->pop();
						}
					}
				}
			}
			$xml->pop(); //config
		}
		$xml->pop(); //PSAtools
	}
	$xml->pop(); //root

}

//
// Creates the XML list of PSA history.
//
function buildPSAHistory($xml, $psaArray)
{
	$xml->push("root");
	foreach ($psaArray as $psa) {
		$xml->push("PSA");
		
		if (array_key_exists('time_sent', $psa)) {
			$xml->element('TimeSent', $psa['time_sent']);
		}
		if (array_key_exists('ticket_id', $psa)) {
			$xml->element('TicketID', $psa['ticket_id']);
		}
		if (array_key_exists('description', $psa)) {
			$xml->element('Description', $psa['description']);
		}
		if (array_key_exists('severity', $psa)) {
			$xml->element('Severity', $psa['severity']);
		}
		if (array_key_exists('tool_name', $psa)) {
			$xml->element('ToolName', $psa['tool_name']);
		}
		$xml->pop();
	}
	$xml->pop();
}

$BP->destroyCookie();
?>