<?php
// defines BP, xml, action, cookie
require 'header.php';

// general library functions
require 'library.php';

// storagefunctions
require 'functions/storagefunctions.php';

$Storage = new StorageFunctions();

//
// Define NAS protection mount point.
//
define("NAS_PROTECT", "/mnt/nas/");


// set storage info to empty array.
$storageInfo = array();

// set dpu ID
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();

$userName = isset($_GET['user']) ? $_GET['user'] : NULL;
$password = isset($_REQUEST['password']) ? urldecode($_REQUEST['password']) : NULL;

switch ($action) {

	case "list":
		$storage = $BP->get_storage_list($dpuID);
		if ($storage !== false) {
			$deviceList = $BP->get_device_list($dpuID);
			$xml->push("root");
			foreach ($storage as $id => $name) {
				$storageInfo = $BP->get_storage_info($id, $dpuID);
				if ($storageInfo == false) {
					$storageInfo = array();
					$storageInfo['name'] = $name;
					$storageInfo['id'] = $id;
				}
				$Storage->createStorageXML($BP, $xml, $storageInfo, $deviceList, $dpuID);
			}
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
					
	case "update":
		// if updating, specify id			
		if (isset($_GET['id'])) {
			$storageInfo['id'] = (int)$_GET['id'];
		} else {
			$BP->buildResult($xml, false, "storage target update must have id specified.");
			echo($xml->getXml());
			break;
		}
		// fall through to add.
	
	case "add":
		// Get the storage definition from arguments.
		if (isset($_GET['name'])) {
			$storageInfo['name'] = $_GET['name'];
		}
		if (isset($_GET['stype'])) {
			$storageInfo['type'] = (int)$_GET['stype'];
		}
		if (isset($_GET['fs'])) {
			$storageInfo['build_filesystem'] = $_GET['fs'] == '1' ? true : false;
		}
		if (isset($_GET['usage'])) {
			$storageInfo['usage'] = $_GET['usage'];
		}
        if (isset($_GET['filesystem'])) {
            $storageInfo['build_filesystem'] = isset($_GET['filesystem']) ? (int)$_GET['filesystem'] : null;
        }
        if (isset($_GET['stateless_type'])) {
            $storageInfo['stateless_type'] = (int)$_GET['stateless_type'];
        }

		// Build the Protocol-specific properties array.
		$properties = array();
		$options = array ('hostname' => 'host',
						  'target' => 'target',
						  'wwpn' => 'wwpn',
						  'model' => 'model',
						  'vendor' => 'vendor',
						  'protocol' => 'pro',
						  'share_name' => 'share',
						  'port' => 'port',
						  'internal_name' => 'dev' );
		
		$bOptions = array( 'auth_enable' => 'use_auth' );
		$properties = buildOptions($properties, $options, $bOptions);
		$iOptions = array('lun' => 'lun');
		$properties = addIntegerOptions($properties, $iOptions);
		
		// cloudhook specific option	
		if (isset($_GET['maxSize'])) {
			$properties['max_size'] = (int)$_GET['maxSize'];
		}
		
		$properties[ "username" ] = $userName;
		
		if ( $password != NULL ) {
			$properties['password'] = $password;
		}
		$storageInfo['properties'] = $properties;

        $disableLogin = false;
        if ( $storageInfo['usage'] == 'stateless' ) {
            $disableLogin = true;
            $BP->disable_user_login($dpuID);
        }
		$BP->buildResult($xml, $BP->save_storage_info($storageInfo, $dpuID));
        if ( $disableLogin === true ) {
            $BP->reenable_user_login($dpuID);
        }
		echo($xml->getXml());
		break;
					
	case "delete":
		$storageID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
		if ($storageID != -1) {
			$BP->buildResult($xml, $BP->delete_storage($storageID, $dpuID));
		} else {
			$stringError = sprintf("delete storage:  storage ID must be provided");
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;
		
	case "disconnect":
		$storageID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
		if ($storageID != -1) {
			$BP->buildResult($xml, $BP->disconnect_storage($storageID, $dpuID));
		} else {
			$stringError = sprintf("disconnect storage:  storage ID must be provided");
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;
		
	case "connect":
		$storageID = isset($_GET['id']) ? (int)$_GET['id'] : -1;
		if ($storageID != -1) {
			$BP->buildResult($xml, $BP->connect_storage($storageID, $dpuID));
		} else {
			$stringError = sprintf("connect storage:  storage ID must be provided");
			$BP->buildResult($xml, false, $stringError);
		}
		echo($xml->getXml());
		break;
		
	case "targets":
		$host = $_GET['host'];
		$port = $_GET['port'];
		
		$targets = $BP->list_iscsi_targets($host, $port, $dpuID );
		if ($targets !== false) {
			$xml->push("root");
			foreach ($targets as $target) {
				$xml->element('Target', $target);
			}
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "fctargets":
		$targets = $BP->list_fc_targets($dpuID);
		if ($targets !== false) {
			$xml->push("root");
			foreach ($targets as $target) {
				$xml->element('Target', $target);
			}
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "luns":
		$host = $_GET['host'];
		$port = $_GET['port'];
		$target = $_GET['target'];
		$luns = $BP->list_iscsi_luns($host, $port, $target, $dpuID );
		if ($luns !== false) {
			$xml->push("root");
			foreach ($luns as $lun) {
				$xml->element('LUN', $lun);
			}
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "get_iqn":
		$result = $BP->get_iqn( $dpuID );
		if ($result !== false) {
			$result = trim($result, "\n");
			$xml->push("root");
			$xml->element( "IQN", $result );
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "fcluns":
		$target = $_GET['target'];
		$luns = $BP->list_fc_luns($target, $dpuID);
		if ($luns !== false) {
			$xml->push("root");
			foreach ($luns as $lun) {
				$xml->element('LUN', $lun);
			}
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "internal":
		$internalList = $BP->get_internal_storage_list($dpuID);
		$storageList = $BP->get_storage_list($dpuID);
		$lvmPartitions = $BP->get_lvm_disks();
		if ($internalList !== false) {
			$xml->push("root");
			foreach ($internalList as $storage) {
				// make sure the internal disk is not used for other purposes and
				// is not already part of the LVM.
				if (!$Storage->inLVM($BP, $storage, $lvmPartitions, NULL, $dpuID) &&
					!$Storage->isAllocated($BP, $storage, $storageList, $dpuID)) {
					$xml->element('Internal', $storage);
				}
			}
			$xml->pop(); //root
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "nasadd":
		$storageName = isset($_GET['name']) ? $_GET['name'] : "";
		$aliasedClientName = $storageName . NAS_POSTFIX;
		// create aliased host entry to localhost.
		$lib = new Library();
		$bValid = $lib->addHostAlias($BP, 'localhost', $aliasedClientName, $dpuID);
		if ($bValid) {
			$result = createAliasedClient($BP, $aliasedClientName, $dpuID);
			if ($result !== false) {
				/*
				Do not create automatic selection lists.
				// create selection lists.
				$result = createSelectionList($BP, $storageName, $aliasedClientName, $dpuID, "exclusion");
				if ($result !== false) {
					$result = createSelectionList($BP, $storageName, $aliasedClientName, $dpuID, "inclusion");
				}
				*/
				$BP->buildResult($xml, $result);	
			} else {
				$BP->buildResult($xml, false);
			}
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "nasdelete":
		$storageName = isset($_GET['name']) ? $_GET['name'] : "";
		$aliasedClientName = $storageName . NAS_POSTFIX;
		// delete the aliased client
		$result = deleteAliasedClient($BP, $aliasedClientName, $dpuID);
		if ($result !== false) {
			// delete aliased host entry from localhost.
			$lib = new Library();
			$bValid = $lib->deleteHostAlias($BP, 'localhost', $aliasedClientName, $dpuID);
			if ($bValid) {
				// delete selection lists.
				$result = deleteSelectionList($BP, $storageName, $aliasedClientName, $dpuID, "exclusion");
				if ($result !== false) {
					$result = deleteSelectionList($BP, $storageName, $aliasedClientName, $dpuID, "inclusion");
				}
				$BP->buildResult($xml, $result);
			} else {
				$BP->buildResult($xml, false);
			}
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "setChapCredential":
		$result = $BP->set_chap( $userName, $password, $dpuID );
		$BP->buildResult($xml, $result);
		echo($xml->getXml());
		break;
	case "getChapCredential":
		$result = $BP->get_chap( $dpuID );
		$BP->buildResult($xml, $result);
		echo($xml->getXml());
		break;

    case "useStateless":
        $result = $BP->use_stateless( $dpuID );
        if ($result !== -1) {
            $xml->push("root");
            if ( $result === 0 ) {
                $DisallowStorageExpansion = false;
                $storageInfoArray = $BP->get_storage_info(1, $dpuID); //Get the info for the internal system
                if ( $storageInfoArray !== false
                    and $storageInfoArray['usage'] === 'stateless'
                    and array_key_exists('stateless_type', $storageInfoArray )
                    and $storageInfoArray['stateless_type'] != 5 ){ // Code for Added Internal
                    $DisallowStorageExpansion = true;
                }
                $xml -> element('DisallowStorageExpansion', $DisallowStorageExpansion === true ? 1 : 0 );
            }
            $xml->element( "isStateless", $result );
            $xml->pop(); //root
        } else {
            $BP->buildResult($xml, false);
        }
        echo($xml->getXml());
        break;
}


function createAliasedClient($BP, $clientName, $dpuID) {
	$result = true;
	// See if client already present.
	$clientID = getClientID($BP, $clientName, $dpuID);
	if ($clientID == -1) {
		// it does not exist, so add it.
		$clientInfo = array();
		$clientInfo['name'] = $clientName;
		$clientInfo['is_enabled'] = true;
		$clientInfo['is_synchable'] = false;
		$clientInfo['is_encrypted'] = false;
		$clientInfo2 = $BP->get_client_info($clientID, $dpuID);
		if ( $clientInfo2['os_type_id'] == '40' && isset($clientInfo2['generic_property']) ) {
			$result = $BP->save_rae_client_info($clientInfo, $dpuID);
		} else {
			$result = $BP->save_client_info($clientInfo, $dpuID);
		}
	}
	return $result;
}

function deleteAliasedClient($BP, $clientName, $dpuID) {
	$result = true;
	// See if client already present.
	$clientID = getClientID($BP, $clientName, $dpuID);
	if ($clientID != -1) {
		// it exists, we can remove it.
		$result = $BP->delete_client($clientID, $dpuID);
	}
	return $result;
}

function createSelectionList($BP, $storageName, $aliasedClientName, $dpuID, $type) {
	$result = false;
	// See if client is present.
	$clientID = getClientID($BP, $aliasedClientName, $dpuID);
	if ($clientID != -1) {
		$clientInfo = $BP->get_client_info($clientID);
		if ($clientInfo !== false) {
			$filenames = array();

			switch($type) {
				case "exclusion":
					$rootList = $BP->get_client_files($clientInfo['id'], LINUX_VOLUME . '/', "", 0, $dpuID);
					$mntList = $BP->get_client_files($clientInfo['id'], LINUX_VOLUME . "/mnt/", "", 0, $dpuID);
					$nasList = $BP->get_client_files($clientInfo['id'], LINUX_VOLUME . NAS_PROTECT, "", 0, $dpuID);
					foreach ($rootList as $entry) {
						if ($entry['name'] != "mnt/" ) {
							$filenames[] = $entry['directory'] . $entry['name'];
						}
					}
					foreach ($mntList as $entry) {
						// Exclude everything in /mnt but /mnt/nas.
						if ($entry['name'] != "nas/") {
							$filenames[] = $entry['directory'] . $entry['name'];
						}
					}
					foreach ($nasList as $entry) {
						// Exclude everything in /mnt/nas but the storage mount point.
						if ($entry['name'] != $storageName . '/') {
							$filenames[] = $entry['directory'] . $entry['name'];
						}
					}		
					break;
					
				case "inclusion":
					// only include the storage under the protection mount point.
					$filenames[] = NAS_PROTECT . $storageName . '/';
					break;	
			}

			$selectionInfo = array();
			$selectionInfo['name'] = $storageName . NAS_POSTFIX . ' ' . $type;
			$selectionInfo['description'] = 'automatically generated ' . $type .' list for NAS storage ' . $storageName;
			$selectionInfo['family'] = "Linux";
			$selectionInfo['type'] = $type;
			$selectionInfo['client'] = $clientInfo['id'];
			$selectionInfo['filenames'] = $filenames;
			$result = $BP->save_selection_list($selectionInfo, $dpuID);
		}
	}
	
	return $result;
}


function deleteSelectionList($BP, $storageName, $aliasedClientName, $dpuID, $type) {
	$result = true;
	$bFound = false;
	// See if selection list is present.
	$selectionLists = $BP->get_selection_lists($dpuID);
	if ($selectionLists !== false) {
		foreach ($selectionLists as $selectionList) {
			if ($selectionList['name'] == $storageName . NAS_POSTFIX . ' ' . $type ) {
				$result = $BP->delete_selection_list($selectionList['id'], $dpuID);
				$bFound = true;
				break;
			}
		}
	}
	if ($bFound) {
		global $Log;
		$Log->writeError("Unable to find selection list named " . $storageName . NAS_POSTFIX . " for deletion.", false);
	}
	return $result;
}

$BP->destroyCookie();
?>
