<?php
// defines BP, xml, action, cookie
require 'header.php';

// variable to differentiate between update and add vaulting configuration.
$bUpdate = false;

switch ($action) {
	case "list":
		// check vault configuration status.			
		if (isset($_GET['id'])) {
			$systemID = (int)$_GET['id'];
			$result = $BP->is_vaulting_configured($systemID);
			if ($result === true) {
				$xml->push('root');
				$xml->element('Vaulting', 1);
				// get the securesync user for display.
				$dpuUser = $BP->get_ini_value('SecureSync', 'SyncUser', $systemID);
				if ($dpuUser === false) {
					$dpuUser = "unknown";
				}
				// see if vaulting process (securesync) is active.
				$result = $BP->is_securesync_enabled($systemID);
				$SecureSync = 0;
				if ($result !== -1) {
					if ($result === true) {
						$SecureSync = 1;
					}
				} else {
					$SecureSync = "unknown";
				}
				$xml->element('SecureSync', $SecureSync);
				$xml->element("DPUUser", $dpuUser);
				$xml->pop();
			} else if ($result === false) {
				$xml->push('root');
				$xml->element('Vaulting', 0);
				$xml->pop();
			} else {
				$BP->buildResult($xml, false);
			}
		} else {
			$BP->buildResult($xml, false, "must specify system ID to check vaulting configuration status.");
		}
		echo($xml->getXml());
		break;
		
	case "update":
		$bUpdate = true;
		// fall through to add.
						
	case "add":
		// if updating, specify id or name.				
		if (isset($_GET['id'])) {
			$systemID = (int)$_GET['id'];
			if (isset($_GET['user'])) {
				$dpuUser = $_GET['user'];
				// If updating, modify the sync user; otherwise, configure vaulting and start sync'ing.
				if ($bUpdate) {
					$result = $BP->modify_sync_user($systemID, $dpuUser);
				} else {
					// See if storage ID is passed in, otherwise use the default, 1.
					$storageID = isset($_GET['stid']) ? (int)($_GET['stid']) : 1;
					$result = $BP->configure_vaulting($systemID, $storageID, $dpuUser);
					if ($result !== false) {
						$result = $BP->enable_securesync(true, $systemID);
					}
				}
				$BP->buildResult($xml, $result);
			} else {
				// get name from system ID.
				$dpuUser = NULL;
				$systems = $BP->get_system_list();
				if ($systems !== false) {
					foreach ($systems as $system) {
						if ($systemID == $system['id']) {
							$dpuUser = $system['host'];
							break;
						}
					}
				}
				if (!is_null($dpuUser)) {
					// See if storage ID is passed in, otherwise use the default, 1.
					$storageID = isset($_GET['stid']) ? (int)($_GET['stid']) : 1;
					$result = ($bUpdate) ? $BP->modify_sync_user($systemID, $dpuUser) : $BP->configure_vaulting($systemID, $storageID, $dpuUser);
					$BP->buildResult($xml, $result);
				} else {
					$BP->buildResult($xml, false, "cannot determine DPU user name.");
				}
			}
		} else {
			$BP->buildResult($xml, false, "must specify system ID to modify DPU user.");
		}
		echo($xml->getXml());
		break;
					
	case "delete":
		global $Log;
		
		$result = removeADRProfile($BP);		
		
		if($result !== false) {
			// If deleting, provide id.	
			if (isset($_GET['id'])) {
				$id = (int)$_GET['id'];
				$result = true;
				// If we've requested to stop securesync first, do it.
				if (isset($_GET['disablesecuresync'])) {
					if ($_GET['disablesecuresync'] == 1) {
						$result = $BP->enable_securesync(false, $id);
					}
				}
				if ($result === false) {
					// attempt to remove anyway, but log an error.
					global $Log;
					$message = " Error disabling securesync before removing vaulting, continuing";
					$message = $BP->getError() . ":" . $message;
					$Log->writeError($message, true);
				}
				$BP->buildResult($xml, $BP->remove_vaulting($id));
				
				
			} else {
				$BP->buildResult($xml, false, "Improper argument format; ID must be specified when removing vaulting configuration.");
			}
		}else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "status":
		// check vault configuration status.  If an id is specified, see if its the local system, otherwise,
		// we know it's the local system.
		$bLocal = false;	
		$result = false;
		if (isset($_GET['id'])) {
			$systemID = (int)$_GET['id'];
			$localID = $BP->get_local_system_id();
			// if the id is the same as the local system, it's the local.
			if ($systemID == $localID) {
				$bLocal = true;
			} else {
				$systemInfo = $BP->get_system_info($systemID);
				// If the remote systemID a virtual DPU, handle as a local system.
				if ($systemInfo !== false && $systemInfo['host'] == 'localhost') {
					$bLocal = true;
				}
			}
			if ($bLocal) {
				$result = $BP->local_system_is_vaulting();
				if (is_string($result)) {
					$result = true;
				}
			} else {
				// it's a remote, see if vaulting.
				$result = $BP->is_vaulting_configured($systemID);
				// one more case.  The remote system may be vaulting somewhere else, so read
				// the variables directly to see if vaulting configured or not.
				if ($result === false) {
					$securesync = $BP->get_securesync_config($systemID);
					if ($securesync !== false) {
						if (array_key_exists('SyncTo', $securesync) && $securesync['SyncTo'] != "") {
							if (array_key_exists('SyncUser', $securesync) && $securesync['SyncUser'] != "") {
								$result = true;
							}
						}
					}
				}
			}
			buildVaultingXML($xml, $BP, $result);
		} else {
			// see if the local system is vaulting.  Will either return a string with the name of the vault,
			// an empty string if vaulting but we can't tell the name (both strings), or false if not vaulting
			// or -1 on error.  If not a string, pass down the result (false or -1) to build the XML.
			$result = $BP->local_system_is_vaulting();
			if (is_string($result)) {
				buildVaultingXML($xml, $BP, true);
			} else {
				buildVaultingXML($xml, $BP, $result);
			}
		}
		echo($xml->getXml());
		break;
}

//
// Builds the result of checking the vaulting status in an XML string.
//
function buildVaultingXML($xml, $BP, $result) {
	if ($result === true) {
		$xml->push('root');
		$xml->element('Vaulting', 1);
		$xml->pop();
	} else if ($result === false) {
		$xml->push('root');
		$xml->element('Vaulting', 0);
		$xml->pop();
	} else {
		$BP->buildResult($xml, false);
	}
}

function removeADRProfile($BP) {
	global $Log;
	$Log->writeVariable("in remove profile function");
	
	$systemID = (int)$_GET['id'];
	
	$result = true;
	
	$dpuUser = $BP->get_ini_value('SecureSync', 'SyncUser', $systemID);
	if($dpuUser !== false) {
		$dpuName = $BP->get_dpu_name($dpuUser);
		if($dpuName !== false) {
			$dpuInfo = $BP->get_host_info($dpuName);
			if($dpuInfo !== false){	
				$dpuIP = $dpuInfo['ip'];
		
				$profile = $BP->load_auto_dr_profile($dpuUser, $dpuIP);	
				if ($profile !== false){
					$bPresent = isset($profile['profile_present']) ? $profile['profile_present'] : false;
					if ($bPresent) {
						$profileInfo = $profile['profile_info'];
						$dpuIP = $profileInfo['target_ip'];
						$result = $BP->remove_auto_dr_profile($dpuUser, $dpuIP);
					}
				}
			}
		}
	}
	return $result;
}

$BP->destroyCookie();
?>
