<?php
// defines BP, xml, action, cookie
require 'header.php';

// general library functions
require $rootdir . 'bpl/library.php';
$lib = new Library();

define("VMWARE_API_KEY_SUPPORTS_VMWARE_RESTORE", "supports_vmware_restore");
define("VMWARE_API_KEY_SUPPORTS_WIR", "supports_wir");
define("VMWARE_API_KEY_SUPPORTS_EFI", "supports_efi");
define("VMWARE_API_KEY_SUPPORTS_2TBDRIVES", "supports_2tb_plus_drives");
define("VMWARE_FILTER_KEY_WIR", "filterwir");
define("VMWARE_FILTER_KEY_EFI", "filterefi");
define("VMWARE_FILTER_KEY_2TBDRIVES", "filterdisks");

// Maximum limit on fuse processes for VMware Instant Recovery.
// Kludge.
//define("MAX_FUSES_ALLOWED", 100);

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;
$clientID = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;

$instanceIDs = isset($_REQUEST['iids']) ? (string)$_REQUEST['iids'] : NULL;

$restoreCache = isset($_GET['r_cache']) ? $_GET['r_cache'] == "1": true;

// figure out which sysID to used based on view.
$sysID = $grandClientView ? $localID : $dpuID;
			
switch ($action) {
		  
	case "list":	// list vmware information for this application (esx server id).
					// The app ID in this case is the UUID and thus a string (don't cast to int).
			$uuid = isset($_GET['aid']) ? $_GET['aid'] : -1;
			$bCache = isset($_GET['cache']) ? true : false;
			$bIsSupported = $BP->is_appconfig_supported($dpuID);
			if($grandClientView){
				$clientInfo = $BP->get_client_info($clientID, $sysID);
				$applications = $clientInfo[ "applications" ];
				foreach ($applications as $id => $application) {
					if ( $application['type'] == "VMware" && isset( $application['servers'] ) ) {
						$serversArray = $application['servers'];
						$vmwareGrandclientInfo = $BP->get_grandclient_vm_info( $serversArray[$uuid], $clientID );
						$vmwareInfo = false;
						break;
					}
				}
			}
			else {
				$vmwareInfo = $BP->get_vm_info( $uuid, $instanceIDs, $bCache, false, $dpuID );
				$vmwareGrandclientInfo = false;
			}
			if ($vmwareInfo !== false) {
				$xml->push("root");
				foreach ($vmwareInfo as $vm) {
					$instanceID = $vm['instance_id'];
					
					$name = $vm['name'];
					$parentKey = isset( $vm['parentKey'] ) ? $vm['parentKey'] : "";
					$parentType = isset( $vm['parentType'] ) ? $vm['parentType'] : "";
					$isTemplate = isset( $vm['template'] ) && $vm['template'] == 1 ? true : false;	
					
					// In some places Identification was used - in some places InstanceID is used.  
					// putting them both in because the inconsistent handling has caused bugs.
					$xml->push('VM', array( 'InstanceID' => $instanceID,
											'Identification' => $instanceID,
											'ParentType' => $parentType,
											'ParentKey' => $parentKey,
											"Name" => $name,
											'Type' => $isTemplate ? "VMWARE_TMP" : "VMWARE_VM"  ));
											
					if (isset($vm['model'])) {
						$modelType = $vm['model'] == 'full' ? "Full" : "All";
						$xml->element('Model', $modelType);
					}
					if (isset($vm['is_running'])) {
						$xml->element('Running', $vm['is_running'] ? '1' : '0');
					}
					if (isset($vm['credentials'])) {
						$credentials = $vm['credentials'];						
						$xml->push("Credential");					
						
						$crid = isset($credentials['credential_id']) ? $credentials['credential_id'] : -1;	
						$xml->element('ID', $crid);				
						$cr_username = isset($credentials['username']) ? $credentials['username'] : "";
						$xml->element('Username', $cr_username);
						$cr_domain = isset($credentials['domain']) ? $credentials['domain'] : "";
						$xml->element('Domain', $cr_domain);
						$cr_is_default = isset($credentials['is_default']) ? $credentials['is_default'] : false;
						$xml->element('Default', $cr_is_default);
						$cr_display_name = isset($credentials['display_name']) ? $credentials['display_name'] : "";
						$xml->element('Name', $cr_display_name);						
						$xml->pop(); // Credential
					}
					
					if (isset($vm['app_aware'])) {
						$xml->element('App_aware', $vm['app_aware'] ? '1' : '0');
					}
		
					// test if dpu has priority/sync functionality, if not return -1			
					if ($bIsSupported) {
						if (isset($vm['priority'])) {
							$priority = $vm['priority'];
							$xml->element('Priority', $priority);
						}
						if (isset($vm['is_synchable'])) {
							$xml->element('Synchable', $vm['is_synchable'] ? '1' : '0');
						}
					} 
					$xml->pop(); // VM
				}
				$xml->element('Supported', $bIsSupported == true ? 1 : 0);
				$xml->pop(); // root
			} else if($vmwareGrandclientInfo !== false) { 
				$xml->push("root");
				$instanceIDsArray = explode( ",", $instanceIDs );
				
				foreach ($vmwareGrandclientInfo as $vm) {
					$instanceID = $vm['instance_id'];
					
					$name = $vm['name'];
					if ( $instanceIDs == NULL || array_search( $instanceID, $instanceIDsArray ) !== false ) {
						$xml->push('VM', array( 'InstanceID' => $instanceID,
												'Identification' => $instanceID,
												"Name" => $name));
					}
					$xml->pop(); // VM
				}
				$xml->element('Supported', $bIsSupported == true ? 1 : 0);
				$xml->pop(); // root
			
			} else {
				$BP->buildResult($xml, false);
			}
								
			echo($xml->getXml());			
			break;

    case "regexList":	// list vmware information for this application (esx server id).
        // The app ID in this case is the UUID and thus a string (don't cast to int).
        $uuid = isset($_GET['aid']) ? $_GET['aid'] : -1;
        $bCache = isset($_GET['cache']) ? true : false;
        $bIsSupported = $BP->is_appconfig_supported($dpuID);

        $regArray = isset($_GET['RegExp']) ? $_GET['RegExp'] : -1;

        // build regular expression array
        if (isset($_REQUEST['RegExp'])) {
            $regs = explode(':::', $_REQUEST['RegExp']);
            for ($n = 0; $n < count($regs); $n+=3) {
                $aReg = array();
                if (isset($regs[$n]) && intval($regs[$n]) !== 0) {
                    $aReg['container_id'] = intval($regs[$n]);
                    $aReg['action_id'] = intval($regs[$n+1]);
                    if (isset($regs[$n+2])) {
                        $aReg['filter'] = $regs[$n+2];
                    }
                }
                if (!empty($aReg)) {
                    $regList[] = $aReg;
                }
            }
        }

        $vmwareRegExpInfo = $BP->bp_generate_schedule_regex_list($uuid, $regList, $bCache, $dpuID);
        if ($vmwareRegExpInfo !== false) {
            $xml->push("root");
            foreach ($vmwareRegExpInfo as $vmr) {
                $instanceID = $vmr['instance_id'];

                $name = $vmr['name'];
                $parentKey = isset( $vmr['parentKey'] ) ? $vmr['parentKey'] : "";
                $parentType = isset( $vmr['parentType'] ) ? $vmr['parentType'] : "";
                $isTemplate = isset( $vmr['template'] ) && $vmr['template'] == 1 ? true : false;

                // In some places Identification was used - in some places InstanceID is used.
                // putting them both in because the inconsistent handling has caused bugs.
                $xml->push('VM', array( 'InstanceID' => $instanceID,
                    'Identification' => $instanceID,
                    'ParentType' => $parentType,
                    'ParentKey' => $parentKey,
                    "Name" => $name,
                    'Type' => $isTemplate ? "VMWARE_TMP" : "VMWARE_VM"  ));

                if (isset($vmr['model'])) {
                    $modelType = $vmr['model'] == 'full' ? "Full" : "All";
                    $xml->element('Model', $modelType);
                }
                if (isset($vmr['is_running'])) {
                    $xml->element('Running', $vmr['is_running'] ? '1' : '0');
                }
                if (isset($vmr['credentials'])) {
                    $credentials = $vmr['credentials'];
                    $xml->push("Credential");

                    $crid = isset($credentials['credential_id']) ? $credentials['credential_id'] : -1;
                    $xml->element('ID', $crid);
                    $cr_username = isset($credentials['username']) ? $credentials['username'] : "";
                    $xml->element('Username', $cr_username);
                    $cr_domain = isset($credentials['domain']) ? $credentials['domain'] : "";
                    $xml->element('Domain', $cr_domain);
                    $cr_is_default = isset($credentials['is_default']) ? $credentials['is_default'] : false;
                    $xml->element('Default', $cr_is_default);
                    $cr_display_name = isset($credentials['display_name']) ? $credentials['display_name'] : "";
                    $xml->element('Name', $cr_display_name);
                    $xml->pop(); // Credential
                }

                if (isset($vmr['app_aware'])) {
                    $xml->element('App_aware', $vmr['app_aware'] ? '1' : '0');
                }

                // test if dpu has priority/sync functionality, if not return -1
                if ($bIsSupported) {
                    if (isset($vmr['priority'])) {
                        $priority = $vmr['priority'];
                        $xml->element('Priority', $priority);
                    }
                    if (isset($vmr['is_synchable'])) {
                        $xml->element('Synchable', $vmr['is_synchable'] ? '1' : '0');
                    }
                }
                $xml->pop(); // VM
            }
            $xml->element('Supported', $bIsSupported == true ? 1 : 0);
            $xml->pop(); // root
        } else {
            $BP->buildResult($xml, false);
        }

        echo($xml->getXml());
        break;
			
	case "target":	// list recovery targets
			// The ID parameter is not actually used by the bp_get_vm_restore_targets API. 
			// If this changes, the callers of this service need to be revisited (VMware restore, WIR on VMware)
			$id = isset($_GET['id']) ? (int)$_GET['id'] : -1;
			
			$filter = array();
			// VMware restore filter. As of 7.4, we don't support restores to 3.x ESX servers.
			$filter[VMWARE_API_KEY_SUPPORTS_VMWARE_RESTORE] = true;
			// WIR filter. If set, only return ESX servers that support WIR
			$filter[VMWARE_API_KEY_SUPPORTS_WIR] = isset($_REQUEST[VMWARE_FILTER_KEY_WIR]) ? ($_REQUEST[VMWARE_FILTER_KEY_WIR] == '1') : false; 
			// EFI filter. If set, only return ESX servers that support EFI
			$filter[VMWARE_API_KEY_SUPPORTS_EFI] = isset($_REQUEST[VMWARE_FILTER_KEY_EFI]) ? ($_REQUEST[VMWARE_FILTER_KEY_EFI] == '1') : false; 
			// Disk filter. If set, only return ESX servers that support >2TB disks
			$filter[VMWARE_API_KEY_SUPPORTS_2TBDRIVES] = isset($_REQUEST[VMWARE_FILTER_KEY_2TBDRIVES]) 
													? ($_REQUEST[VMWARE_FILTER_KEY_2TBDRIVES] == '1') : false;

			$vmwareInfo = $BP->get_vm_restore_targets($id, $sysID);			
			if ($vmwareInfo !== false) {
				$xml->push("root");
				if (isset($vmwareInfo['warning'])) {
					$xml->element('Warning', $vmwareInfo['warning']);
				}
				$servers = $vmwareInfo['servers'];
				if (is_array($servers) && count($servers) > 0) {
					foreach ($servers as $server) {
						$uuid = $server['uuid'];
						$name = $server['name'];
						$version = $server['version'];
						
						if (isset($server['capabilities'])) {
							$filterOutServer = shouldFilterOutServer($filter, $server['capabilities']);
							//$Log->writeVariable("vmware_log: name - $name, skip - $skipThisServer");
							if ($filterOutServer == true) {
								continue;
							}
						}
						
						$isStandAlone = isset( $server[ 'standalone' ] )? $server[ 'standalone' ] : false;
						$xml->push('ESX', array("ID" => $uuid, "Name" => $name, "StandAlone" => $isStandAlone ? '1' : '0' ,
												"Version" => $version ));
						$bRunning = $server['is_running'];
						$xml->element('Running', $bRunning ? '1' : '0');
						
						$resourcePools = $BP->get_resource_pool_info( $uuid, $sysID );
						$vapps         = $BP->get_vApp_info( $uuid, $sysID );
						
						$include_vGroups = ( $resourcePools!== false || $vapps!== false  );
						
						if ( $include_vGroups ) $xml->push( "VGroups", array("Name" => $name ));
						
						if ( $resourcePools!== false && $resourcePools != NULL ) {
							foreach($resourcePools as $rp ){
								$rpKey = $rp['key'];
								$rpName = $rp['name'];
								$rpParentType = $rp['parentType'];
								$rpParentKey = $rp['parentKey'];
								$xml->push('VG', array( 'Key' => $rpKey,
														'Name' => $rpName,
														'ParentType' => $rpParentType,
														'ParentKey' => $rpParentKey,
														'Type'=>"0" ));
								$xml->pop();
							}
						}
						
						if ( $vapps!== false && $vapps != NULL ) {
							foreach($vapps as $va ){
								$vaKey = $va['key'];
								$vaName = $va['name'];
								$vaParentType = $va['parentType'];
								$vaParentKey = $va['parentKey'];
								$xml->push('VG', array( 'Key' => $vaKey,
														'Name' => $vaName,
														'ParentType' => $vaParentType,
														'ParentKey' => $vaParentKey,
														'Type'=> "1" ));
								$xml->pop();
							}
						}
						
						if ( $include_vGroups ) $xml->pop();
						
						if (isset($server['datastores'])) {
							$datastores = $server['datastores'];
							foreach ($datastores as $datastore) {
								$xml->push('Datastore', array("Name" => $datastore['name']));
								$xml->element('Size', $datastore['mb_size']);
								$xml->element('Free', $datastore['mb_free']);
								$xml->pop(); // Datastore
							}
						}
						
						if (isset($server['vms'])) {
							$vms = $server['vms'];
							foreach ($vms as $name => $running) {
								$xml->push('VM', array(	"Name" => $name));
								$xml->element('Running', $running ? '1' : '0');
								$xml->pop(); // VM
							}
						}
	
						$xml->pop(); // ESX
					}
				}
				$xml->pop();
			} else {
				$BP->buildResult($xml, false);
			}
								
			echo($xml->getXml());			
			break;
			
	case "getVMGroups":
			$xml->push("root");
			$uuid = isset($_GET['aid']) ? $_GET['aid'] : -1;
			
			$resourcePools = $BP->get_resource_pool_info( $uuid, $sysID );
			$vapps         = $BP->get_vApp_info( $uuid, $sysID );
			
			$include_vGroups = ( $resourcePools!== false || $vapps!== false  );
			
			if ( $include_vGroups ) $xml->push( "VGroups", array("Name" => $name ));
							
			if ( $resourcePools!== false && $resourcePools != NULL ) {
				foreach($resourcePools as $rp ){
					$rpKey = $rp['key'];
					$rpName = $rp['name'];
					$rpParentType = $rp['parentType'];
					$rpParentKey = $rp['parentKey'];
					$xml->push('VG', array( 'Key' => $rpKey,
											'Name' => $rpName,
											'ParentType' => $rpParentType,
											'ParentKey' => $rpParentKey,
											'Type'=>"0" ));
					$xml->pop();
				}
			}
			
			if ( $vapps!== false && $vapps != NULL ) {
				foreach($vapps as $va ){
					$vaKey = $va['key'];
					$vaName = $va['name'];
					$vaParentType = $va['parentType'];
					$vaParentKey = $va['parentKey'];
					$xml->push('VG', array( 'Key' => $vaKey,
											'Name' => $vaName,
											'ParentType' => $vaParentType,
											'ParentKey' => $vaParentKey,
											'Type'=> "1" ));
					$xml->pop();
				}
			}
			
			if ( $include_vGroups ) $xml->pop();
			$xml->pop();
			echo($xml->getXml());			
			break;
			
	case "save":	// save vmware information for this application (esx server id).
			$VMs = xmlToVMs($_REQUEST['vms']);
			$vm_info = array();
			
			
	/*		  [0]=>
			  array(4) {
				["Name"]=>
				string(12) "aries-vdpu64"
				["Identification"]=>
				int(65)
				["Priority"]=>
				int(500)
				["Synchable"]=>
				bool(false)
			  }							*/	
			foreach ($VMs as $VM) {	
				// make name lower case
				if (isset($VM['Name'])) {
					$VM['Name'] = strtolower($VM['Name']);
				}
				// cast priority and identification to int
				if (isset($VM['Priority'])) {
					$VM['Priority'] = (int) $VM['Priority'];
				} 
				if (isset($VM['Identification'])) {
					$VM['Identification'] = (int) $VM['Identification'];
				} 
				// cast syncable to a boolean
				if (isset($VM['Synchable'])) {
					$VM['Synchable'] == "true" ? $VM['Synchable'] = (bool) $VM['Synchable'] = true : (bool) $VM['Synchable'] = false;
				}
		
				$vm_info[] = $VM;
			}
			  
			$result = $BP->save_vm_info($vm_info, $dpuID);
			  if($result !== false){
				  $BP->buildResult($xml, $result);
			  } else {
				  $BP->buildResult($xml, false);
			  }
		
			echo($xml->getXml());
			break;
		
	case "start": //start vmware instant recovery  
			$esxUID         = isset($_GET['esx'] ) ? $_GET['esx']      : "";
			$vmName         = isset($_GET['name']) ? $_GET['name']     : "";
			$rKey           = isset($_GET['rkey']) ? $_GET['rkey']     : NULL;
			$datastore      = isset($_GET['dts'] ) ? $_GET['dts']      : "";
			$restoreAddress = isset($_GET['add'] ) ? $_GET['add']      : "";
			$audit          = isset($_GET['aud'] ) ? (int)$_GET['aud'] :  0;
			$backups        = isset($_GET['bks'] ) ? $_GET['bks']      : "";
			$power          = isset($_GET['power']) ? (int)$_GET['power'] : 1;

			/* Removed fuse check for 8.0. This is now done at core level
			if (overFuseLimit($backups)) {
				$BP->buildResult($xml, false,
					"Cannot start instant recovery.  You must tear down another session before starting one for this VM at this point-in-time.");
				echo($xml->getXml());
				break;
			}
			*/
			
			$vmIRStartResult = $BP->vmware_ir_start($esxUID, $vmName, $datastore, $restoreAddress, $audit, $backups, $power, $rKey, $sysID );
			
			if ($vmIRStartResult !== false) {
				$xml->push("root");
				$xml->element('Result', $vmIRStartResult);
				$xml->pop(); //root
			} else {
				$BP->buildResult($xml, false);
			}					
			echo($xml->getXml());			
			break;

	case "destroy":  //destroy vmware instant recovery job
			$aID = isset($_GET['ID']) ? $_GET['ID'] : "";
			$force = isset($_GET['force']) ? (int)$_GET['force'] : 0;
			
			$vmIRDestroyResult = $BP->vmware_ir_destroy($aID, $force, $sysID);	
			if ($vmIRDestroyResult !== false) {
				$xml->push("root");
				$xml->element('Result', $vmIRDestroyResult);							
				$xml->pop(); //root
			} else {
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());			
			break;
		
	case "status": //get vmware instant recovery status 
			$vmIRStatus = $BP->get_vm_ir_status($sysID);	 
			$vm_status_info = array();
			 
			$AppInstance = "";
			$AppItem = "";
							 
			if ($vmIRStatus !== false) {	
				$xml->push("VMIRStatus");
				foreach ($vmIRStatus as $vm_status_info) {	
					$xml->push("VM");
					if (isset($vm_status_info['id'])) {
						$xml->element('ID', $vm_status_info['id']);
						//get name of vm/esx from app inst id
						$appInstName = $BP->get_appinst_name($vm_status_info['id'], $sysID);
						//need to check for existence of "|" if so take it apart							
						$strPipe = "|";
						if (strlen(strstr($appInstName, $strPipe)) > 0) {
							 $InstPart1 = explode("|", $appInstName);
							 $AppInstance = $InstPart1[0];
							 $AppItem = $InstPart1[1];
						}
						$xml->element('ServerName', $AppInstance);
						$xml->element('VMName', $AppItem);
					} 					
					if (isset($vm_status_info['status'])) {
						$xml->element('Status', $vm_status_info['status']);
					} 					
					if (isset($vm_status_info['detail'])) {
						$xml->element('Detail', $vm_status_info['detail']);
					} 					
					if (isset($vm_status_info['percentage'])) {
						$xml->element('Percentage', $vm_status_info['percentage']);
					} 
					if (isset($vm_status_info['time'])) {
						$xml->element('Time', $vm_status_info['time']);
					} 
					if (isset($vm_status_info['audit'])) {
						$xml->element('audit', $vm_status_info['audit']);
					}
					if (isset($vm_status_info['vm_moref'])) {
						$xml->element('vm_moref', $vm_status_info['vm_moref']);
					}
					$xml->pop(); //VM  
				}
				$xml->pop(); //VM IR Status	 
			 } else {
				$BP->buildResult($xml, false);
			 }
			 echo($xml->getXml());	
			 break;
		 
		 
	case 'setcred': // set credentials for vmware		 	 	 				
			$info_array = array();
			$ids = isset($_GET['idlist']) ? $lib->getStringArray($_GET['idlist']) : -1;
			
			if ($ids == -1) {
				$BP->buildResult($xml, false, "Error: You must select a VM client first.");
				$bValid = false;
			} else {
				$credential_id = (int)$_GET['crid'];
				
				foreach ($ids as $id) {
					$info = array();
					$info['instance_id'] = $id;
					
					if ($credential_id) {
						$info['credential_id'] = $credential_id;
					}
					if (isset($_GET['app_aware'])) {
						$info['app_aware'] = $_GET['app_aware'] == '1' ? true : false;
					}
					
					$info_array[] = $info;
				}
				
				$result = $BP->save_app_credentials_info($info_array, $sysID);
				if($result !== false){
					$BP->buildResult($xml, $result);
				} else {
					$BP->buildResult($xml, false);
				}
			}
			
			echo($xml->getXml());
		 	break;
			
	case 'unsetcred': // delete credentials for vmware
			$info_array = array();
			$ids = isset($_GET['idlist']) ? $lib->getStringArray($_GET['idlist']) : -1;
			if ($ids == -1) {
				$BP->buildResult($xml, false, "Error: You must select a VM client first.");
				$bValid = false;
			}
			else {
				foreach ($ids as $id) {
					$info = array();
					$info['instance_id'] = $id;
					// when deleting credentials, no_credentials flag must be true and both credential_id and app_aware cannot be passed in 
					$info['no_credentials'] = true;
					$info_array[] = $info;
				}
					
				$result = $BP->save_app_credentials_info($info_array, $sysID);
				if($result !== false){
					$BP->buildResult($xml, $result);
				} else {
					$BP->buildResult($xml, false);
				}
			}
			
			echo($xml->getXml());
		 	break;
			
			
	case 'setappawr': // toggle app_aware flag for vmware
			$info_array = array();
			$ids = isset($_GET['idlist']) ? $lib->getStringArray($_GET['idlist']) : -1;
			
			if ($ids == -1) {
				$BP->buildResult($xml, false, "Error: You must select a VM client first.");
				$bValid = false;
			}			
			else {
				$app_aware = isset($_GET['app_aware']) && $_GET['app_aware'] == '1' ? true : false;
				
				foreach ($ids as $id) {
					$info = array();
					$info['instance_id'] = $id;
					$info['app_aware'] = $app_aware;
					$info_array[] = $info;
				}
					
				$result = $BP->save_app_credentials_info($info_array, $sysID);
				if($result !== false){
					$BP->buildResult($xml, $result);
				} else {
					$BP->buildResult($xml, false);
				}
			}
			
			echo($xml->getXml());
		 	break;
			
	case 'getdisks': // gets the list of disks assigned to each VM for exclusion
			$ids = isset($_GET['idlist']) ? $lib->getStringArray($_GET['idlist']) : -1;
			//$Log->writeVariable("Inside getDisks");
			if ($ids === -1) 
			{
				$BP->buildResult($xml, false, "Error: You must select a VM client first.");
				$bValid = false;
			} 
			else 
			{
				$xml->push("root");
				//$Log->writeVariable( $ids );
				
				$foundError = false;
				$errorString = "";
				
				foreach ( $ids as $id ) 
				{
					//$Log->writeVariable( "Calling get_vm_disks" );
					//$Log->writeVariable( "With ID $id" );
					//$Log->writeVariable( "With system ID $sysID" );
					
					$result = $BP->get_vm_disks( $id, false, $sysID );
					
					if( $result !== false ) 
					{
						$xml->push("Disks");
						foreach ( $result as $diskInfo  ) {
							$xml->push("Disk");
							
							if ( isset( $diskInfo['instance_id'] ) )
							{
								$xml->element('InstanceID', $diskInfo['instance_id'] );
							}
							
							if ( isset( $diskInfo['name'] ) )
							{
								$xml->element('Name', $diskInfo['name'] );
							}
							
							if ( isset( $diskInfo['disk_id'] ) )
							{
								$xml->element('DiskID', $diskInfo['disk_id'] );
							}
							
							if ( isset( $diskInfo['key'] ) )
							{
								$xml->element('Key', $diskInfo['key'] );
							}
							
							if ( isset( $diskInfo['mb_size'] ) )
							{
								$xml->element('MBSize', $diskInfo['mb_size'] );
							}
							
							if ( isset( $diskInfo['is_excluded'] ) )
							{
								$isExcluded = ( $diskInfo['is_excluded'] ? "1" : "0" );
								$xml->element('IsExcluded', $isExcluded );
							}
							
							if ( isset( $diskInfo['is_system'] ) )
							{
								$isSystem = ( $diskInfo['is_system'] ? "1" : "0" );
								$xml->element('IsSystem', $isSystem );
							}
							
							if ( isset( $diskInfo['rdm'] ) )
							{
								$xml->element('RDMType', (string) $diskInfo['rdm'] );
							}
							
							if ( isset( $diskInfo['disk_mode'] ) )
							{
								$xml->element('DiskMode', (string) $diskInfo['disk_mode'] );
							}
							
							$xml->pop(); // Disk Node
						}
						
						$xml->pop(); //Disks node
					} 
					else 
					{
						/*
						$xml->push("Error");
						$xml->element('InstanceID', $id );
						$xml->pop(); // error
						$xml->element('ErrorString', $BP->getError() );
						*/
						
						$errorString .= $BP->getError().",";
						
						if ( !$foundError ){
							$foundError = true;
						}
					}
				}
				if ( $foundError ) {
					$xml->element("Result", 0);
					$xml->element("ErrorString", $errorString );
					$returnValue = false;
				}
				$xml->pop(); //root node
			}
			echo($xml->getXml());
			break;
			
	case 'setdiskexc' : // sets the exlusion settings for a given disk
			
			// exclusions are parse in the form of $instanceID|$diskUUID|$excValue
			// like 145|83783748738762-393839-338|1
			// where 145 is the instance id, 83783748738762-393839-338 is the UUID, 1 is true
			
			$diskExclusions = isset($_GET['diskexc']) ? $lib->getStringArray($_GET['diskexc']) : -1;
			// array is now composed of 
			// [ 145|83783748738762-393839-338|1,  
			//   143|83744448738762-326639-333|0,  
			//   190|33554678938762-456677-221|1 ]
			// we will iterate through and break them into the data
			// for the core call
			if ( $diskExclusions == -1 ) {
				$BP->buildResult($xml, false, "Error: The arguments provided are incompatible.");
				$bValid = false;
			} else {
				$foundError = false;
				
				//$Log->writeVariable("Setting Disk exclusions");
				
				$count = count( $diskExclusions );
				//$Log->writeVariable( "The number of exclusions is $count ");
				
				$disk_info = array();
				
				foreach ( $diskExclusions as $diskExclusionString ) {
					//$Log->writeVariable( "unexploded DE:  $diskExclusionString" );
					
					list( $instanceID, $diskKey, $isExcluded ) = explode( "|", $diskExclusionString );
					//$Log->writeVariable("Setting ex value: $isExcluded for VM $instanceID - disk $diskKey" );
					
					$isExcludedBool = $isExcluded === "1" ? true : false;
					
					//$Log->writeVariable("isExcludedBool is $isExcludedBool" );
					
					$disk_exclude_Settings = array( "instance_id"  => (integer)$instanceID, 
													 "key"         => (integer)$diskKey, 
													 "is_excluded" => (bool)$isExcludedBool);
					array_push( $disk_info, $disk_exclude_Settings );
				}
				
				
				//$Log->writeVariable("Calling set_vm_disks with for system $sysID");
					
				//$Log->writeVariable($disk_info);
					
				$result = $BP->set_vm_disks( $disk_info, $sysID );
					
				if($result !== false){
					$BP->buildResult($xml, $result);
				} else {
					$BP->buildResult($xml, false);
				}
			}
			echo($xml->getXml());
			break;
			
	case "backups":
			// get list of VMs that have backups
			if ($grandClientView) {
				// for replicated backups, either filter to 1 source, specified by 'sid'; if no sid, show all.
				$showAll = !isset($_GET['sid']);
				if ($showAll) {
					// loop through all systems to get replicated VM list.
					$vms = array();
					$systems = $BP->get_system_list();
					foreach ($systems as $id => $name) {
						$sysvms = $BP->get_protected_vmware_vms(true, $id);
						for ($i = 0; $i < count($sysvms); $i++) {
							$sysvms[$i]['source_id'] = $id;
							$sysvms[$i]['source_name'] = $name;
						}
						$vms = array_merge($vms, $sysvms);
					}
				} else {
					$vms = $BP->get_protected_vmware_vms(true, $dpuID);
				}
			} else {
				// for non-replicated backups, do not use grand client view.
				$vms = $BP->get_protected_vmware_vms(false, $dpuID);
			}
			if ($vms !== false) {
				$xml->push('VMs');
				foreach ($vms as $vm) {
					$xml->push('VM');
					$xml->element('Name', $vm['vm_name']);
					$xml->element('ID', $vm['instance_id']);
					if (isset($vm['guest_os'])) {
						$xml->element('OS', $vm['guest_os']);
					}
					$xml->element('Server', $vm['server_name']);
					if (isset($vm['source_id'])) {
						$attrs = array('ID' => $vm['source_id'], 'Name' => $vm['source_name']);
						$xml->emptyElement("Source", $attrs);
					}
					$xml->pop();
				}
				$xml->pop();
			} else {
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());
			break;
		
}

/*
<VMs>
  <VM Identification="60" Name="Ashu_build_x64">
	<Priority>500</Priority>
	<Synchable>True</Synchable>
  </VM>
</VMs>
*/
function xmlToVMs($xml)
{
	$parser = xml_parser_create();
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	xml_parse_into_struct($parser, $xml, $values, $tags);
	xml_parser_free($parser);

	$returnArray = array();
	$oneVM = array();

	$stack = array();
	$i = 0;
	foreach ($values as $val) {
		if ($val['type'] == "open") {
			if ($val['tag'] != "VMs" && $val['tag'] != "VM") {
				$field = buildFieldName($val['tag']);
				array_push($stack, $field);
			}

		} else if ($val['type'] == "close") {
			if ($val['tag'] != "VMs" && $val['tag'] != "VM") {
				array_pop($stack);
			}
		} else if ($val['type'] == "complete") {
			$field = buildFieldName($val['tag']);
			array_push($stack, $field);
			setArrayValue($oneVM, $stack, $val['value'], false);
		
			array_pop($stack);		
			if ($val['tag'] == 'Synchable') {
				$returnArray[] = $oneVM;
				$oneVM = array();			
			}						
		}
	}
	return $returnArray;	
}

function setArrayValue(&$array, $stack, $value, $bBoolean) {
	if ($stack) {
		$key = array_shift($stack);
		setArrayValue($array[$key], $stack, $value, $bBoolean);
	} else {
		if ($bBoolean) {
			$array = $value == 'true' ? true : false;
		}else {
			$array = $value;
		}
	}
}

function buildFieldName($tag)
{
	$field = "";

	switch ($tag) {
		case "Name":
			$field = "Name";
			break;
		
		case "ID":
			$field = "Identification";
			break;
		
		case "Priority":
			$field = "Priority";
			break;
			
		case "Synchable":
			$field = "Synchable";
			break;
	}		
	return $field;
}

/**
* Capabilities returned as of 3/19/2014:
* supports_wir
* supports_efi
* supports_2tb_plus_drives
* supports_vmware_restore
*/
function shouldFilterOutServer ($filter, $capabilities) {
	//GLOBAL $Log;
	//test filter
    //$filter[VMWARE_API_KEY_SUPPORTS_VMWARE_RESTORE] = false;
	//$filter[VMWARE_API_KEY_SUPPORTS_WIR] = true;
	//$filter[VMWARE_API_KEY_SUPPORTS_EFI] = true;
	//$filter[VMWARE_API_KEY_SUPPORTS_2TBDRIVES] = false;
	
	$shouldFilterOutServer = false;
	
	foreach ( $filter as $filterKey=>$filterValue ) {		
		switch ( $filterKey ) {
			case VMWARE_API_KEY_SUPPORTS_VMWARE_RESTORE:
			case VMWARE_API_KEY_SUPPORTS_WIR:
			case VMWARE_API_KEY_SUPPORTS_EFI:
			case VMWARE_API_KEY_SUPPORTS_2TBDRIVES:
				// Compare filter with capabilities if filter is true/turned on
				// If server does not have given capability, set shouldFilterOutServer to true
				//$Log->writeVariable("filter $filterKey=$filterValue, capabilities=$capabilities[$filterKey]");
				if ($filterValue == true && isset($capabilities[$filterKey]) && $capabilities[$filterKey] != 1) {
					//$Log->writeVariable("shouldFilterOutServer");
					$shouldFilterOutServer = true;
				}
				break;
		}
		
		if ($shouldFilterOutServer == true) {
			break;	
		}
	}
	
	return $shouldFilterOutServer;
}


/**
 * Returns true if the new session will exceed the limit of this appliance, false otherwise.
 * takes the comma-separated list of backup IDs as a parameter, because this directly impacts
 * the total number of fuse processes that will be created.
 */
/*
function overFuseLimit($backupList) {
	$overLimit = false;
	$currentCount = 0;

	// the group size correlates to the number of fuse processes created.
	$backupArray = split(",", $backupList);
	$groupSize = count($backupArray);

	// see how many fuse processes are currently running.
	$command = sprintf("ps -ef | grep fuse | grep -v grep | wc -l");
	$handle = popen($command, 'r');
	if ($handle) {
		$currentCount = (int)fread($handle, 1024);
		pclose($handle);
	}
	$newCount = $currentCount + $groupSize;
	if ($newCount > MAX_FUSES_ALLOWED) {
		$overLimit = true;
	}
	return $overLimit;
}
*/

$BP->destroyCookie();
?>
