<?php

// Simon Willison, 16th April 2003
// Based on Lars Marius Garshol's Python XMLWriter class
// See http://www.xml.com/pub/a/2003/04/09/py-xml.html

class ria_xmlWriter {
    var $xml;
    var $indent;
    var $stack = array();
	var $lastxml;
    function ria_xmlWriter($deprecated = false, $useHeader = true, $indent = '  ') {
        $this->indent = $indent;
		if ($useHeader) {
            $this->xml = '<?xml version="1.0" encoding="utf-8"?>'."\n";
		}
    }
    function _indent() {
        for ($i = 0, $j = count($this->stack); $i < $j; $i++) {
            $this->xml .= $this->indent;
        }
    }
    function push($element, $attributes = array()) {
        $this->_indent();
        $this->xml .= '<'.$element;
        foreach ($attributes as $key => $value) {
            $this->xml .= ' '.$key.'="'.$this->xmlEntities($value).'"';
        }
        $this->xml .= ">\n";
        $this->stack[] = $element;
    }
    function element($element, $content, $attributes = array()) {
        $this->_indent();
        $this->xml .= '<'.$element;
        foreach ($attributes as $key => $value) {
            $this->xml .= ' '.$key.'="'.$this->xmlEntities($value).'"';
        }
        $this->xml .= '>'.$this->xmlEntities($content).'</'.$element.'>'."\n";
    }
    function emptyelement($element, $attributes = array()) {
        $this->_indent();
        $this->xml .= '<'.$element;
        foreach ($attributes as $key => $value) {
            $this->xml .= ' '.$key.'="'.$this->xmlEntities($value).'"';
        }
        $this->xml .= " />\n";
    }
    function content($content) {
        $this->_indent();
        $this->xml .= $content;
        $this->xml .= "\n";
    }
    function pop() {
        $element = array_pop($this->stack);
        $this->_indent();
        // lmc - if we have lastXML and are popping the root tag, add the lastXML before closing.
		if (isset($this->lastxml) && $element == 'root') {
			$this->xml .= $this->lastxml;
		}
		$this->xml .= "</$element>\n";
    }
    function getXml() {
		global $Log;
		$Log->writeXML($this->xml);
        return $this->xml;
    }
	
    function clear($deprecated = false) {
        $this->xml = '<?xml version="1.0" encoding="iso-8859-1"?>'."\n";
    }
	
	function cData($value) {
        $this->_indent();
		$this->xml .= "<![CDATA[$value]]>\n";
	}
	
    function lastTag($element, $innerTag, $attributes = array()) {
        $this->lastxml .= "<$element>\n";
        $this->lastxml .= "<$innerTag>\n";
        foreach ($attributes as $key => $value) {
            $this->lastxml .= '<'.$key.'>';
			$this->lastxml .= $value;
			$this->lastxml .= "\n";
			$this->lastxml .= "</$key>\n";
        }
		$this->lastxml .= "</$innerTag>\n";
		$this->lastxml .= "</$element>\n";
	}
    
	
	// Translate a string into a valid form for XML parser consumption.
	function xmlEntities($string, $quote_style=ENT_QUOTES) {
		static $trans;
		/*
		if (!isset($trans)) {
			$trans = get_html_translation_table(HTML_ENTITIES, $quote_style);
			foreach ($trans as $key => $value) {
				$trans[$key] = '&#'.ord($key).';';
			}
			// dont translate the '&' in case it is part of &xxx;
			$trans["&"] = '&';
			// Add % to the translation table.
			$trans["%"] = '&#37;';
		}
		// after the initial translation, _do_ map standalone '&' into '&#38;'
		return preg_replace("/&(?![A-Za-z]{0,4}\w{2,3};|#[0-9]{2,3};)/","&#38;" , strtr($string, $trans));
		*/
		if (!isset($trans)) {
			$trans = array();
			$trans['&'] = '&amp;';
			$trans['"'] = '&quot;';
			$trans["'"] = '&apos;';
			$trans['<'] = '&lt;';
			$trans['>'] = '&gt;';
			$trans['%'] = '&#37;';
		}
		return strtr($string, $trans);
	}
}
?>